using System;
using System.Configuration;
using System.Data;
using System.IO;
using System.Web.Mvc;
using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using SYSCore.aControl;
using SYSCore.Models;
using static SYSCore.aControl.EmailSender;

namespace SYSCore.Controllers
{
    public class BatchController : Controller
    {
        private DBManager odb = new DBManager();
        private String gSQL = "";
        private String mDBName = ConfigurationManager.AppSettings["SYSDBName"];
        private DBOracle oOracledb = new DBOracle();

        private DataService dataService = new DataService();


        public ActionResult ReceivePartInform()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_RECEIVE_INFORM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN_E", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    ReceivePartInform receivePart = new ReceivePartInform(ds.Tables[0], ds.Tables[1]);

                    if(ds.Tables[0].Rows.Count > 0 || ds.Tables[1].Rows.Count > 0)
                    {
                        string ReceivePartInformHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("ReceivePartInform");
                        configure.content = ReceivePartInformHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                    //return View("~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "ReceivePartInform");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }


        public ActionResult ReceiveTempPart()
        {
            try
            {
                DataSet dsHeader = new DataSet();
                DataSet ds2 = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR015_GET_DATA_BY_PIC_EMAIL";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    dsHeader = oOracledb.SqlQueryDS_CMD(cmd);

                    for (int i = 0; i < dsHeader.Tables[0].Rows.Count; i++)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR015_GET_DATA_BY_PART_EMAIL";
                        cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = dsHeader.Tables[0].Rows[i][0].ToString();
                        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                        ds2 = oOracledb.SqlQueryDS_CMD(cmd);

                        ModelReceiveTempPart receivePart = new ModelReceiveTempPart(ds2.Tables[0]);

                        if (ds2.Tables[0].Rows.Count > 0)
                        {
                            string ReceivePartInformHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/ReceiveTempPart.cshtml", receivePart);

                            //Get email configuration
                            EmailConfigure configure = dataService.GetEmailConfigure("ReceiveTempPart");
                            configure.content = ReceivePartInformHTML;
                            configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));
                            configure.to = new string[] { dsHeader.Tables[0].Rows[i][1].ToString() };
                           // configure.cc = new string[] {  };
                            //Send email with attachments
                            EmailSender sender = new EmailSender();
                            sender.Email(configure);
                        }
                    }                                  

                    return Content("Done");

                    //return View("~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "ReceivePartInform");
                    return Content(ex.Message);
                }

               
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }


        private string RenderViewToString(ControllerContext context, string viewPath, object model = null, bool partial = false)
        {
            // first find the ViewEngine for this view
            ViewEngineResult viewEngineResult = null;
            if (partial)
                viewEngineResult = ViewEngines.Engines.FindPartialView(context, viewPath);
            else
                viewEngineResult = ViewEngines.Engines.FindView(context, viewPath, null);

            if (viewEngineResult == null)
                throw new FileNotFoundException("View cannot be found.");

            // get the view and attach the model to view data
            var view = viewEngineResult.View;
            context.Controller.ViewData.Model = model;

            string result = null;

            using (var sw = new StringWriter())
            {
                var ctx = new ViewContext(context, view, context.Controller.ViewData, context.Controller.TempData, sw);
                view.Render(ctx, sw);
                result = sw.ToString();
            }

            return result;
        }
        public ActionResult PickSuspendEmail()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_PICKING_SUSPEND";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    PickSuspendEmail pickSuspend = new PickSuspendEmail(ds.Tables[0]);

                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        string PickSuspendEmailHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/PickSuspendEmail.cshtml", pickSuspend);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("PickSuspendEmail");
                        configure.content = PickSuspendEmailHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "PickSuspendEmail");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }

        public ActionResult TransactionMinusEmail()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_TRANSACTION_MINUS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    TransactionMinusEmail TransactionMinus = new TransactionMinusEmail(ds.Tables[0]);

                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        string TransactionMinusEmailHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/TransactionMinusEmail.cshtml", TransactionMinus);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("TransactionMinus");
                        configure.content = TransactionMinusEmailHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "TransactionMinusEmail");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }
        public ActionResult CheckDuplicateInvoiceEmail()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_DUP_INVOICE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    string content = string.Empty;

                    CheckDuplicateInvoiceEmail CheckDuplicateInvoice = new CheckDuplicateInvoiceEmail(ds.Tables[0]);

                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        content = "Done";
                        string CheckDuplicateInvoiceEmailHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/CheckDuplicateInvoiceEmail.cshtml", CheckDuplicateInvoice);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("CheckDuplicateInvoice");
                        configure.content = CheckDuplicateInvoiceEmailHTML;
                        configure.subject = String.Concat(configure.subject, " on ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }
                    else
                    {
                        content = "Done with no row duplicate";
                    }

                    return Content(content);

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckDuplicateInvoiceEmail");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }

        public ActionResult CheckAvgCostChangeEmail()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_AVG_COST_CHANGE";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    string content = string.Empty;

                    CheckAvgCostChangeEmail CheckAvgCostChange = new CheckAvgCostChangeEmail(ds.Tables[0]);

                    if (ds.Tables[0].Rows.Count > 0)
                    {
                        content = "Done";
                        string CheckAvgCostChangeEmailHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/CheckAvgCostChangeEmail.cshtml", CheckAvgCostChange);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("CheckAvgCostChange");
                        configure.content = CheckAvgCostChangeEmailHTML;
                        configure.subject = String.Concat(configure.subject, " on ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }
                    else
                    {
                        content = "Done with no row changed";
                    }

                    return Content(content);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckAvgCostChangeEmail");
                    return Content(ex.Message);
                }
                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }
        public ActionResult CheckCSIDiscount()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_CHECK_CSI_DISCOUNT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_5", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_6", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_7", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_8", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_9", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_10", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
    
                    CheckCSIDiscount receivePart = new CheckCSIDiscount(ds.Tables[0], ds.Tables[1], ds.Tables[2], ds.Tables[3], ds.Tables[4], ds.Tables[5], ds.Tables[6], ds.Tables[7], ds.Tables[8], ds.Tables[9]);

                    if (ds.Tables[0].Rows.Count > 0 || ds.Tables[1].Rows.Count > 0 || ds.Tables[2].Rows.Count > 0 || ds.Tables[3].Rows.Count > 0 || ds.Tables[4].Rows.Count > 0 || ds.Tables[5].Rows.Count > 0 || ds.Tables[6].Rows.Count > 0 || ds.Tables[7].Rows.Count > 0 || ds.Tables[8].Rows.Count > 0 || ds.Tables[9].Rows.Count > 0)
                    {
                        string CheckCSIDiscountHTML = RenderViewToString(ControllerContext, "~/Views/EmailTemplate/CheckCSIDiscount.cshtml", receivePart);

                        //Get email configuration
                        EmailConfigure configure = dataService.GetEmailConfigure("CheckCSIDiscount");
                        configure.content = CheckCSIDiscountHTML;
                        configure.subject = String.Concat(configure.subject, " - ", DateTime.Now.ToString("dd-MMM-yyyy HH:mm:ss"));

                        //Send email with attachments
                        EmailSender sender = new EmailSender();
                        sender.Email(configure);
                    }

                    return Content("Done");

                    //return View("~/Views/EmailTemplate/ReceivePartInform.cshtml", receivePart);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "ReceivePartInform");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }
        public ActionResult Run_Batch_Calulate_Vendor_Score_Monthly()
        {
            try
            {
                DataSet ds = new DataSet();

                try
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNEN065_CALCULATE_VENDOR_SCORE_MONTHLY_RUN_BATCH";

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    return Content("Done");

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "Run_Batch_Calulate_Vendor_Score_Monthly");
                    return Content(ex.Message);
                }

                return Content("Something wrong.");
            }
            catch (Exception ex)
            {
                return Content(ex.Message);
            }
        }
    }
}
