﻿using Oracle.ManagedDataAccess.Client;
using SYS_Core.aBiz;
using SYS_Core.aControl;
using SYSCore.Attributes;
using SYSCore.Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/hh")]
    public class HandheldController : ApiController
    {
        public HandheldController() { }
        private DBOracle oOracledb = new DBOracle();
        private String gSQL = "";
        private DataModel DataModel = new DataModel();

        [HttpGet, Route("GetPartFromLocation/{location}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPartFromLocation(string location)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_LO_PARTS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPartFromLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetLocationFromPart/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetLocationFromPart(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_PARTS_LO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLocationFromPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetInvoiceDetail")]
        [HandheldAuthorize]
        public IHttpActionResult GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("GetDetailByItem_R")]
        [HandheldAuthorize]
        public IHttpActionResult GetDetailByItem_R([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_SCAN_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDetailByItem_R");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetDetailByItem_S")]
        [HandheldAuthorize]
        public IHttpActionResult SNPE015_S_GetDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_SCAN_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetPreShelveList")]
        [HandheldAuthorize]
        public IHttpActionResult GetPreShelveList([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_PRE_SHELVE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPreShelveList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("ScanPreShelve")]
        [HandheldAuthorize]
        public IHttpActionResult ScanPreShelve([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_PRE_SHELVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);          
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ScanPreShelve");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("DeletePreShelve")]
        [HandheldAuthorize]
        public IHttpActionResult DeletePreShelve([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_PRE_SHELVE_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "DeletePreShelve");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("GetInvoicePart")]
        [HandheldAuthorize]
        public IHttpActionResult GetInvoicePart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_INV_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetDetailByItem");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReceived")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveReceived([FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_R_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                oOracledb.SqlQueryReturnValue(cmd);

                if (partList.Count > 0)
                {

                    //Run insert detail
                    foreach (var part in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPE015_R_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                        cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                        cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                        cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                        cmd.Parameters.Add("P_RECEIVE_FLAG", OracleDbType.Varchar2).Value = part["RECEIVE_FLAG"];
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_R_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_MANUAL_RECEIVE", OracleDbType.Varchar2).Value = pData["MANUAL_RECEIVE"] ?? "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                    return Json(new { Result = "Done", DocumentNo = documentNo });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: items are not equally." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: status code." });
                }
                else
                {
                    throw (ex);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceived");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SaveShelved/{location}")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveShelved([FromBody] dynamic pData, string location)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> invoiceList = new List<dynamic>(pData.INVOICE_LIST);
                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                //Run insert detail
                foreach (var part in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_S_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = part["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = part["INVOICE_NO"];
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                    cmd.Parameters.Add("P_SHELVE_FLAG", OracleDbType.Varchar2).Value = part["SHELVE_FLAG"];
                    cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = part["LOCATION"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }


                //Run insert invoice
                foreach (var invoice in invoiceList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_S_SAVE_INV_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = invoice["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoice["INVOICE_NO"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_S_SAVE_HANDHELD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new { Result = "Done" });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {

                    return Json(new { Result = "Fail", Message = oOracledb.GetErrorMessage(ex) });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveShelved");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SetRelocationStatus/{location}")]
        [HandheldAuthorize]
        public IHttpActionResult SetRelocationStatus(string location, [FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_HH_SET_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pData.status;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();

            }
            catch (OracleException ex)
            {

                throw (ex);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SetRelocationStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveRelocation")]
        [HandheldAuthorize]
        [RunBatchGuard]
        public IHttpActionResult SaveRelocation([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_HH_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OLD_LOCATION_NO", OracleDbType.Varchar2).Value = pData.OldLocation;
                cmd.Parameters.Add("P_NEW_LOCATION_NO", OracleDbType.Varchar2).Value = pData.NewLocation;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.RelocationQTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();

            }
            catch (OracleException ex)
            {

                throw (ex);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SetRelocationStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("UpdateToken")]
        [HandheldAuthorize]
        public IHttpActionResult UpdateToken([FromBody] dynamic pToken)
        {

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE015_P_GET_REC_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
            cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            DataTable dtHeader = ds.Tables[0];
            DataTable dtDetail = ds.Tables[1];

            string gGUID = pToken["GUID"];
            SYS_Core.aBiz.ReceiveTokenService receiveToken = new SYS_Core.aBiz.ReceiveTokenService();

            string tokenH = receiveToken.GenerateKeyHeader(gGUID, dtHeader);

            //Update Header
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_H";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
            cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenH;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

            oOracledb.SqlQueryReturnValue(cmd);

            //Update Detail
            foreach (DataRow dataDetail in dtDetail.Rows)
            {
                string tokenD = receiveToken.GenerateKeyDetail(gGUID, dtHeader, dataDetail);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pToken["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pToken["VENDOR_CODE"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = dataDetail["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = dataDetail["PART_NO"];
                cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenD;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }

            return Ok("Done");
        }

        [HttpGet, Route("AvailableTasks")]
        public IHttpActionResult GetAvailableTasks()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_AVAIL_TASKS";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableTasks");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("AvaliableTasksExportPack/{userRole}")]
        [HandheldAuthorize]
        public IHttpActionResult GetAvaliableTasksExportPack(string userRole)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_AVAIL_TASKS_EXPORT_PACK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ROLE", OracleDbType.Varchar2).Value = userRole;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvaliableTasksExportPack");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetStockInfo/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetStockInfo(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_STOCK_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetStockInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCountStockRound")]
        [HandheldAuthorize]
        public IHttpActionResult GetCountStockRound()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_HH_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCountStockRound");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCountStockInfo/{partNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCountStockInfo(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_GET_STOCK_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCountStockInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SaveCountStock")]
        [HandheldAuthorize]
        public IHttpActionResult SaveCountStock([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_COUNT_STOCK_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = pData.roundId;
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData.locationNo;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.partNo;
                cmd.Parameters.Add("P_COUNT_QTY", OracleDbType.Decimal).Value = pData.QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveCountStock");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        [HttpPost, Route("SaveCountStockNoPart")]
        [HandheldAuthorize]
        public IHttpActionResult SaveCountStockNoPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_COUNT_STOCK_NO_PART_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = pData.roundId;
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData.locationNo;
                cmd.Parameters.Add("P_COUNT_QTY", OracleDbType.Decimal).Value = pData.QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveCountStockNoPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetCycleCountDetail/{headerId}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCycleCountDetail(string headerId)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_HH_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(headerId);
                cmd.Parameters.Add("C_RETURN_PART_LIST", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_COUNT_LIST", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(dataSet));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCycleCountDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("GetLocation/{locationNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetLocation(string locationNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_GET_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = locationNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("CheckLocationMove")]
        [HandheldAuthorize]
        public IHttpActionResult CheckLocationMove([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_LOCATION_MOVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData["LOCATION_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckLocationMove");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("CheckLocationInto")]
        [HandheldAuthorize]
        public IHttpActionResult CheckLocationIntoNewLocation([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_LOCATION_INTO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData["LOCATION_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                return Json(DTFM.convertToList(dataTable));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckLocationIntoNewLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("ConfirmInitialReLocation")]
        [HandheldAuthorize]
        public IHttpActionResult ConfirmInitialReLocation([FromBody] dynamic pData)
        {
            try
            {
                string relocationMode = pData["mode"].ToString();

                if (relocationMode == "Relocation")
                {
                    dynamic pLocation = pData["location"];
                    InitialRelocation(pLocation);
                }
                else if (relocationMode == "SwapLocation")
                {
                    dynamic pLocationNew = pData["locationNew"];
                    dynamic pLocationOld = pData["locationOld"];
                    SwapLocation(pLocationOld, pLocationNew);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ConfirmInitialReLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        private void InitialRelocation(dynamic pLocation)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pLocation["LOCATION_NO_OLD"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pLocation["LOCATION_NO_NEW"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pLocation["PART_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = "HANDHELD";
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet dataSet = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dataTable = dataSet.Tables[0];

                if (dataTable.Rows[0]["STATUS"].ToString() != "DONE")
                {
                    throw new Exception(dataTable.Rows[0]["MESSAGE"].ToString());
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "InitialRelocation");
                throw (ex);
            }
        }

        private void SwapLocation(dynamic pLocationOld, dynamic pLocationNew)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_SWAP_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pLocationOld["LOCATION_NO"];
                cmd.Parameters.Add("P_PART_NO_OLD", OracleDbType.Varchar2).Value = pLocationOld["PART_NO"];
                cmd.Parameters.Add("P_LOCATION_NO_NEW", OracleDbType.Varchar2).Value = pLocationNew["LOCATION_NO"];
                cmd.Parameters.Add("P_PART_NO_NEW", OracleDbType.Varchar2).Value = pLocationNew["PART_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = "HANDHELD";
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";

                oOracledb.SqlExecute_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SwapLocation");
                throw (ex);
            }
        }

        #region Picking
        [HttpGet, Route("GetPickListByUser")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickListByUser()
        {
            List<PickGroup> pickGroups = new List<PickGroup>();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_GET_BYUSER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { Result = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { Result = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPickNo/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickNo(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_PICK_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { status = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("RemovePickNo/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult RemovePickNo(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_HH_REMOVE_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RemovePickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveProgressPicking")]
        [HandheldAuthorize]
        public IHttpActionResult SaveProgressPicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                List<dynamic> pickGroups = new List<dynamic>(pData.PICK_GROUPS);

                if (pickGroups.Count > 0)
                {
                    foreach (var group in pickGroups)
                    {

                        var pickHeader = group.pickHeader;
                        List<dynamic> pickDetail = new List<dynamic>(group.pickDetail);

                        foreach (var item in pickDetail)
                        {
                            if (item.QTY != null && Convert.ToInt16(item.QTY.ToString()) > 0)
                            {
                                cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNWR006_SAVE";
                                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickHeader.PICKING_LIST_NO.ToString();
                                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PICK_LOT_DATE.ToString());
                                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Int16).Value = Convert.ToInt16(item.PICK_LOT_SEQ.ToString());
                                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = item.INSTRUCTION_NO.ToString();
                                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO.ToString();
                                cmd.Parameters.Add("P_QTY", OracleDbType.Int16).Value = Convert.ToInt16(item.QTY.ToString());
                                cmd.Parameters.Add("P_IS_UNLOCK", OracleDbType.Varchar2).Value = item.IS_UNLOCK;
                                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                                oOracledb.SqlQueryReturnValue(cmd);
                            }
                        }
                    }

                    return Json(new { Result = "Done" });
                }

                return Json(new { Result = "Fail", Message = "NO_ROW_TO_PROCESS" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveProgressPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReportProblemPart")]
        [HandheldAuthorize]
        public IHttpActionResult SaveReportProblemPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_CREATE_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pData.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pData.SUB_ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.MISSING_QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Fail", Message = "DUPLICATE_INSTRUCTION" });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReportProblemPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region Pack
        [HttpGet, Route("GetAvailableCase/{caseGroup}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetAvailableCase(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("R_PRINT_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String printDate = cmd.Parameters["R_PRINT_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, PrintDate = printDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Fail", Message = "UNAVAILABLE_CASE_NO" });
                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Done", Message = "UNCASE_AVAILABLE" });

                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPackInstruction/{pickListNo}/{instructionNo}/{allowExists}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPackInstruction(string pickListNo, string instructionNo, string allowExists)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_HH_PACK_INST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_ALLOW_EXISTS", OracleDbType.Varchar2).Value = allowExists;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                DataTable packedData = ds.Tables[1];
                DataTable cardStatus = ds.Tables[2];

                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData), PackedData = DTFM.convertToList(packedData), CardStatus = DTFM.convertToList(cardStatus) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPackInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetCaseDetail/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCaseDetail(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];

                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPackInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SavePackedData")]
        [HandheldAuthorize]
        public IHttpActionResult SavePackedData([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            int deleteCount = 0;
            string caseNo = "";
            try
            {
                string customerCode = pDataList.customerCode;
                string branchCode = pDataList.branchCode;
                List<dynamic> caseList = new List<dynamic>(pDataList.caseList);
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);


                foreach (var data in caseList)
                {
                    caseNo = data.caseNo;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_DELETE_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);

                    deleteCount++;
                }

                foreach (var data in packingList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePackedData");

                if (deleteCount > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_REVERT_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.branchCode;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpGet, Route("ClearCase/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult ClearCase(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_CLEAR_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region Delivery
        [HttpGet, Route("GetDelivery/{invoiceNo}/{caseNo?}")]
        [HandheldAuthorize]
        public IHttpActionResult GetDelivery(string invoiceNo, string caseNo = null)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_DELIVERY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceNo;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveUnsentDelivery")]
        [HandheldAuthorize]
        public IHttpActionResult SaveUnsentDelivery([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_UNSENT_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveUnsentDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveSentDelivery")]
        [HandheldAuthorize]
        public IHttpActionResult SaveSentDelivery([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_HH_SENT_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveSentDelivery");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion



        #region Picking Export
        [HttpGet, Route("GetPickListExportByUser")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickListExportByUser()
        {
            List<PickGroup> pickGroups = new List<PickGroup>();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_HH_GET_BYUSER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { Result = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { Result = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickListExportByUser");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPickNoExport/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPickNoExport(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_HH_PICK_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_P", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { status = "Done", Data = DTFM.convertToList(ds) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPickNoExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("RemovePickNoExport/{pickListNo}/{instructionNo}")]
        [HandheldAuthorize]
        public IHttpActionResult RemovePickNoExport(string pickListNo, string instructionNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_HH_REMOVE_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }

                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RemovePickNoExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveProgressPickingExport")]
        [HandheldAuthorize]
        public IHttpActionResult SaveProgressPickingExport([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                List<dynamic> pickGroups = new List<dynamic>(pData.PICK_GROUPS);

                if (pickGroups.Count > 0)
                {
                    foreach (var group in pickGroups)
                    {

                        var pickHeader = group.pickHeader;
                        List<dynamic> pickDetail = new List<dynamic>(group.pickDetail);

                        foreach (var item in pickDetail)
                        {
                            if (item.QTY != null && Convert.ToInt16(item.QTY.ToString()) > 0)
                            {
                                cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNWR026_SAVE";
                                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickHeader.PICKING_LIST_NO.ToString();
                                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PICK_LOT_DATE.ToString());
                                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Int16).Value = Convert.ToInt16(item.PICK_LOT_SEQ.ToString());
                                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = item.INSTRUCTION_NO.ToString();
                                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO.ToString();
                                cmd.Parameters.Add("P_QTY", OracleDbType.Int16).Value = Convert.ToInt16(item.QTY.ToString());
                                cmd.Parameters.Add("P_IS_UNLOCK", OracleDbType.Varchar2).Value = item.IS_UNLOCK;
                                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                                oOracledb.SqlQueryReturnValue(cmd);
                            }
                        }
                    }

                    return Json(new { Result = "Done" });
                }

                return Json(new { Result = "Fail", Message = "NO_ROW_TO_PROCESS" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveProgressPickingExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReportProblemPartExport")]
        [HandheldAuthorize]
        public IHttpActionResult SaveReportProblemPartExport([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_CREATE_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pData.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pData.SUB_ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.MISSING_QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Fail", Message = "DUPLICATE_INSTRUCTION" });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReportProblemPartExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion



        #region Receive Export 

        [HttpPost, Route("CheckReceiveBabyCard")]
        [HandheldAuthorize]
        public IHttpActionResult CheckReceiveBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_CHECK_RECEIVE_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                string status = ds.Tables[0].Rows[0]["STATUS"].ToString();
                string qrcode = ds.Tables[0].Rows[0]["PICKING_QR_CODE"].ToString();
                return Json(new { data = DTFM.convertToList(ds), status = status, qrcode = qrcode });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckReceiveBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("GetDataReceiveExport")]
        [HandheldAuthorize]
        public IHttpActionResult GetDataReceiveExport([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pParam.INSTRUCTION_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dtMessage = ds.Tables[ds.Tables.Count - 1]; //Last Table in dataset (always message table)
                string message = dtMessage.Rows[0]["MESSAGE"].ToString();
                string processFlag = dtMessage.Rows[0]["PROCESS_FLAG"].ToString();
                string processMessage = dtMessage.Rows[0]["PROCESS_MESSAGE"].ToString();
                string partno = dtMessage.Rows[0]["PART_NO"].ToString();
                return Json(new { data = DTFM.convertToList(ds), message = message, processflag = processFlag, processmessage = processMessage, partno = partno });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDataReceiveExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetFreeBabyCard/{babyNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetFreeBabyCard(string babyNo)
        {
            DataSet ds = new DataSet();
            babyNo = ismoUtil.escapeCharacter(babyNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_GET_AVAILABLE_BABY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABY_NO", OracleDbType.Varchar2).Value = babyNo;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_BABYCARD_NO", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String staTus = cmd.Parameters["R_STATUS"].Value.ToString();
                String baByNo = cmd.Parameters["R_BABYCARD_NO"].Value.ToString();
                return Json(new { Result = staTus, BABYCARD_NO = baByNo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetFreeBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveReceiveExport")]
        [HandheldAuthorize]
        public IHttpActionResult SaveReceiveExport([FromBody] dynamic pData)
        {
            try
            {
                var scannedCard = pData.SCANNED_CARD;
                List<dynamic> receiveList = new List<dynamic>(pData.RECEIVE_LIST);
                DataSet ds = new DataSet();
                OracleCommand cmd;
                int i = 0;
                foreach (var data in receiveList)
                {
                    i = i + 1;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR033_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = data.PICKING_LIST_NO;
                    cmd.Parameters.Add("P_RECEIVE_QTY", OracleDbType.Varchar2).Value = data.RECEIVE_QTY;
                    cmd.Parameters.Add("P_BABY_QTY", OracleDbType.Varchar2).Value = data.BABY_QTY;
                    cmd.Parameters.Add("P_BABY_LEVEL", OracleDbType.Varchar2).Value = data.BABY_LEVEL;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = i == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Varchar2).Value = data.SEQ;
                    cmd.Parameters.Add("P_TOGGLE_INPUT", OracleDbType.Varchar2).Value = data.TOGGLE_INPUT;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                if (scannedCard?.USER_SETTING_LEVEL == "Y")
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR033_SAVE_SETTING_BABYLEVEL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = scannedCard.PICKING_LIST_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = scannedCard.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = scannedCard.PART_NO;
                    cmd.Parameters.Add("P_TYPE_A", OracleDbType.Varchar2).Value = scannedCard.TYPE_A_TEMP;
                    cmd.Parameters.Add("P_TYPE_B", OracleDbType.Varchar2).Value = scannedCard.TYPE_B_TEMP;
                    cmd.Parameters.Add("P_TYPE_C", OracleDbType.Varchar2).Value = scannedCard.TYPE_C_TEMP;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_RECEIVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string status = cmd.Parameters["R_STATUS"].Value.ToString();


                return Ok(new { status = "Done" });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveReceiveExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("CalculateBabyLevelExport")]
        [HandheldAuthorize]
        public IHttpActionResult CalculateBabyLevelExport([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd;
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_CALCULATE_BABYLEVEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pParam.INSTRUCTION_NO;
                cmd.Parameters.Add("P_TYPE_A", OracleDbType.Varchar2).Value = pParam.TYPE_A_TEMP;
                cmd.Parameters.Add("P_TYPE_B", OracleDbType.Varchar2).Value = pParam.TYPE_B_TEMP;
                cmd.Parameters.Add("P_TYPE_C", OracleDbType.Varchar2).Value = pParam.TYPE_C_TEMP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { data = DTFM.convertToList(ds)});
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CalculateBabyLevelExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion

        #region Pack Export
        [HttpPost, Route("CheckPackBabyCard")]
        [HandheldAuthorize]
        public IHttpActionResult CheckPackBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CHECK_PACK_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckPackBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("CheckPackInnerCase")]
        [HandheldAuthorize]
        public IHttpActionResult CheckPackInnerCase([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CHECK_PACK_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INNERCASE_NO", OracleDbType.Varchar2).Value = pParam.INNERCASE_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckPackInnerCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetAvailableCaseExport/{customerCode}/{branchCode}/{orderType}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetAvailableCaseExport(string customerCode, string branchCode, string orderType, string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dtCase = ds.Tables[0];

                return Json(new { Result = "Done", Data = DTFM.convertToList(dtCase) });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Invoice", Message = "Case already pre-invoice." });
                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Exist", Message = "Case is exist, do you want to unpack." });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { Result = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                else if (ex.Number == 20000)
                {
                    return Json(new { Result = "Notfound", Message = "Case is not found, Please generate case first." });
                }
                else
                {
                    throw (ex);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetCaseDetailExport/{customerCode}/{branchCode}/{orderType}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCaseDetailExport(string customerCode, string branchCode, string orderType, string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCaseDetailExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetReceiveData/{baby_no}")]
        [HandheldAuthorize]
        public IHttpActionResult GetReceiveData(string baby_no)
        {
            DataSet ds = new DataSet();
            baby_no = ismoUtil.escapeCharacter(baby_no);
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR027_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = baby_no;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetReceiveData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SavePackedDataExport")]
        [HandheldAuthorize]
        public IHttpActionResult SavePackedDataExport([FromBody] dynamic pDataList)
        {
            OracleCommand cmd;
            int deleteCount = 0;
            try
            {
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_DELETE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//Date for delete current case no (protect old case no)
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                deleteCount++;
                int i = 0;
                foreach (var data in packingList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR027_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE); //Date for new case (But not used ToDateTime Function)
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_INNER_CASE_NO", OracleDbType.Varchar2).Value = data.INNER_CASE_NO;
                    cmd.Parameters.Add("P_INSERT_TYPE", OracleDbType.Varchar2).Value = "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_SAVE_PACKING_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);


                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePackedData");

                if (deleteCount > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR027_REVERT_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("ClearCaseExport")]
        [HandheldAuthorize]
        public IHttpActionResult ClearCaseExport([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CLEAR_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pParam.scannedCase.caseDate);//for not delete old case no
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("ClearCaseExportTemp")]
        [HandheldAuthorize]
        public IHttpActionResult ClearCaseExportTemp([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CLEAR_CASE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam["ORDER_TYPE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("P_DELETE_BY_USER", OracleDbType.Varchar2).Value = pParam["DELETE_BY_USER"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearCaseExportTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetCaseTypeExportAll")]
        [HandheldAuthorize]
        public IHttpActionResult GetCaseTypeExportAll()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_CASE_TYPE_ALL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCaseTypeExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetCaseTypeExport/{type}")]
        [HandheldAuthorize]
        public IHttpActionResult GetCaseTypeExport(string type)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_CASE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = type;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCaseTypeExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetInnerPackDataExport/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetInnerPackDataExport(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_AVAILABLE_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInnerPackDataExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        // [HttpGet, Route("SNWR027_GetBabyCardInfo/{caseNo}")]

        [HttpPost, Route("SNWR027_GetBabyCardInfo")]
        [HandheldAuthorize]
        public IHttpActionResult SNWR027_GetBabyCardInfo([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_BABY_CARD_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABY_CARD_NO", OracleDbType.Varchar2).Value = param.BABY_CARD_NO;       
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetBabyCardInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region Pack Inner Export
        [HttpPost, Route("CheckInnerPackBabyCard")]
        [HandheldAuthorize]
        public IHttpActionResult CheckInnerPackBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CHECK_INNERPACK_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckInnerPackBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetAvailableInnerCaseExport/{caseGroup}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetAvailableInnerCaseExport(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_AVAILABLE_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_CASE_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                String caseDate = cmd.Parameters["R_CASE_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, CaseDate = caseDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Invoice", Message = "Case already pre-invoice." });

                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Exist", Message = "Case is exist, do you want to unpack." });

                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { Result = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER

                }
                else if (ex.Number == 20000)
                {
                    return Json(new { Result = "Notfound", Message = "Case is not found, Please generate case first." });
                }
                else
                {
                    throw (ex);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetAvailableInnerCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetInnerCaseTypeExportAll")]
        [HandheldAuthorize]
        public IHttpActionResult GetInnerCaseTypeExportAll()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_INNERCASE_TYPE_ALL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInnerCaseTypeExportAll");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetInnerCaseTypeExport/{type}")]
        [HandheldAuthorize]
        public IHttpActionResult GetInnerCaseTypeExport(string type)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_INNERCASE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = type;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInnerCaseTypeExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("GetBabyCardData/{babyNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetBabyCardData(string babyNo)
        {
            DataSet ds = new DataSet();
            babyNo = ismoUtil.escapeCharacter(babyNo);
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR039_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = babyNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetBabyCardData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SavePackedInnerDataExport")]
        [HandheldAuthorize]
        public IHttpActionResult SavePackedInnerDataExport([FromBody] dynamic pDataList)
        {
            OracleCommand cmd;
            int deleteCount = 0;
            try
            {
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_DELETE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//Date for delete current case no (protect old case no)
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                deleteCount++;

                int i = 0;
                foreach (var data in packingList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR039_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE); //Date for new case (But not used ToDateTime Function)(in C# time system is not correct)
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_INSERT_TYPE", OracleDbType.Varchar2).Value = "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_SAVE_PACKING_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePackedInnerDataExport");
                if (deleteCount > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR039_REVERT_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        [HttpGet, Route("GetInnerCaseDetailExport/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetInnerCaseDetailExport(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetInnerCaseDetailExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("ClearInnerCaseExport")]
        [HandheldAuthorize]
        public IHttpActionResult ClearInnerCaseExport([FromBody] dynamic pDataList)
        {
            try
            {
                string status = string.Empty;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CLEAR_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//for not delete old case no
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                status = cmd.Parameters["P_STATUS"].Value.ToString();
                if (status == "USED")
                {
                    return Ok(new { status = "Used" });
                }
                else
                {
                    return Ok(new { status = "Done" });
                }
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearInnerCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("ClearInnerCaseExporTemp")]
        [HandheldAuthorize]
        public IHttpActionResult ClearInnerCaseExportTemp([FromBody] dynamic pDataList)
        {
            try
            {
                string status = string.Empty;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CLEAR_CASE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.CASE_NO;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearInnerCaseExportTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("DeleteInnerBabyCardExport")]
        [HandheldAuthorize]
        public IHttpActionResult DeleteInnerBabyCardExport([FromBody] dynamic pDataList)
        {
            try
            {
                string status = string.Empty;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_DELETE_CARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.CASE_NO;
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pDataList.BABYCARD_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                status = cmd.Parameters["P_STATUS"].Value.ToString();
                if (status == "USED")
                {
                    return Ok(new { status = "Used" });
                }
                else
                {
                    return Ok(new { status = "Done" });
                }
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "ClearInnerCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region Input Case Size
        [HttpGet, Route("GetPackingCase/{customerCode}/{branchCode}/{orderType}/{caseNo}")]
        [HandheldAuthorize]
        public IHttpActionResult GetPackingCase(string customerCode, string branchCode, string orderType, string caseNo)
        {
            DataSet ds = new DataSet();
            //caseNo = ismoUtil.escapeCharacter(caseNo);
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_HH_GET_PACKING_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPackingCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SaveCaseSize")]
        [HandheldAuthorize]
        public IHttpActionResult SaveCaseSize([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            int deleteCount = 0;
            try
            {
                foreach (var data in pDataList)
                {
                    deleteCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_HH_SAVE_TEMP_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = data.CASE_WIDTH;
                    cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = data.CASE_DEPTH;
                    cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = data.CASE_HEIGHT;
                    cmd.Parameters.Add("P_CASE_WEIGHT", OracleDbType.Varchar2).Value = data.CASE_WEIGHT;
                    cmd.Parameters.Add("P_GROSS_WEIGHT", OracleDbType.Varchar2).Value = data.GROSS_WEIGHT;
                    cmd.Parameters.Add("P_GROSS_MM", OracleDbType.Varchar2).Value = data.GROSS_MM;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = deleteCount == 1 ? "Y" : "N";
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_HH_SAVE_CASE_SIZE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);


                return Ok("Done");
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveCaseSize");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion
        #region VAN
        [HttpGet, Route("GetVan/{van}")]
        [HandheldAuthorize]
        public IHttpActionResult GetVan(string van)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_HH_GET_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = van;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "OK", TableData = DTFM.convertToList(ds) });
            }
            catch (OracleException orx)
            {
                if (orx.Number == 20001)
                {
                    return Json(new { status = "COMPLETE" });
                }
                else if (orx.Number == 20002)
                {
                    return Json(new { status = "NOT_VAN" });
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", orx.Message, "GetVan OracleException");
                    return Json(new { status = "Error" });
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetVan");
                return Json(new { status = "Error" });
            }
        }


        [HttpPost, Route("CheckVanCase")]
        [HandheldAuthorize]
        public IHttpActionResult CheckVanCase([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_HH_CHECK_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                //cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                //cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                //cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                //cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                //cmd.Parameters.Add("R_CASE_NAME", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                //cmd.Parameters.Add("R_CASE_TYPE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                //cmd = oOracledb.SqlQueryReturnValue(cmd);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                
                //string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                //string _CASE_NAME = cmd.Parameters["R_CASE_NAME"].Value.ToString();
                //string _CASE_TYPE = cmd.Parameters["R_CASE_TYPE"].Value.ToString();
                return Json(new { STATUS_MESSAGE = "Done", TableData = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckVanCase");
                return Json(new { STATUS_MESSAGE = "ERROR" });
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("RemoveVanCase")]
        [HandheldAuthorize]
        public IHttpActionResult RemoveVanCase([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_HH_REMOVE_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new { STATUS_MESSAGE = "Done" });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RemoveVanCase");
                return Json(new { STATUS_MESSAGE = "ERROR" });
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("SaveVan")]
        [HandheldAuthorize]
        public IHttpActionResult SaveVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                int itemCount = 0;
                foreach (var data in pData.caseList)
                {
                    itemCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_HH_SAVE_TEMP_VAN_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = pData.vanNo;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = itemCount == 1 ? "Y" : "N";
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_HH_SAVE_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.vanNo;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "VAN HH";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveVan HH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpGet, Route("DeleteVan/{van}")]
        [HandheldAuthorize]
        public IHttpActionResult DeleteVan(string van)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR035_DELETE_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = van;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "DeleteVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion VAN
    }
}
