﻿using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;
using System.IO;
using System.Text;
using SYS_Core.aBiz;
using RandomSolutions;
using System.Net.Http.Headers;
using System.Security.Claims;
using System.Collections;
using Newtonsoft.Json;
using SYSCore.Attributes;
using SYSCore.ReportingService;
using SYSCore.aControl;

namespace SYS_Core.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/WH")]
    public class WarehouseApiController : ApiController
    {

        private DBOracle oOracledb = new DBOracle();
        private String gSQL = "";
        private DataModel DataModel = new DataModel();

        #region WareHouse Monitering

        [HttpGet, Route("SNWR020_SetTask")]
        public IHttpActionResult SNWR020_SetTask()
        {

            //string context = "0 0/1 0,8,20 ? * MON,TUE,WED,THU,FRI *";
            string context = "0 0/5 * * * ?";
            _ = JobSchedule.CreateJob("Monitoring1", "PMS", context);
            return Ok("Done");
        }


        [HttpGet, Route("SNWR020_DeleteTask")]
        public IHttpActionResult SNWR020_TEST()
        {
            _ = JobSchedule.DeleteJob("Monitoring1", "PMS");
            return Ok("Done");
        }


        [HttpGet, Route("SNWR025_SetTask")]
        public IHttpActionResult SNWR025_SetTask()
        {

            //string context = "0 0/1 0,8,20 ? * MON,TUE,WED,THU,FRI *";
            string context = "0 0/5 * * * ?";
            _ = JobSchedule.CreateJob("Monitoring2", "PMS", context);
            return Ok("Done");
        }


        [HttpGet, Route("SNWR025_DeleteTask")]
        public IHttpActionResult SNWR025_TEST()
        {
            _ = JobSchedule.DeleteJob("Monitoring2", "PMS");
            return Ok("Done");
        }

        #endregion


        #region VAN SNWR047
        [HttpPost, Route("SNWR047_GetVan")]
        [Authorize]
        public IHttpActionResult SNWR047_GetVan([FromBody] dynamic param)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR047_GET_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetVan");
                return Json(new { status = "Error" });
            }
        }

        [HttpPost, Route("SNWR047_GetVanDetail")]
        [Authorize]
        public IHttpActionResult SNWR047_GetVanDetail([FromBody] dynamic param)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR047_GET_VAN_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = param.VAN_GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR047_GetVanDetail");
                return Json(new { status = "Error" });
            }
        }


        [HttpPost, Route("SNWR047_Save")]
        [Authorize]
        public IHttpActionResult SNWR047_Save(SNWR047_SAVE pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                var json = JsonConvert.SerializeObject(pData.vanList);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR047_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_TARGET_VAN_GROUP", OracleDbType.Varchar2).Value = pData.vanTarget.VAN_GROUP;
                cmd.Parameters.Add("P_TARGET_VAN_NO", OracleDbType.Varchar2).Value = pData.vanTarget.VAN_NO;
                cmd.Parameters.Add("P_VAN_LIST", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR047";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR047_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        public class SNWR047_SAVE
        {
            public VanSelect vanTarget { get; set; }
            public List<VanSelect> vanList { get; set; }
        }

        public class VanSelect
        {
            public string VAN_NO { get; set; }
            public string VAN_GROUP { get; set; }
        }

        #endregion SNWR047


        #region VAN SNWR035
        [HttpPost, Route("SNWR035_GetVan")]
        [Authorize]
        public IHttpActionResult SNWR035_GetVan([FromBody] dynamic param)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR035_GET_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = param.ORDER_TYPE;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = param.STATUS;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                //cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (OracleException orx)
            {
                if (orx.Number == 20001)
                {
                    return Json(new { status = "COMPLETE" });
                }
                else if (orx.Number == 20002)
                {
                    return Json(new { status = "NOT_VAN" });
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", orx.Message, "GetVan OracleException");
                    return Json(new { status = "Error" });
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetVan");
                return Json(new { status = "Error" });
            }
        }


        [HttpPost, Route("SNWR035_GetNewVan")]
        [Authorize]
        public IHttpActionResult SNWR035_GetNewVan([FromBody] dynamic param)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR035_GET_NEW_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = param.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                //cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (OracleException orx)
            {
                if (orx.Number == 20001)
                {
                    return Json(new { status = "COMPLETE" });
                }
                else if (orx.Number == 20002)
                {
                    return Json(new { status = "NOT_VAN" });
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", orx.Message, "GetVan OracleException");
                    return Json(new { status = "Error" });
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetVan");
                return Json(new { status = "Error" });
            }
        }


        [HttpGet, Route("SNWR035_GetVanDetail/{van}")]
        [Authorize]
        public IHttpActionResult SNWR035_GetVanDetail(string van)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR035_GET_VAN_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = van;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetVanDetail");
                return Json(new { status = "Error" });
            }
        }


        [HttpPost, Route("SNWR035_CheckVanCase")]
        [Authorize]
        public IHttpActionResult CheckVanCase([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_CHECK_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckVanCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }




        [HttpPost, Route("SNWR035_SaveVan")]
        [Authorize]
        public IHttpActionResult SNWR035_SaveVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                int itemCount = 0;
                string vanGroup = string.Empty;//Guid.NewGuid().ToString();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GET_DOC_VAN";
                cmd.Parameters.Add("R_DOC", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                vanGroup = cmd.Parameters["R_DOC"].Value.ToString();
                foreach (var data in pData.vanList)
                {
                    itemCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR035_UPDATE_VANNING";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data.VAN_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.customer;
                    cmd.Parameters.Add("P_OLD_GROUP", OracleDbType.Varchar2).Value = data.VAN_GROUP;
                    cmd.Parameters.Add("P_GROUP_ID", OracleDbType.Varchar2).Value = vanGroup;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR035 WEB";
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                    oOracledb.SqlQueryReturnValue(cmd);
                    string A = cmd.Parameters["R_STATUS"].Value.ToString();
                }

                foreach (var data in pData.caseList)
                {
                    itemCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR035_SAVE_TEMP_VAN_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = pData.vanNo;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.customer;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.branch;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.order_type;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = itemCount == 1 ? "Y" : "N";
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_SAVE_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = vanGroup;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR035WEB";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS, VAN_GROUP = vanGroup });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_SaveVan WEB");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpGet, Route("SNWR035_DeleteVan/{van}")]
        [Authorize]
        public IHttpActionResult DeleteVan(string van)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR035_DELETE_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = van;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "DeleteVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR035_GetCaseByCustomer")]
        [Authorize]
        public IHttpActionResult GetCaseByCustomer([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GETCASE_BY_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckVanCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }


        [HttpPost, Route("SNWR035_GetCaseByVan")]
        [Authorize]
        public IHttpActionResult SNWR035_GetCaseByVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GETCASE_BY_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckVanCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }



        [HttpPost, Route("SNWR035_GetCaseByNewVan")]
        [Authorize]
        public IHttpActionResult SNWR035_GetCaseByNewVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GETCASE_BY_NEWVANGROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetCaseByNewVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }




        [HttpPost, Route("SNWR035_GetVanForEdit")]
        [Authorize]
        public IHttpActionResult SNWR035_GetVanForEdit([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GETCASE_BY_VANGROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetVanForEdit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpGet, Route("SNWR035_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR035_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR035_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR035_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic Pparam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR035_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = Pparam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR035_ReturnVan")]
        [Authorize]
        public IHttpActionResult SNWR035_ReturnVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR035_RETURN_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR035";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_ReturnVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }




        [HttpPost, Route("SNWR035_GetCaseDetail")]
        [Authorize]
        public IHttpActionResult SNWR035_GetCaseDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR035_GET_CASE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pData.PACK_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR035_GetCaseDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        #endregion VAN

        #region SNSQ090


        [HttpGet, Route("SNSQ090_GetStaffWH/{staff}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ090_GetStaffWH(string staff, int noofshowrec)
        {
            DataSet ds = new DataSet();

            staff = ismoUtil.escapeCharacter(staff);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ090_STAFF_WH";
            cmd.Parameters.Add("P_STAFF", OracleDbType.Varchar2).Value = staff.ToUpper() == "NULL" ? (object)DBNull.Value : staff;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSQ090_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ090_GetData([FromBody] dynamic param)
        {
            string SpName = string.Empty;
            DataSet ds = new DataSet();
            switch (Int32.Parse(param.CURRENT_TAB.ToString()))
            {
                case 0:
                    SpName = "SP_NP_SNSQ090_TAB1";
                    break;
                case 1:
                    SpName = "SP_NP_SNSQ090_TAB2";
                    break;
                case 2:
                    SpName = "SP_NP_SNSQ090_TAB3";
                    break;
                case 3:
                    SpName = "SP_NP_SNSQ090_TAB4";
                    break;
                default:
                    Console.WriteLine("not found!");
                    break;
            }
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = SpName;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Varchar2).Value = param.FROM_DATE_FORMAT;
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Varchar2).Value = param.TO_DATE_FORMAT;
            cmd.Parameters.Add("P_DATE_TYPE", OracleDbType.Varchar2).Value = param.DATE_TYPE;
            cmd.Parameters.Add("P_SECTION_CODE", OracleDbType.Varchar2).Value = param.SECTION_CODE;
            cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = param.STAFF_ID;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ090Tab1_Download")]
        [Authorize]
        public HttpResponseMessage SNSQ090Tab1_Download(List<DataModel.SNSQ090_Tab1> Data)
        {
            DataTable dt = new DataTable();
            dt = DTFM.ToDatatable(Data);
            var x = SNSQ090_Writefile(dt, "0");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ090_Tab1");
            return respons;
        }

        [HttpPost, Route("SNSQ090Tab2_Download")]
        [Authorize]
        public HttpResponseMessage SNSQ090Tab2_Download(List<DataModel.SNSQ090_Tab2> Data)
        {
            DataTable dt = new DataTable();
            dt = DTFM.ToDatatable(Data);
            var x = SNSQ090_Writefile(dt, "1");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ090_Tab2");
            return respons;
        }

        [HttpPost, Route("SNSQ090Tab3_Download")]
        [Authorize]
        public HttpResponseMessage SNSQ090Tab3_Download(List<DataModel.SNSQ090_Tab3> Data)
        {
            DataTable dt = new DataTable();
            dt = DTFM.ToDatatable(Data);
            var x = SNSQ090_Writefile(dt, "2");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ090_Tab3");
            return respons;
        }


        [HttpPost, Route("SNSQ090Tab4_Download")]
        [Authorize]
        public HttpResponseMessage SNSQ090Tab4_Download(List<DataModel.SNSQ090_Tab4> Data)
        {
            DataTable dt = new DataTable();
            dt = DTFM.ToDatatable(Data);
            var x = SNSQ090_Writefile(dt, "3");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ090_Tab4");
            return respons;
        }



        private string SNSQ090_Writefile(DataTable dt, string CurrentTab)
        {
            bool Succes = false;
            string fileName = string.Empty;
            string firstRow = string.Empty;
            string[] HeaderForExport = new string[] { "" };
            switch (Int32.Parse(CurrentTab))
            {
                case 0:

                    fileName = "SNSQ090_LocationType";
                    firstRow = ",,,,Productivitie by Item,,,,,,,Productivitie by Qty";
                    HeaderForExport = new string[]
                    {"Seq","Section","Staff ID","Staff Name","Small","Medium","Large","Heavy","2nd","Other","Total Item","Small","Medium","Large","Heavy","2nd","Other","Total Qty"};
                    break;
                case 1:

                    fileName = "SNSQ090_OrderClass";
                    firstRow = ",,,,Productivitie by Item,,,,,,,Productivitie by Qty";
                    HeaderForExport = new string[]
                    {"Seq","Section","Staff ID","Staff Name","Claim","Emerg","Indent","Regular","WS","Oil","Total Item","Claim","Emerg","Indent","Regular","WS","Oil","Total Qty"};
                    break;
                case 2:

                    fileName = "SNSQ090_SaleChannel";
                    firstRow = ",,,,Productivitie by Item,,,,,,,Productivitie by Qty" + "\n" + ",,,,,-----Domestic-----,,,,,,,,,,,-----Export-----,,,,,,,,,,,,-----Domestic-----,,,,,,,,,,,-----Export-----";
                    HeaderForExport = new string[]
                    {"Seq","Section","Staff ID","Staff Name","BKK Car","BKK Part","UPC Car","UPC Part","Other","Air","Sea","Total Item","BKK Car","BKK Part","UPC Car","UPC Part","Other","Air","Sea","Total Qty"};
                    break;
                case 3:

                    fileName = "SNSQ090_Timing";
                    firstRow = ",,,,,Productivitie by Item,,,,,,,,,,,,,,,,,,,,,,,Productivitie by Qty" + "\n" + ",,,,,-----Domestic-----,,,,,,,,,,,-----Export-----,,,,,,,,,,,,-----Domestic-----,,,,,,,,,,,-----Export-----";
                    HeaderForExport = new string[]
                    {"Seq","Section","Staff ID","Staff Name","Order Class","8:00-9:00","9:00-10:00","10:00-11:00","11:00-12:00","12:00-13:00","13:00-14:00","14:00-15:00","15:00-16:00","16:00-17:00","OT", "Total","8:00-9:00","9:00-10:00","10:00-11:00","11:00-12:00","12:00-13:00","13:00-14:00","14:00-15:00"
                    ,"15:00-16:00","16:00-17:00","OT", "Total","Item Total","8:00-9:00","9:00-10:00","10:00-11:00","11:00-12:00","12:00-13:00","13:00-14:00","14:00-15:00","15:00-16:00","16:00-17:00","OT", "Total","8:00-9:00","9:00-10:00","10:00-11:00","11:00-12:00","12:00-13:00","13:00-14:00","14:00-15:00"
                    ,"15:00-16:00","16:00-17:00","OT", "Total,Qty Total"};
                    break;
                default:
                    Console.WriteLine("not found!");
                    break;
            }

            string gLocation = "~/tempFile/" + "SNSQ090" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);

            try
            {

                StringBuilder sb = new StringBuilder();
                sb.AppendLine(firstRow);

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {

                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }

                File.WriteAllText(gFileLocation, sb.ToString());
                Succes = true;

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }
        }


        #endregion

        #region SNM016P
        [HttpGet, Route("SNM016P_GetRoute/{route}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM016P_GetRoute(string route, int noofshowrec)
        {
            DataSet ds = new DataSet();

            route = ismoUtil.escapeCharacter(route);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM016P_ROUTECODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ROUTECODE", OracleDbType.Varchar2).Value = route.ToUpper() == "NULL" ? (object)DBNull.Value : route;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM016P_GetRouteDetail")]
        [Authorize]
        public IHttpActionResult SNM016P_GetRouteDetail([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM016P_ROUTECODE_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ROUTECODE", OracleDbType.Varchar2).Value = param.ROUTE_CODE;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM016P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM016P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM016P_INSERT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = param.ROUTE_CODE;
            cmd.Parameters.Add("P_ROUTE_NAME_EN", OracleDbType.Varchar2).Value = param.ROUTE_NAME_EN;
            cmd.Parameters.Add("P_ROUTE_NAME_TH", OracleDbType.Varchar2).Value = param.ROUTE_NAME_TH;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM016P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM016P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM016P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM016P_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = param.ROUTE_CODE;
            cmd.Parameters.Add("P_ROUTE_NAME_EN", OracleDbType.Varchar2).Value = param.ROUTE_NAME_EN;
            cmd.Parameters.Add("P_ROUTE_NAME_TH", OracleDbType.Varchar2).Value = param.ROUTE_NAME_TH;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM016P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM016P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM016P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM016P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = param.ROUTE_CODE;
            cmd.Parameters.Add("P_ROUTE_NAME_EN", OracleDbType.Varchar2).Value = param.ROUTE_NAME_EN;
            cmd.Parameters.Add("P_ROUTE_NAME_TH", OracleDbType.Varchar2).Value = param.ROUTE_NAME_TH;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM016P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM007P

        [HttpGet, Route("SNM007P_GetTransport/{transport}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM007P_GetTransport(string transport, int noofshowrec)
        {
            DataSet ds = new DataSet();

            transport = ismoUtil.escapeCharacter(transport);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM007P_TRANSPORT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TRANSPORT", OracleDbType.Varchar2).Value = transport.ToUpper() == "NULL" ? (object)DBNull.Value : transport;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM007P_GetTransportDetail")]
        [Authorize]
        public IHttpActionResult SNM007P_GetTransportDetail([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM007P_TRANSPORT_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TRANSPORT", OracleDbType.Varchar2).Value = param.TRANSPORT;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM007P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM007P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM007P_INSERT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = param.TRANSPORT_CODE;
            cmd.Parameters.Add("P_TRANS_NAME_EN", OracleDbType.Varchar2).Value = param.TRANS_NAME_EN;
            cmd.Parameters.Add("P_TRANS_NAME_TH", OracleDbType.Varchar2).Value = param.TRANS_NAME_TH;
            cmd.Parameters.Add("P_TRANS_ADDR1", OracleDbType.Varchar2).Value = param.TRANS_ADDR1;
            cmd.Parameters.Add("P_TRANS_ADDR2", OracleDbType.Varchar2).Value = param.TRANS_ADDR2;
            cmd.Parameters.Add("P_TRANS_ADDR3", OracleDbType.Varchar2).Value = param.TRANS_ADDR3;
            cmd.Parameters.Add("P_TRANS_ADDR4", OracleDbType.Varchar2).Value = param.TRANS_ADDR4;
            cmd.Parameters.Add("P_TRANS_CONTACT", OracleDbType.Varchar2).Value = param.TRANS_CONTACT;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM007P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM007P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM007P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM007P_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = param.TRANSPORT_CODE;
            cmd.Parameters.Add("P_TRANS_NAME_EN", OracleDbType.Varchar2).Value = param.TRANS_NAME_EN;
            cmd.Parameters.Add("P_TRANS_NAME_TH", OracleDbType.Varchar2).Value = param.TRANS_NAME_TH;
            cmd.Parameters.Add("P_TRANS_ADDR1", OracleDbType.Varchar2).Value = param.TRANS_ADDR1;
            cmd.Parameters.Add("P_TRANS_ADDR2", OracleDbType.Varchar2).Value = param.TRANS_ADDR2;
            cmd.Parameters.Add("P_TRANS_ADDR3", OracleDbType.Varchar2).Value = param.TRANS_ADDR3;
            cmd.Parameters.Add("P_TRANS_ADDR4", OracleDbType.Varchar2).Value = param.TRANS_ADDR4;
            cmd.Parameters.Add("P_TRANS_CONTACT", OracleDbType.Varchar2).Value = param.TRANS_CONTACT;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM007P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM007P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM007P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM007P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = param.TRANSPORT_CODE;
            cmd.Parameters.Add("P_TRANS_NAME_EN", OracleDbType.Varchar2).Value = param.TRANS_NAME_EN;
            cmd.Parameters.Add("P_TRANS_NAME_TH", OracleDbType.Varchar2).Value = param.TRANS_NAME_TH;
            cmd.Parameters.Add("P_TRANS_ADDR1", OracleDbType.Varchar2).Value = param.TRANS_ADDR1;
            cmd.Parameters.Add("P_TRANS_ADDR2", OracleDbType.Varchar2).Value = param.TRANS_ADDR2;
            cmd.Parameters.Add("P_TRANS_ADDR3", OracleDbType.Varchar2).Value = param.TRANS_ADDR3;
            cmd.Parameters.Add("P_TRANS_ADDR4", OracleDbType.Varchar2).Value = param.TRANS_ADDR4;
            cmd.Parameters.Add("P_TRANS_CONTACT", OracleDbType.Varchar2).Value = param.TRANS_CONTACT;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM007P";
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM033

        [HttpPost, Route("SNM033_GetData")]
        [Authorize]
        public IHttpActionResult SNM033_GetData([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_GET_DATA";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATOIN_NO", OracleDbType.Varchar2).Value = pData.LOCATION_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pData.RACK_CODE;
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pData.RACK_GROUP_CODE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int64).Value = pData.SHOW_RECORD;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM033_GetEmptyLocation")]
        [Authorize]
        public IHttpActionResult SNM033_GetEmptyLocation([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_EMPTY_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_GetEmptyLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM033_Save")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM033_Save([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_UPDATE";

                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO.ToString();
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MAX.ToString();
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MID.ToString();
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MIN.ToString();
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE.ToString();
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE.ToString();
                cmd.Parameters.Add("P_SEQUENCE_NO", OracleDbType.Varchar2).Value = pDataList.SEQUENCE_NO.ToString();
                cmd.Parameters.Add("P_LOCK_LOCATION", OracleDbType.Varchar2).Value = pDataList.LOCK_LOCATION.ToString();
                cmd.Parameters.Add("P_CAPACITY_QTY", OracleDbType.Varchar2).Value = pDataList.CAPACITY_QTY.ToString();
                cmd.Parameters.Add("P_SHELVE_QTY", OracleDbType.Varchar2).Value = pDataList.SHELVE_QTY.ToString();
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM033";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM033_SavePart")]
        [Authorize]
        public IHttpActionResult SNM033_SavePart([FromBody] dynamic pData)
        {

            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_UPDATE_PART";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATOIN_NO", OracleDbType.Varchar2).Value = pData.LOCATION_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pData.DIMENSION_MAX;
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pData.DIMENSION_MID;
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pData.DIMENSION_MIN;
                cmd.Parameters.Add("P_LAST_PGM", OracleDbType.Varchar2).Value = "SNM033";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_SavePart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("GetLockLocation/{lock_location}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetLockLocation(string lock_location, int noofshowrec)
        {
            DataSet ds = new DataSet();

            try
            {
                lock_location = ismoUtil.escapeCharacter(lock_location);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_GET_LOCK_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCK_LOCATION", OracleDbType.Varchar2).Value = lock_location == "null" ? (object)DBNull.Value : lock_location;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLockLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM033_GetPartDetail/{part_no}")]
        [Authorize]
        public IHttpActionResult SNM033_GetPartDetail(string part_no)
        {


            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_GET_PART_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part_no;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_GetPartDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM033_GetControlSystem/{Location_No}")]
        [Authorize]
        public IHttpActionResult SNM033_GetControlSystem(string Location_No)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033_GET_CONTROL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = Location_No;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033_GetControlSystem");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM033P_ClearPart")]
        [Authorize]
        public IHttpActionResult SNM033P_ClearPart([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_CLEAR_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATOIN_NO", OracleDbType.Varchar2).Value = pData.LOCATION_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM033P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_ClearPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM033P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM033P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM033P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                SystemApiController sys = new SystemApiController();
                //dt = sys.ReadDataExcel(gFileLocation);
                dt = SNM033P_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }


        public DataTable SNM033P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("MSG");
            dt.Columns.Add("ERR_FLAG");
            Hashtable hTable = new Hashtable();
            int iCount = 0;

            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["LOCATION_NO"].ToString()))
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM033P_VALIDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = row["LOCATION_NO"];
                    cmd.Parameters.Add("R_ERROR_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    var CHECK = cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString();

                    row["SEQ"] = iCount;
                    if (CHECK == "PASS")
                    {
                        row["MSG"] = DBNull.Value;
                        row["ERR_FLAG"] = "N";
                    }
                    else
                    {
                        row["MSG"] = cmd.Parameters["R_ERROR_MESSAGE"].Value;
                        row["ERR_FLAG"] = "Y";
                    }

                    if (row["LOCATION_NO"].ToString().Length > 10)
                    {
                        row["MSG"] = "Location No cannot mare than 10 charactor";
                        row["ERR_FLAG"] = "Y";
                    }


                    if (hTable.Contains(row["LOCATION_NO"].ToString().Trim()))
                    {
                        //row["SEQ"] = iCount;
                        row["MSG"] = "Duplicate Location No at Seq " + hTable[row["LOCATION_NO"]];
                        row["ERR_FLAG"] = "Y";
                    }
                    else
                    {
                        hTable.Add(row["LOCATION_NO"].ToString().Trim(), iCount);
                    }

                }
            }

            return dt;
        }


        [HttpPost, Route("SNM033P_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM033P_SaveByUpload([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM033P_SAVE_TEMP";
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = d["LOCATION_NO"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_TEMP_LOCATION", OracleDbType.Varchar2).Value = d["TEMP_LOCATION"];
                    cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = d["PART_CATEGORY"];
                    cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = d["RACK_CODE"];
                    cmd.Parameters.Add("P_RACKGROUP", OracleDbType.Varchar2).Value = d["RACKGROUP"];
                    cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = d["DIMENSION_MAX"];
                    cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = d["DIMENSION_MID"];
                    cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = d["DIMENSION_MIN"];
                    cmd.Parameters.Add("P_LOCK_LOCATE", OracleDbType.Varchar2).Value = d["LOCK_LOCATE"];
                    cmd.Parameters.Add("P_DISTANCE_RECEIVE", OracleDbType.Varchar2).Value = d["DISTANCE_RECEIVE"];
                    cmd.Parameters.Add("P_DISTANCE_DELIVERY", OracleDbType.Varchar2).Value = d["DISTANCE_DELIVERY"];
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNM033P_SaveByUpload");
                }

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_SaveByUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }
        #endregion

        #region SNM025P
        [HttpGet, Route("SNM025P_GetRack/{rackcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM025P_GetRack(string rackcode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                rackcode = ismoUtil.escapeCharacter(rackcode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM025P_GETRACK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = rackcode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM025P_GetRack");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM025P_GetRack_List/{rackcode=null}")]
        [Authorize]
        public IHttpActionResult SNM025P_GetRack_List(string rackcode = null)
        {
            try
            {
                DataSet ds = new DataSet();

                rackcode = ismoUtil.escapeCharacter(rackcode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM025P_GETRACK_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = (rackcode != null && rackcode == "null" ? "" : rackcode);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM025P_GetRack_List");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM025P_GetRack_Info/{rackcode}")]
        [Authorize]
        public IHttpActionResult SNM025P_GetRack_Info(string rackcode)
        {
            try
            {
                DataSet ds = new DataSet();

                rackcode = ismoUtil.escapeCharacter(rackcode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM025P_GETRACK_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = rackcode != "@@@" ? rackcode : null;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM025P_GetRack_Info");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM025P_Rack_Save")]
        [Authorize]
        public IHttpActionResult SNM025P_Rack_Save([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM025P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pDataList.ITEM_NO;
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MAX;
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MID;
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MIN;
                cmd.Parameters.Add("P_VOLUMN_MAX", OracleDbType.Varchar2).Value = pDataList.VOLUMN_MAX;
                cmd.Parameters.Add("P_VOLUMN_MIN", OracleDbType.Varchar2).Value = pDataList.VOLUMN_MIN;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = pDataList.CREATE_BY;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM025P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = pDataList.LAST_IP_ADDR;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM025P_Rack_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM025P_delete_Rack")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM025P_delete_Rack([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM025P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = pDataList.CREATE_BY;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM025P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = pDataList.LAST_IP_ADDR;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM025P_delete_Rack");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNM026P
        [HttpPost, Route("SNM026P_GetData")]
        [Authorize]
        public IHttpActionResult SNM026P_GetData([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_GETDATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pData.RACK_CODE;
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pData.PART_CATEGORY;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM026P_GetRack/{rackcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM026P_GetRack(string rackcode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                rackcode = ismoUtil.escapeCharacter(rackcode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_GETRACK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = rackcode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_GetRack");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM026P_GetPartCategory")]
        [Authorize]
        public IHttpActionResult SNM026P_GetPartCategory()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_GETCAT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_GetPartCategory");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM026P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM026P_Insert([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                //IPAddress ipv4Addresses = Array.Find(Dns.GetHostEntry(string.Empty).AddressList, a => a.AddressFamily == AddressFamily.InterNetwork);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY;
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE;
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM026P";
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM026P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM026P_Delete([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                //IPAddress ipv4Addresses = Array.Find(Dns.GetHostEntry(string.Empty).AddressList, a => a.AddressFamily == AddressFamily.InterNetwork);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY;
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE;
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM026P";
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM026P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM026P_Update([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM026P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY;
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE;
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM026P";
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM026P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE184
        [HttpPost, Route("SNSE184_GetData")]
        [Authorize]
        public IHttpActionResult SNSE184_GetData([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE184_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CURRENT_LOCATION_FROM", OracleDbType.Varchar2).Value = pData.CURRENT_LOCATION_FROM;
                cmd.Parameters.Add("P_CURRENT_LOCATION_TO", OracleDbType.Varchar2).Value = pData.CURRENT_LOCATION_TO;
                cmd.Parameters.Add("P_PART_FROM", OracleDbType.Varchar2).Value = pData.PART_FROM;
                cmd.Parameters.Add("P_PART_TO", OracleDbType.Varchar2).Value = pData.PART_TO;
                cmd.Parameters.Add("P_NEW_LOCATION_FROM", OracleDbType.Varchar2).Value = pData.NEW_LOCATION_FROM;
                cmd.Parameters.Add("P_NEW_LOCATION_TO", OracleDbType.Varchar2).Value = pData.NEW_LOCATION_TO;
                cmd.Parameters.Add("P_PRINT_DATE_FROM", OracleDbType.Varchar2).Value = pData.PRINT_DATE_FROM.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_PRINT_DATE_TO", OracleDbType.Varchar2).Value = pData.PRINT_DATE_TO.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_STATUS_NO", OracleDbType.Varchar2).Value = pData.STATUS_NO;
                cmd.Parameters.Add("P_UPDATE_FLAG", OracleDbType.Varchar2).Value = pData.UPDATE_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE184_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE184_GetDataLog")]
        [Authorize]
        public IHttpActionResult SNSE184_GetDataLog([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE184_GET_DATA_LOG";
                cmd.Parameters.Add("P_PARTS_NO", OracleDbType.Varchar2).Value = pData.PARTS_NO;
                cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = pData.PRINT_CODE;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE184_GetDataLog");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetLocation/{location}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetLocation(string location, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                location = ismoUtil.escapeCharacter(location);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE184_RELOCATION";
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetNewLocation/{location}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetNewLocation(string location, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                location = ismoUtil.escapeCharacter(location);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE184_NEW_LOCATION";
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetNewLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetPrintCode/{printcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetPrintCode(string printcode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                printcode = ismoUtil.escapeCharacter(printcode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE184_PRINT_CODE";
                cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = printcode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetPrintCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE184_Export")]
        public HttpResponseMessage SNSE184_Export(List<DataModel.SNSE184> dModel)
        {
            //DataTable dt = new DataTable();
            //string x = string.Empty;
            //string filename = "SNSE184.csv";
            //string floderLocation = "SNSE184";
            //dt = DTFM.ToDatatable<DataModel.SNSE184>(dModel);
            //x = DataService.WriteCSV(dt, filename, floderLocation, "SNSE184");
            //Controllers.SystemApiController response = new Controllers.SystemApiController();
            //var File = response.downloadFile(x.Substring(1), filename);
            //return File;

            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSE184.csv";
            string floderLocation = "SNSE184";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq.\"");
            dt.Rows.Add("C;X2;Y1;K\"Print Date\"");
            dt.Rows.Add("C;X3;Y1;K\"Part No.\"");
            dt.Rows.Add("C;X4;Y1;K\"Part Name EN\"");
            dt.Rows.Add("C;X5;Y1;K\"Part Name TH\"");
            dt.Rows.Add("C;X6;Y1;K\"Loc Div\"");
            dt.Rows.Add("C;X7;Y1;K\"Current Location\"");
            dt.Rows.Add("C;X8;Y1;K\"New Location\"");
            dt.Rows.Add("C;X9;Y1;K\"Status No.\"");
            dt.Rows.Add("C;X10;Y1;K\"Update Flag\"");
            dt.Rows.Add("C;X11;Y1;K\"Relo QTY\"");
            dt.Rows.Add("C;X12;Y1;K\"Pick QTY\"");
            dt.Rows.Add("C;X13;Y1;K\"Case No.\"");
            dt.Rows.Add("C;X14;Y1;K\"Pick Date\"");
            dt.Rows.Add("C;X15;Y1;K\"Shelve QTY\"");
            dt.Rows.Add("C;X16;Y1;K\"Shelve Date\"");


            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].PRINT_CODE + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].PARTS_NO + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME_EN + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME_TH + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].LOC_DIV + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].LOCATION_NO + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].NEW_LOCATION + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].STATUS_CODE + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].UPD_FLAG + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].RELO_QTY + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].PICK_QTY + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].CASE_NO + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].PICK_HT_DATE + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].SHELVE_QTY + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].SHELVE_HT_DATE + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }
        #endregion

        #region SNM109P
        [HttpGet, Route("SNM109P_Query/{prefix_part_no}")]
        [Authorize]
        public IHttpActionResult SNM109P_Query(string prefix_part_no)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM109P_QUERY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PREFIX_PART_NO", OracleDbType.Varchar2).Value = prefix_part_no == "null" ? (object)DBNull.Value : prefix_part_no;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM109P_Save")]
        [Authorize]
        public IHttpActionResult SNM109P_Save([FromBody] dynamic[] pPrefixParts)
        {
            foreach (var prefixPart in pPrefixParts)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM109P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PREFIX_PART_NO", OracleDbType.Varchar2).Value = prefixPart["PREFIX_PART_NO"];
                cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = prefixPart["DESCRIPTION"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);
            }


            return Ok();
        }


        [HttpPost, Route("SNM109P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM109P_Delete([FromBody] dynamic[] pPrefixParts)
        {
            foreach (var prefixPart in pPrefixParts)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM109P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PREFIX_PART_NO", OracleDbType.Varchar2).Value = prefixPart["PREFIX_PART_NO"];

                oOracledb.SqlQueryReturnValue(cmd);
            }

            return Ok();
        }
        [HttpPost, Route("SNM109P_Deletes")]
        [Authorize]
        public IHttpActionResult SNM109P_Deletes([FromBody] dynamic pPrefixParts)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM109P_DELETES";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PREFIX_PART_NO", OracleDbType.Varchar2).Value = pPrefixParts.PREFIX_PART_NO.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM109P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM109P_Insert([FromBody] dynamic pPrefixParts)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM109P_INSERT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PREFIX_PART_NO", OracleDbType.Varchar2).Value = pPrefixParts.PREFIX_PART_NO.ToString();
            cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = pPrefixParts.PREFIX_DEXCRIPTION.ToString() == "null" ? (object)DBNull.Value : pPrefixParts.PREFIX_DEXCRIPTION.ToString();
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM033P

        [HttpGet, Route("SNM033P_GetData/{location_no?}")]
        [Authorize]
        public IHttpActionResult SNM033P_GetData(string location_no = null)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location_no;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM033P_InsertLocation")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM033P_InsertLocation([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO.ToString();
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MAX.ToString();
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MID.ToString();
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MIN.ToString();
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE.ToString();
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE.ToString();
                cmd.Parameters.Add("P_SEQUENCE_NO", OracleDbType.Varchar2).Value = pDataList.SEQUENCE_NO.ToString();
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY.ToString();
                cmd.Parameters.Add("P_LOCK_LOCATION", OracleDbType.Varchar2).Value = pDataList.LOCK_LOCATION.ToString();
                cmd.Parameters.Add("P_FLOOR_NO", OracleDbType.Varchar2).Value = pDataList.FLOOR_NO.ToString();
                cmd.Parameters.Add("P_MAX_WEIGHT", OracleDbType.Varchar2).Value = pDataList.MAX_WEIGHT.ToString();
                cmd.Parameters.Add("P_CAPACITY_QTY", OracleDbType.Varchar2).Value = pDataList.CAPACITY_QTY.ToString();
                cmd.Parameters.Add("P_SHELVE_QTY", OracleDbType.Varchar2).Value = pDataList.SHELVE_QTY.ToString();

                cmd.Parameters.Add("P_TEMP_LOCATION", OracleDbType.Varchar2).Value = pDataList.TEMP_LOCATION.ToString();

                cmd.Parameters.Add("P_DISTANCE_RECEIVE1", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE1.ToString();
                cmd.Parameters.Add("P_DISTANCE_RECEIVE2", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE2.ToString();
                cmd.Parameters.Add("P_DISTANCE_RECEIVE3", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE3.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY1", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY1.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY2", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY2.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY3", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY3.ToString();

                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pDataList.ATTRIBUTE2.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM033P";

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_InsertLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM033P_UpdateLocation")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM033P_UpdateLocation([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MAX.ToString();
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MID.ToString();
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList.DIMENSION_MIN.ToString();
                cmd.Parameters.Add("P_RACK_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_CODE.ToString();
                cmd.Parameters.Add("P_RACK_GROUP_CODE", OracleDbType.Varchar2).Value = pDataList.RACK_GROUP_CODE.ToString();
                cmd.Parameters.Add("P_SEQUENCE_NO", OracleDbType.Varchar2).Value = pDataList.SEQUENCE_NO.ToString();
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY.ToString();
                cmd.Parameters.Add("P_LOCK_LOCATION", OracleDbType.Varchar2).Value = pDataList.LOCK_LOCATION.ToString();
                cmd.Parameters.Add("P_FLOOR_NO", OracleDbType.Varchar2).Value = pDataList.FLOOR_NO.ToString();
                cmd.Parameters.Add("P_MAX_WEIGHT", OracleDbType.Varchar2).Value = pDataList.MAX_WEIGHT.ToString();
                cmd.Parameters.Add("P_CAPACITY_QTY", OracleDbType.Varchar2).Value = pDataList.CAPACITY_QTY.ToString();
                cmd.Parameters.Add("P_SHELVE_QTY", OracleDbType.Varchar2).Value = pDataList.SHELVE_QTY.ToString();
                cmd.Parameters.Add("P_TEMP_LOCATION", OracleDbType.Varchar2).Value = pDataList.TEMP_LOCATION.ToString();

                cmd.Parameters.Add("P_DISTANCE_RECEIVE1", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE1.ToString();
                cmd.Parameters.Add("P_DISTANCE_RECEIVE2", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE2.ToString();
                cmd.Parameters.Add("P_DISTANCE_RECEIVE3", OracleDbType.Varchar2).Value = pDataList.DISTANCE_RECEIVE3.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY1", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY1.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY2", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY2.ToString();
                cmd.Parameters.Add("P_DISTANCE_DELIVERY3", OracleDbType.Varchar2).Value = pDataList.DISTANCE_DELIVERY3.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pDataList.ATTRIBUTE2.ToString();
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pDataList.ATTRIBUTE5.ToString();
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM033P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_UpdateLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM033P_Delete/{location_no}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM033P_Delete(string location_no)
        {

            DataSet ds = new DataSet();
            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM033P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATOIN_NO", OracleDbType.Varchar2).Value = location_no;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM033P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }





        #endregion

        #region SNM108P
        [HttpPost, Route("SNM108P_GetData")]
        [Authorize]
        public IHttpActionResult SNM108P_GetData([FromBody] dynamic pParam)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_GETDATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
            cmd.Parameters.Add("P_HUB_CODE", OracleDbType.Varchar2).Value = pParam.HUB_CODE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM108P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM108P_Insert([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_INSERT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE.ToString();
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE.ToString();
            cmd.Parameters.Add("P_HUB_CODE", OracleDbType.Varchar2).Value = pData.HUB_CODE.ToString();
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM108P";
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM108P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM108P_Update([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE.ToString();
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE.ToString();
            cmd.Parameters.Add("P_HUB_CODE", OracleDbType.Varchar2).Value = pData.HUB_CODE.ToString();
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM108P";
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM108P_Validate/{Customer_Code}/{Branch_Code}")]
        [Authorize]
        public IHttpActionResult SNM108P_Validate(string Customer_Code, string Branch_Code)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_VALIDATE";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = Customer_Code;
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = Branch_Code;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM108P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM108P_Delete([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM108P";
            cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);


            return Json(DTFM.convertToList(ds));


        }

        [HttpGet, Route("GetCustomerAddHub/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetCustomerAddHub(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customer = ismoUtil.escapeCharacter(customer);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM108P_GET_CUSTOMER";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNPP999Z
        [HttpGet, Route("SNPP999Z_Query/{prefix_part_no}")]
        [Authorize]
        public IHttpActionResult SNPP999Z_Query(string strInvoiceNo)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM109P_QUERY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = strInvoiceNo == "null" ? (object)DBNull.Value : strInvoiceNo;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion
        //start BF.
        #region SNSE037
        //Start BF. 10M3Y2020
        [HttpGet, Route("SNSE037_DocDate_Find/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE037_DocDate_Find(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPR037_GET_DOC_DATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOC_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE037_DocNo_Find/{strFindText}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE037_DocNo_Find(string strFindText, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPR037_GET_DOC_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_TEXT", OracleDbType.Varchar2).Value = strFindText == "null" ? (object)DBNull.Value : strFindText;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE037_DocType_Find/{strFindText}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE037_DocType_Find(string strFindText, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            //cmd.CommandText = "SP_NP_SNPR037_GET_DOC_DATE";
            cmd.CommandText = "SP_NP_SNPR037_GET_DOC_TYPE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_TEXT", OracleDbType.Varchar2).Value = strFindText == "null" ? (object)DBNull.Value : strFindText;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        //End BF. 10M3Y2020
        #endregion

        #region SNSE039P

        //start BF. 25M02Y2020

        //[HttpGet, Route("SNSE039PInvoiceDateList/{InvoiceDate}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE039PInvoiceDateList(string InvoiceDate, int noofshowrec)
        //{
        //    DataSet ds = new DataSet();

        //    InvoiceDate = ismoUtil.escapeCharacter(InvoiceDate);

        //    try
        //    {
        //        OracleCommand cmd = new OracleCommand();
        //        cmd.CommandText = "SP_NP_SNSE039P_GET_DATE";
        //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = InvoiceDate.ToUpper() == "NULL" ? (object)DBNull.Value : InvoiceDate;
        //        cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
        //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //        ds = oOracledb.SqlQueryDS_CMD(cmd);

        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE039PInvoiceDateList");
        //        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
        //    }
        //}

        [HttpPost, Route("SNSE039PInvoiceSeqList/{InvoiceSeq}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE039PInvoiceSeqList(string InvoiceSeq, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            InvoiceSeq = ismoUtil.escapeCharacter(InvoiceSeq);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE039P_GET_SEQ_BY_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = pData["INVOICE_DATE"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_INVOICE_SEQ", OracleDbType.Varchar2).Value = InvoiceSeq.ToUpper() == "NULL" ? (object)DBNull.Value : InvoiceSeq;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE039PInvoiceSeqList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE039P_CheckDataReport")]
        [Authorize]
        public IHttpActionResult SNSE039P_CheckDataReport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE039R_P_A4_RPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO_FROM", OracleDbType.Varchar2).Value = pParam.BEGIN_INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_NO_TO", OracleDbType.Varchar2).Value = pParam.END_INVOICE_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE039P_CheckDataReport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE039P_SetPrintData")]
        [Authorize]
        public IHttpActionResult SNSE039P_SetPrintData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE039R_P_SET_PRINT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO_FROM", OracleDbType.Varchar2).Value = pParam.BEGIN_INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_NO_TO", OracleDbType.Varchar2).Value = pParam.END_INVOICE_NO;
                cmd.Parameters.Add("P_PRINT_RECORD", OracleDbType.Int32).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                int returnRecords;

                if (Int32.TryParse(cmd.Parameters["P_PRINT_RECORD"].Value.ToString(), out returnRecords))
                {
                    if (returnRecords > 0)
                    {
                        return Ok("Exists");
                    }
                    return Ok("No record");
                }

                return Ok("Unable to print");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE039P_SetPrintData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //ends BF. 25M02Y2020
        #endregion

        #region SNSE054D_ZB
        //start BF. 4M03Y2020
        [HttpGet, Route("SNSE054D_ZB_PitkLotDate_Find_T/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_ZB_PitkLotDate_Find_T(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_ZB_GETDATE_T";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNSE054D_ZB_PitkLotDate_Find_S/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_ZB_PitkLotDate_Find_S(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_ZB_GETDATE_S";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_ZB_SEQ_BY_DATE/{strInvoiceDate}/{strProcessType}")]
        [Authorize]
        public IHttpActionResult SNSE054D_ZB_SEQ_BY_DATE(string strInvoiceDate, string strProcessType)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_ZB_GET_SEQ";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_ZB_CUST_BY_DATE_SEQ/{strInvoiceDate}/{strProcessType}/{strSeq}")]
        [Authorize]
        public IHttpActionResult SNSE054D_ZB_CUST_BY_DATE_SEQ(string strInvoiceDate, string strProcessType, string strSeq)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_ZB_GET_CUST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = strSeq == "null" ? (object)DBNull.Value : strSeq.Trim();
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_ZB_SAVE_TO_TEMP/{strInvoiceDate}/{strProcessType}/{strSeq}/{strCustomerCode}")]
        [Authorize]
        public IHttpActionResult SNSE054D_ZB_SAVE_TO_TEMP(string strInvoiceDate, string strProcessType, string strSeq, string strCustomerCode)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_ZB_PRE_D_TO_RPT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = strSeq == "null" ? (object)DBNull.Value : strSeq.Trim();
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = strCustomerCode == "null" ? (object)DBNull.Value : strSeq.Trim();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        //End BF 4M03Y2020
        #endregion

        #region SNSE054D_Z
        //start BF. 26M02Y2020

        [HttpGet, Route("SNSE054D_Z_PitkLotDate_Find_E21/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_PitkLotDate_Find_E21(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_DATE_E21";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_PitkLotDate_Find_E5/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_PitkLotDate_Find_E5(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_DATE_E5";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_PitkLotDate_Find_R5/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_PitkLotDate_Find_R5(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_DATE_R5";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_PitkLotDate_Find_I3/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_PitkLotDate_Find_I3(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_DATE_I3";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_SEQ_BY_DATE/{strInvoiceDate}/{strProcessType}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_SEQ_BY_DATE(string strInvoiceDate, string strProcessType)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_SEQ";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_CUST_BY_DATE_SEQ/{strInvoiceDate}/{strProcessType}/{strSeq}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_CUST_BY_DATE_SEQ(string strInvoiceDate, string strProcessType, string strSeq)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_GET_CUST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = strSeq == "null" ? (object)DBNull.Value : strSeq.Trim();
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 100;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE054D_Z_SAVE_TO_TEMP/{strInvoiceDate}/{strProcessType}/{strSeq}/{strCustomerCode}")]
        [Authorize]
        public IHttpActionResult SNSE054D_Z_SAVE_TO_TEMP(string strInvoiceDate, string strProcessType, string strSeq, string strCustomerCode)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE054D_Z_PRE_D_TO_RPT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = strProcessType == "null" ? (object)DBNull.Value : strProcessType.Trim();
            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate.Trim();
            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = strSeq == "null" ? (object)DBNull.Value : strSeq.Trim();
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = strCustomerCode == "null" ? (object)DBNull.Value : strCustomerCode.Trim();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        //end BF. 26M02Y2020
        #endregion


        #region SNSR041
        //start BF. 24M12Y2019
        [HttpGet, Route("SNSR041_FindCustomer/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSR041_FindCustomer(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR041_CUSTOMER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSR041_Customer_Info/{prefix_Customer_Code}")]
        [Authorize]
        public IHttpActionResult SNSR041_Customer_Info(string prefix_Customer_Code)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR041_CUST_INFO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Customer_Code == "null" ? (object)DBNull.Value : prefix_Customer_Code.ToString().Trim();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSR041_DocumentByCustomer/{prefix_Customer_Code}")]
        [Authorize]
        public IHttpActionResult SNSR041_DocumentByCustomer(string prefix_Customer_Code)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR041_DOC_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Customer_Code == "null" ? (object)DBNull.Value : prefix_Customer_Code.ToString().Trim();
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16, 100, ParameterDirection.Input);

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNSR040
        [HttpGet, Route("SNSR040_FindCustomer40/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSR040_FindCustomer40(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR040_CUSTOMER40";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNSR040_Customer_Info/{prefix_Customer_Code}")]
        [Authorize]
        public IHttpActionResult SNSR040_Customer_Info(string prefix_Customer_Code)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR040_CUST_INFO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Customer_Code == "null" ? (object)DBNull.Value : prefix_Customer_Code.ToString().Trim();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSR040_DocumentByCustomer/{prefix_Customer_Code}")]
        [Authorize]
        public IHttpActionResult SNSR040_DocumentByCustomer(string prefix_Customer_Code)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR040_DOC_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Customer_Code == "null" ? (object)DBNull.Value : prefix_Customer_Code.ToString().Trim();
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16, 100, ParameterDirection.Input);

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSR040_SaveStickerBeforePrint")]
        public IHttpActionResult SNSR040_SaveStickerBeforePrint([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            gSQL = "SP_NP_SNSR040_SAVE_STICKER";

            try
            {
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"].ToString();
                cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = data["CUSTOMER_NAME"].ToString();
                cmd.Parameters.Add("P_CUSTOMER_AMPHUR", OracleDbType.Varchar2).Value = data["CUSTOMER_AMPHUR"].ToString();
                cmd.Parameters.Add("P_CUSTOMER_PROVINCE", OracleDbType.Varchar2).Value = data["CUSTOMER_PROVINCE"].ToString();
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"].ToString();
                cmd.Parameters.Add("P_BRANCH_NAME", OracleDbType.Varchar2).Value = data["BRANCH_NAME"].ToString();
                cmd.Parameters.Add("P_CASHEER_NO", OracleDbType.Varchar2).Value = data["CASHEER_NO"].ToString();
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = data["DOCUMENT_NO"].ToString();
                cmd.Parameters.Add("P_C_TOTAL_PAGE", OracleDbType.Int32).Value = data["C_TOTAL_PAGE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine("Exception: {0}", ex.ToString());
            }

            return Json(DTFM.convertToList(ds));
        }

        //For auto complete version
        [HttpPost, Route("SNSR040_GET_DOCUMENT/{prefix_Doc_No}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSR040_GET_DOCUMENT(string prefix_Doc_No, int noofshowrec, [FromBody] dynamic pParam)
        {
            //For Document Autocomplete
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSR040_DOC_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = prefix_Doc_No == "null" ? (object)DBNull.Value : prefix_Doc_No.ToString().Trim();
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16, noofshowrec, ParameterDirection.Input);
            //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Doc_No == "null" ? (object)DBNull.Value : prefix_Doc_No.ToString().Trim();
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2, pParam["CUSTOMER_CODE"], ParameterDirection.Input);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        //ends BF. 24M12Y2019
        #endregion

        #region SNSE052

        //start BF. 15M03Y2020
        [HttpGet, Route("SNSE052_FindCustomer/{strInvoiceDate}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE052_FindCustomer(string strInvoiceDate, int pShowRec)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_GET_CUSTOMER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FIND_TEXT", OracleDbType.Varchar2).Value = strInvoiceDate == "null" ? (object)DBNull.Value : strInvoiceDate;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE052_DocumentByCustomer/{pOrderNo}/{pShowRec}")]
        [Authorize]
        public IHttpActionResult SNSE052_DocumentByCustomer(string pOrderNo, int pShowRec, [FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_GET_ORDERNO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CustomerCode.ToString().Trim();
            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pOrderNo == "null" ? (object)DBNull.Value : pOrderNo;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = pShowRec;

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSE052_SearchData")]
        [Authorize]
        public IHttpActionResult SNSE052_SearchData([FromBody] dynamic data)
        {
            string prefix_Customer_Code = data["CUSTOMER_CODE"].ToString();
            string prefix_Order_No = data["ORDER_NO"].ToString();
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_SCH_RESULT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = prefix_Customer_Code == "null" ? (object)DBNull.Value : prefix_Customer_Code.ToString().Trim();
            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = prefix_Order_No == "null" ? (object)DBNull.Value : prefix_Order_No.ToString().Trim();
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16, 100, ParameterDirection.Input);

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE052_ClearOldData")]
        [Authorize]
        public IHttpActionResult SNSE052_ClearOldData([FromBody] dynamic data)
        {
            string strIP = data["pIP"].ToString();
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_CLEAR_ROW";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = strIP == "null" ? (object)DBNull.Value : strIP.ToString().Trim();

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE052_SaveRowSelect")]
        [Authorize]
        public IHttpActionResult SNSE052_SaveRowSelect([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_SAVE_ROW";

            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = data["pIP"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pIP"].ToString().Trim();
            cmd.Parameters.Add("P_AUTO_PRINT", OracleDbType.Varchar2).Value = data["pAUTO_PRINT"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pAUTO_PRINT"].ToString().Trim();
            cmd.Parameters.Add("P_SELECT", OracleDbType.Varchar2).Value = data["pSELECT"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pSELECT"].ToString().Trim();

            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"].ToString().Trim() == "null" ? (object)DBNull.Value : data["CUSTOMER_CODE"].ToString().Trim();
            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data["ORDER_NO"].ToString().Trim() == "null" ? (object)DBNull.Value : data["ORDER_NO"].ToString().Trim();
            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data["ITEM_NO"].ToString().Trim() == "null" ? (object)DBNull.Value : data["ITEM_NO"].ToString().Trim();
            cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data["SUB_ITEM_NO"].ToString().Trim() == "null" ? (object)DBNull.Value : data["SUB_ITEM_NO"].ToString().Trim();
            cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data["INSTRUCTION_NO"].ToString().Trim() == "null" ? (object)DBNull.Value : data["INSTRUCTION_NO"].ToString().Trim();

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
            //return Json(DTFM.convertToList("Done"));
        }

        [HttpPost, Route("SNSE052_SaveAllRow")]
        [Authorize]
        public IHttpActionResult SNSE052_SaveAllRow([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052_SAVE_ALL";

            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = data["pIP"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pIP"].ToString().Trim();
            cmd.Parameters.Add("P_AUTO_PRINT", OracleDbType.Varchar2).Value = data["pAUTO_PRINT"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pAUTO_PRINT"].ToString().Trim();
            cmd.Parameters.Add("P_SELECT", OracleDbType.Varchar2).Value = data["pSELECT"].ToString().Trim() == "null" ? (object)DBNull.Value : data["pSELECT"].ToString().Trim();

            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"].ToString().Trim() == "null" ? (object)DBNull.Value : data["CUSTOMER_CODE"].ToString().Trim();
            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data["ORDER_NO"].ToString().Trim() == "null" ? (object)DBNull.Value : data["ORDER_NO"].ToString().Trim();

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
            //return Json(DTFM.convertToList("Done"));
        }
        //end BF

        #endregion



        //ends BF.



        #region SNPE013
        [HttpGet, Route("SNPE013_GetVendor/{vendorcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE013_GetVendor(string vendorcode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vendorcode = ismoUtil.escapeCharacter(vendorcode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE013_VENDOR_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorcode != null && vendorcode == "null" ? "" : vendorcode;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE013_GetInvoice/{invoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE013_GetInvoice(string invoiceNo, int noofshowrec, [FromBody] dynamic oParams)
        {
            DataSet ds = new DataSet();

            invoiceNo = ismoUtil.escapeCharacter(invoiceNo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE013_INVOICE_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = oParams.vendorCode;
            cmd.Parameters.Add("P_INOVICE_NO", OracleDbType.Varchar2).Value = invoiceNo != null && invoiceNo == "null" ? "" : invoiceNo;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNPE013_GetData/{vendorCode}/{invoiceNo}/{invoiceDate}")]
        [Authorize]
        public IHttpActionResult SNPE013_GetData(string vendorCode, string invoiceNo, string invoiceDate)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE013_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorCode;
            cmd.Parameters.Add("P_INOVICE_NO", OracleDbType.Varchar2).Value = ismoUtil.escapeCharacter(invoiceNo);
            cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(invoiceDate);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE013_SavetoShelve/{vendorCode}/{invoiceNo}/{statusCode}")]
        [Authorize]
        public IHttpActionResult SNPE013_SavetoShelve([FromBody] dynamic[] pDetails, string vendorCode, string invoiceNo, string statusCode)
        {
            if (pDetails.Length > 0)
            {
                OracleCommand cmd = new OracleCommand();
                foreach (var detail in pDetails)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE013_SAVE_DETAIL";
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = detail["PO_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = detail["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = detail["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_QUANTITY", OracleDbType.Int16).Value = detail["ACCEPT_QTY"];
                    cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = detail["STATUS_CODE"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE013_SAVE_TO_SHELVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorCode;
                    cmd.Parameters.Add("P_INOVICE_NO", OracleDbType.Varchar2).Value = ismoUtil.escapeCharacter(invoiceNo);
                    cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = statusCode;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }
                catch (Exception ex)
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

                return Json("Done");
            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.NoContent, "No row to process."));
        }
        #endregion

        #region SNSE182

        [HttpGet, Route("SNSE182_Current_Loca_Find/{CurrentLoca}/{numberrecord}")]
        [Authorize]
        public IHttpActionResult SNSE182_Current_Loca_Find(string CurrentLoca, int numberrecord)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE182_CURRENTLO_FINE";
                cmd.Parameters.Add("P_CURRENT_LOCA", OracleDbType.Varchar2, CurrentLoca != null && CurrentLoca == "null" ? "" : CurrentLoca, ParameterDirection.Input);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_Current_Loca_Find");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE182_Parts_No_Find/{PartsNo}/{numberrecord}")]
        [Authorize]
        public IHttpActionResult SNSE182_Parts_No_Find(string PartsNo, int numberrecord)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE182_PART_NO_FINE";
                cmd.Parameters.Add("P_PARTS_NO", OracleDbType.Varchar2, PartsNo != null && PartsNo == "null" ? "" : PartsNo, ParameterDirection.Input);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_Parts_No_Find");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE182_New_Location_Find/{NewLacation}/{numberrecord}")]
        [Authorize]
        public IHttpActionResult SNSE182_New_Location_Find(string NewLacation, int numberrecord)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE182_NEW_LOCA_FINE";
                cmd.Parameters.Add("P_NEW_LOCATION", OracleDbType.Varchar2, NewLacation != null && NewLacation == "null" ? "" : NewLacation, ParameterDirection.Input);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_New_Location_Find");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE182_PrintDate_Find/{PrintDate}/{numberrecord}")]
        [Authorize]
        public IHttpActionResult SNSE182_PrintDate_Find(string PrintDate, int numberrecord)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE182_PRINT_DATE_FINE";
                cmd.Parameters.Add("P_PRINT_DATE", OracleDbType.Varchar2, PrintDate != null && PrintDate == "null" ? "" : PrintDate, ParameterDirection.Input);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_PrintDate_Find");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE182_GetRelocationParts")]
        [Authorize]
        public IHttpActionResult SNSE182_GetRelocationParts([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE182_GET_RELOCA_PARTS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_LOCATION_NO", OracleDbType.Varchar2).Value = pData.FROM_LOCATION_NO;
                cmd.Parameters.Add("P_TO_LOCATION_NO", OracleDbType.Varchar2).Value = pData.TO_LOCATION_NO;
                cmd.Parameters.Add("P_FROM_NEW_LOCATION", OracleDbType.Varchar2).Value = pData.FROM_NEW_LOCATION;
                cmd.Parameters.Add("P_TO_NEW_LOCATION", OracleDbType.Varchar2).Value = pData.TO_NEW_LOCATION;
                cmd.Parameters.Add("P_FROM_PARTS_NO", OracleDbType.Varchar2).Value = pData.FROM_PARTS_NO;
                cmd.Parameters.Add("P_TO_PARTS_NO", OracleDbType.Varchar2).Value = pData.TO_PARTS_NO;
                cmd.Parameters.Add("P_FROM_PRINT_DATE", OracleDbType.Varchar2).Value = pData.FROM_PRINT_CODE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_TO_PRINT_DATE", OracleDbType.Varchar2).Value = pData.TO_PRINT_CODE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = pData.STATUS_CODE;
                cmd.Parameters.Add("P_UPD_FLAG", OracleDbType.Varchar2).Value = pData.UPD_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_GetRelocationParts");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE182_GetRelocationLog")]
        [Authorize]
        public IHttpActionResult SNSE182_GetRelocationLog([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            gSQL = "SP_NP_SNSE182_GET_RELOCA_LOG";

            try
            {
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PARTS_NO.ToString();
                cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = data.PRINT_CODE.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine("Exception: {0}", ex.ToString());
            }

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE182_ConfirmUpdateLocation")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE182_ConfirmUpdateLocation([FromBody] dynamic[] pPart)
        {
            OracleCommand cmd;
            foreach (var vPart in pPart)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE182_CONFIRM_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = vPart["PRINT_CODE"].ToString();
                    cmd.Parameters.Add("P_PARTS_NO", OracleDbType.Varchar2).Value = vPart["PARTS_NO"].ToString();
                    cmd.Parameters.Add("P_NEW_LOCATION", OracleDbType.Varchar2).Value = vPart["NEW_LOCATION"].ToString();
                    cmd.Parameters.Add("P_UPD_FLAG", OracleDbType.Varchar2).Value = vPart["UPD_FLAG"].ToString();
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_IP_ADDRESS);
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE182_ConfirmUpdateLocation");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

            }

            return Ok();
        }


        //[HttpPost, Route("SNSE182_GetRelocationParts_Export")]
        //[Authorize]
        //public HttpResponseMessage SNSE182_GetRelocationParts_Export([FromBody] dynamic data)
        //{
        //    DataSet ds = new DataSet();

        //    string xxx = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNSE182_EXP_RELOCA_PARTS";
        //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //    cmd.Parameters.Add("P_FROM_LOCATION_NO", OracleDbType.Varchar2).Value = data["FROM_LOCATION_NO"].ToString() == "" ? (object)DBNull.Value : data["FROM_LOCATION_NO"].ToString();
        //    cmd.Parameters.Add("P_TO_LOCATION_NO", OracleDbType.Varchar2).Value = data["TO_LOCATION_NO"].ToString() == "" ? (object)DBNull.Value : data["TO_LOCATION_NO"].ToString();
        //    cmd.Parameters.Add("P_FROM_NEW_LOCATION", OracleDbType.Varchar2).Value = data["FROM_NEW_LOCATION"].ToString() == "" ? (object)DBNull.Value : data["FROM_NEW_LOCATION"].ToString();
        //    cmd.Parameters.Add("P_TO_NEW_LOCATION", OracleDbType.Varchar2).Value = data["TO_NEW_LOCATION"].ToString() == "" ? (object)DBNull.Value : data["TO_NEW_LOCATION"].ToString();
        //    cmd.Parameters.Add("P_FROM_PARTS_NO", OracleDbType.Varchar2).Value = data["FROM_PARTS_NO"].ToString() == "" ? (object)DBNull.Value : data["FROM_PARTS_NO"].ToString();
        //    cmd.Parameters.Add("P_TO_PARTS_NO", OracleDbType.Varchar2).Value = data["TO_PARTS_NO"].ToString() == "" ? (object)DBNull.Value : data["TO_PARTS_NO"].ToString();
        //    cmd.Parameters.Add("P_FROM_PRINT_DATE", OracleDbType.Varchar2).Value = data["FROM_PRINT_CODE"].ToString() == "" ? (object)DBNull.Value : data["FROM_PRINT_CODE"].ToString();
        //    cmd.Parameters.Add("P_TO_PRINT_DATE", OracleDbType.Varchar2).Value = data["TO_PRINT_CODE"].ToString() == "" ? (object)DBNull.Value : data["TO_PRINT_CODE"].ToString();
        //    cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = data["STATUS_CODE"].ToString() == "" ? (object)DBNull.Value : data["STATUS_CODE"].ToString();
        //    cmd.Parameters.Add("P_UPD_FLAG", OracleDbType.Varchar2).Value = data["UPD_FLAG"].ToString() == "" ? (object)DBNull.Value : data["UPD_FLAG"].ToString();
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

        //    ds = oOracledb.SqlQueryDS_CMD(cmd);


        //    string x = SNSE182_ReloParts_Export(ds.Tables[0], "SNSE182_Relocation_Parts.csv", "SNSE182");
        //    SystemApiController sys = new SystemApiController();
        //    var respons = sys.downloadFile(x.Substring(1), "SNSE182_Relocation_Parts.csv");
        //    return respons;

        //}

        //public static string SNSE182_ReloParts_Export(DataTable dt, string fileName, string FloderLocation)
        //{

        //    bool Succes = false;
        //    //DataTable dt = new DataTable();
        //    //dt = ds.Tables[0];
        //    string gLocation = "~/tempFile/" + FloderLocation + "/";
        //    string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

        //    FileInfo filePath = new FileInfo(gWriteLocation);
        //    filePath.Directory.Create();
        //    string gFileLocation = Path.Combine(gWriteLocation, fileName);
        //    string gReturnLocation = Path.Combine(gLocation, fileName);
        //    SystemApiController sys = new SystemApiController();
        //    try
        //    {
        //        StringBuilder sb = new StringBuilder();

        //        IEnumerable<string> columnNames = dt.Columns.Cast<DataColumn>().Select(column => column.ColumnName);
        //        sb.AppendLine(string.Join(",", columnNames));

        //        foreach (DataRow row in dt.Rows)
        //        {
        //            for (int i = 0; i < dt.Columns.Count; i++)
        //            {

        //                sb.Append(CsvEscape(row[i].ToString()));
        //                sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
        //            }
        //        }

        //        File.WriteAllText(gFileLocation, sb.ToString());

        //        Succes = true;
        //    }
        //    catch (Exception ex)
        //    {
        //        Console.WriteLine(ex);
        //    }

        //    if (Succes)
        //    {
        //        return gReturnLocation;
        //    }
        //    else
        //    {

        //        return "";
        //    }

        //}

        //[HttpPost, Route("SNSE182_GetRelocationLog_Export")]
        //[Authorize]
        //public HttpResponseMessage GetBO_By_PartNo([FromBody] dynamic data)
        //{
        //    DataSet ds = new DataSet();

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNSE182_EXP_RELOCA_LOG";
        //    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PartNO;
        //    cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = data.PrintCode;
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //    ds = oOracledb.SqlQueryDS_CMD(cmd);


        //    string x = SNSE182_ReloLog_Export(ds.Tables[0], "SNSE182_Relocation_Log.csv", "SNSE182");
        //    SystemApiController sys = new SystemApiController();
        //    var respons = sys.downloadFile(x.Substring(1), "SNSE182_Relocation_Log.csv");
        //    return respons;

        //}

        //public static string SNSE182_ReloLog_Export(DataTable dt, string fileName, string FloderLocation)
        //{

        //    bool Succes = false;
        //    //DataTable dt = new DataTable();
        //    //dt = ds.Tables[0];
        //    string gLocation = "~/tempFile/" + FloderLocation + "/";
        //    string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

        //    FileInfo filePath = new FileInfo(gWriteLocation);
        //    filePath.Directory.Create();
        //    string gFileLocation = Path.Combine(gWriteLocation, fileName);
        //    string gReturnLocation = Path.Combine(gLocation, fileName);
        //    SystemApiController sys = new SystemApiController();
        //    try
        //    {
        //        StringBuilder sb = new StringBuilder();

        //        IEnumerable<string> columnNames = dt.Columns.Cast<DataColumn>().Select(column => column.ColumnName);
        //        sb.AppendLine(string.Join(",", columnNames));

        //        foreach (DataRow row in dt.Rows)
        //        {
        //            for (int i = 0; i < dt.Columns.Count; i++)
        //            {

        //                sb.Append(CsvEscape(row[i].ToString()));
        //                sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
        //            }
        //        }

        //        File.WriteAllText(gFileLocation, sb.ToString());

        //        Succes = true;
        //    }
        //    catch (Exception ex)
        //    {
        //        Console.WriteLine(ex);
        //    }

        //    if (Succes)
        //    {
        //        return gReturnLocation;
        //    }
        //    else
        //    {

        //        return "";
        //    }

        //}
        static string CsvEscape(string value)
        {
            if (value.Contains(","))
            {
                return "\"" + value.Replace("\"", "\"\"") + "\"";
            }
            return value;
        }

        #endregion

        #region SNPR029

        [HttpGet, Route("SNPR029_GetPart/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPR029_GetPart(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPR029_GETPART";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        public IHttpActionResult SNPR029_DELETE()
        {
            var row = 0;
            string sql = "DELETE FROM TPM_SEQ_SNPR029";
            row = oOracledb.SqlExecute(sql);
            return Ok(row);
        }

        [HttpPost, Route("SNPR029_INSERT")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPR029_INSERT([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            SNPR029_DELETE();
            foreach (var data in pDataList)
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPR029_INSERT";
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data["PART_NO"].ToString();
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = data["QTY"].ToString();
                cmd.Parameters.Add("P_PRN", OracleDbType.Varchar2).Value = data["PRN"].ToString();
                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            return Ok("DONE");
        }
        #endregion

        #region SNSE195

        [HttpGet, Route("SNSE195_Customer_Find/{Customer}/{numberrecord}")]
        [Authorize]
        public IHttpActionResult SNSE195_Customer_Find(string Customer, int numberrecord)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE195_CUSTOMER_FINE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2, Customer != null && Customer == "null" ? "" : Customer, ParameterDirection.Input);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE195_GetCustomerLabel")]
        [Authorize]
        public IHttpActionResult SNSE195_GetCustomerLabel([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            gSQL = "SP_NP_SNSE195_CUS_LABEL_BKK";

            try
            {
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                System.Console.WriteLine("Exception: {0}", ex.ToString());
            }

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE195_CustomerBKK_Save")]
        [Authorize]
        public IHttpActionResult SNSE195_CustomerBKK_Save([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE195_CUS_LABEL_SAVE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LABEL_NAME", OracleDbType.Varchar2).Value = data["LABEL_NAME"].ToString();
            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = data["ROUTE_CODE"].ToString();
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"].ToString();
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"].ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Ok("Done");
        }

        [HttpPost, Route("SNSE195_CustomerBKK_Add")]
        [Authorize]
        public IHttpActionResult SNSE195_CustomerBKK_Add([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE195_CUS_LABEL_ADD";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"].ToString();
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"].ToString();
            cmd.Parameters.Add("P_LABEL_NAME", OracleDbType.Varchar2).Value = data["LABEL_NAME"].ToString();
            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = data["ROUTE_CODE"].ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Ok("Done");
        }

        [HttpPost, Route("SNSE195_GetCustomerLabel_Export")]
        [Authorize]
        public HttpResponseMessage SNSE195_GetCustomerLabel_Export([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            string xxx = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE195_EXP_CUS_BKK";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            dt = ds.Tables[0];

            var json = JsonConvert.SerializeObject(dt);
            var List1 = JsonConvert.DeserializeObject<List<DataModel.SNSE195>>(json);
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE195" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE195_CustomerLabel(BKK).xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE195_CustomerLabel(BKK).xlsx");
            var excel2 = List1.ToExcel(
            x => x
            .AddColumn("SEQ", z => z.SEQ)
            .AddColumn("CUSTOMER_CODE", z => z.CUSTOMER_CODE)
            .AddColumn("BRANCH_CODE", z => z.BRANCH_CODE)
            .AddColumn("LABEL_NAME", z => z.LABEL_NAME)
            .AddColumn("ROUTE_CODE", z => z.ROUTE_CODE)
            );
            byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE195_CustomerLabel(BKK).xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE195_CustomerLabel(BKK).xlsx";
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE195_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;

            //string x = SNSE195_CustomerLabelBKK_Export(ds.Tables[0], "SNSE195_CustomerLabel(BKK).xlsx", "SNSE195");
            //SystemApiController sys = new SystemApiController();
            //var respons = sys.downloadFile(x.Substring(1), "SNSE195_CustomerLabel.xlsx");
            //return respons;

        }

        public static string SNSE195_CustomerLabelBKK_Export(DataTable dt, string fileName, string FloderLocation)
        {

            bool Succes = false;
            //DataTable dt = new DataTable();
            //dt = ds.Tables[0];
            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                IEnumerable<string> columnNames = dt.Columns.Cast<DataColumn>().Select(column => column.ColumnName);
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {

                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }

                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }

        }

        #endregion


        #region SNPR029_M

        [HttpGet, Route("SNPR029_M_GetPart/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPR029_M_GetPart(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPR029_GETPART";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNPR029_M_Branch/{CustomerCode?}")]
        [Authorize]
        public IHttpActionResult SNPR029_M_Branch(string CustomerCode = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPR029_M_GET_BRANCH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        public IHttpActionResult SNPR029_M_DELETE()
        {
            var row = 0;
            string sql = "DELETE FROM TPM_SEQ_SNPR029_M";
            row = oOracledb.SqlExecute(sql);
            return Ok(row);
        }

        [HttpPost, Route("SNPR029_M_INSERT")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPR029_M_INSERT([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            SNPR029_M_DELETE();
            foreach (var data in pDataList)
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPR029_M_INSERT";
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data["PART_NO"].ToString();
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = data["QTY"].ToString();
                cmd.Parameters.Add("P_PRN", OracleDbType.Varchar2).Value = data["PRN"].ToString();
                cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = data["REMARK"].ToString();
                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            return Ok("DONE");
        }
        #endregion

        #region SNM102P


        [HttpPost, Route("SNM102P_UploadandGet")]
        [Authorize]
        public async Task<IHttpActionResult> SNM102P_UploadandGet(HttpRequestMessage request)
        {
            try




            {





                if (!request.Content.IsMimeMultipartContent())
                {
                    throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
                }

                var reqParts = await request.Content.ReadAsMultipartAsync();

                string FullfileName = String.Concat("SNM102P", "-", DateTime.Now.ToString("yyyyMMddHHmm"), ".CSV");
                string outputLocation = "~/tempFile/" + "SNM102P" + "/";
                string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                string fileLocation = String.Concat(outputPath, FullfileName);

                foreach (var file in reqParts.Contents)
                {

                    var buffer = await file.ReadAsByteArrayAsync();


                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM102P_DELETE_TEMP";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlQueryReturnValue(cmd);
                string Header = "PART_NO,PART_NAME,STICKER_PACKAGE,NET_WEIGHT,DIMENSION_MAX,DIMENSION_MID,DIMENSION_MIN";
                DataTable dt = DataService.ConvertCSVtoDataTable(fileLocation, Header);
                int count = 0;
                foreach (DataRow data in dt.Rows)
                {
                    count++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM102P_INSERT_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data["PART_NO"].ToString();
                    cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = data["PART_NAME"].ToString();
                    cmd.Parameters.Add("P_STICKER_PACKAGE", OracleDbType.Varchar2).Value = data["STICKER_PACKAGE"].ToString();
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = data["NET_WEIGHT"].ToString();
                    cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = data["DIMENSION_MAX"].ToString();
                    cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = data["DIMENSION_MID"].ToString();
                    cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = data["DIMENSION_MIN"].ToString();
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
                DataSet ds = SNM102P_ValidateAndGet();
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM102P_UploadandGet");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
        }

        [HttpGet, Route("SNM102P_ValidateAndGet")]
        public DataSet SNM102P_ValidateAndGet()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "sp_NP_SNM102P_VALIDATE_AND_GET";
            cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return ds;
        }


        [HttpPost, Route("SNM102P_Update")]
        [Authorize]
        public IHttpActionResult SNM102P_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;

            try
            {
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM102P_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data["PART_NO"];
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = data["NET_WEIGHT"];
                    cmd.Parameters.Add("P_STICKER_PACKAGE", OracleDbType.Varchar2).Value = data["STICKER_PACKAGE"];
                    cmd.Parameters.Add("P_PART_SIZE_MAX", OracleDbType.Varchar2).Value = data["PART_SIZE_MAX"];
                    cmd.Parameters.Add("P_PART_SIZE_MID", OracleDbType.Varchar2).Value = data["PART_SIZE_MID"];
                    cmd.Parameters.Add("P_PART_SIZE_MIN", OracleDbType.Varchar2).Value = data["PART_SIZE_MIN"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Int16).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM102P";
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM102P_UPDATE_PART_MASTER";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Int16).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM102P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM102P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }

        }
        #endregion



        #region SNSE202

        [HttpGet, Route("SNSE202_Get_Sugguestion_Header/{processDate}")]
        [Authorize]
        public IHttpActionResult SNSE202_Get_Sugguestion_Header(string processDate)
        {
            DataSet ds = new DataSet();

            processDate = ismoUtil.escapeCharacter(processDate);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "sp_np_snse202_get_sugg_header";
            cmd.Parameters.Add("P_PROCESSDATE", OracleDbType.Varchar2).Value = processDate != "@@@" ? processDate : null;
            cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE202_Get_Sugguestion_Part/{processDate}")]
        [Authorize]
        public IHttpActionResult SNSE202_Get_Sugguestion_Part(string processDate)
        {
            DataSet ds = new DataSet();


            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "sp_np_snse202_get_sugg_part";
            cmd.Parameters.Add("P_PROCESSDATE", OracleDbType.Date).Value = Convert.ToDateTime(processDate);
            cmd.Parameters.Add("cSugguestionList", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("cPartExclue", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE202_Retrive_Incoming_Part")]
        [Authorize]
        public IHttpActionResult SNSE202_Retrive_Incoming_Part([FromBody] dynamic pData)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE202_RETR_INCOM_PARTS";
            cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData.PROCESS_DATE);
            cmd.Parameters.Add("P_RANGE_NEW_PART", OracleDbType.Varchar2).Value = pData.RANGE_NEW_PART;
            cmd.Parameters.Add("P_RANGE_ETD_INCOMING", OracleDbType.Varchar2).Value = pData.RANGE_ETD_INCOMING;
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE202_Confirm_Factor")]
        [Authorize]
        public IHttpActionResult SNSE202_Confirm_Factor([FromBody] dynamic pData)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE202_CONFIRM_FACTOR";
            cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData.PROCESS_DATE);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE202_Parts_Move")]
        [Authorize]
        public IHttpActionResult SNSE202_Parts_Move([FromBody] dynamic[] pData)
        {
            int ErrCount = 0;
            DataSet ds = new DataSet();
            foreach (var item in pData)
            {
                if (item.IS_CHECK != null)
                {
                    if (item.IS_CHECK.Value)
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE202_PART_MOVE";
                        cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PROCESS_DATE);
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO;
                        cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
                        cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                        ds = oOracledb.SqlQueryDS_CMD(cmd);
                        if (ds != null & ds.Tables.Count > 0 & ds.Tables[0].Rows.Count > 0)
                        {
                            if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                            {
                                ErrCount++;
                            }
                        }
                    }
                }
            }
            if (ErrCount == 0)
            {
                return Json("DONE");
            }
            else
            {
                return Json("Error : " + ErrCount + " Record");
            }
        }

        [HttpPost, Route("SNSE202_Update_Dimension")]
        [Authorize]
        public IHttpActionResult SNSE202_Update_Dimension([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE202_UPDATE_DIMENSION";
            cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData.PROCESS_DATE);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("Process_Suggestion")]
        [Authorize]
        public IHttpActionResult Process_Suggestion([FromBody] dynamic[] pData)
        {
            String CatList = "";
            foreach (var item in pData)
            {
                if (string.IsNullOrEmpty(Convert.ToString(item.ATTRIBUTE1)))
                {
                    if (item.ESTIMATE_CATEGORY == "1" || item.ESTIMATE_CATEGORY == "2" || item.ESTIMATE_CATEGORY == "3")
                    {
                        CatList = CatList.Contains("F") ? CatList : CatList + "F,";
                        continue;
                    }
                    if (item.ESTIMATE_CATEGORY == "4" || item.ESTIMATE_CATEGORY == "5")
                    {
                        CatList = CatList.Contains("M") ? CatList : CatList + "M,";
                        continue;
                    }
                    CatList = CatList.Contains("S") ? CatList : CatList + "S,"; ;
                }
            }
            if (CatList.Length > 0)
            {
                CatList = CatList.Substring(0, CatList.Length - 1);
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE202_PROCESS_SUGGEST";
                cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CATAEGORY_LIST", OracleDbType.Varchar2).Value = CatList;
                cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData[0].PROCESS_DATE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            else
            {
                return Json(new { STATUS = "DONE" });
            }


        }
        [HttpPost, Route("SNSE202_Export")]
        public HttpResponseMessage SNSE202_Export(List<DataModel.SNSE202> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSE202.csv";
            string floderLocation = "SNSE202";
            //dt = DTFM.ToDatatable<DataModel.SNPE044>(dModel);


            dt.Columns.Add("Process Date");
            dt.Columns.Add("Part No.");
            dt.Columns.Add("Part Name");
            dt.Columns.Add("Incoming Q'ty");
            dt.Columns.Add("Group Average Demand");
            dt.Columns.Add("Estimate Stock");
            dt.Columns.Add("Estimate Category");
            dt.Columns.Add("Suggest Location No.");
            dt.Columns.Add("Message");

            for (var i = 0; i < dModel.Count; i++)
            {
                DataRow dr = dt.NewRow();
                dr[0] = dModel[i].PROCESS_DATE;
                dr[1] = dModel[i].PART_NO;
                dr[2] = dModel[i].PART_NO;
                dr[3] = dModel[i].INCOMING_QTY;
                dr[4] = dModel[i].AVERAGE_DEMAND;
                dr[5] = dModel[i].ESTIMATE_STOCK;
                dr[6] = dModel[i].ESTIMATE_CATEGORY;
                dr[7] = dModel[i].LOCATION_NO;
                dr[8] = dModel[i].ATTRIBUTE1;
                dt.Rows.Add(dr);

            }

            dt.Rows.Add("E");

            x = DataService.WriteCSV(dt, filename, floderLocation, "SNSE202");
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }
        [HttpPost, Route("ConFirm_Location")]
        [Authorize]
        public IHttpActionResult ConFirm_Location([FromBody] dynamic[] pData)
        {
            String PartList = "";
            foreach (var item in pData)
            {
                if (Convert.ToBoolean(item.IS_CHECK))
                {
                    PartList = PartList + item.PART_NO + ",";
                }
            }
            if (PartList.Length > 0)
            {
                PartList = PartList.Substring(0, PartList.Length - 1);
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE202_UPDATE_LOCATION";
                cmd.Parameters.Add("P_COM_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData[0].PROCESS_DATE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE202";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PART_LIST", OracleDbType.Varchar2).Value = PartList;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            else
            {
                return Json(new { STATUS = "NO_CHECK" });
            }


        }
        #endregion

        #region SNSE064_M2

        [HttpGet, Route("SNSE064_M2_REF_RO_NO/{returnRequestNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE064_M2_REF_RO_NO(string returnRequestNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            returnRequestNo = ismoUtil.escapeCharacter(returnRequestNo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE064_M2_REF_RO_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_REF_RO_NO", OracleDbType.Varchar2).Value = returnRequestNo.ToUpper() == "NULL" ? (object)DBNull.Value : returnRequestNo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE064_M2_Search")]
        [Authorize]
        public IHttpActionResult SNSE064_M2_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE064_M2_QUERY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_REF_RO_NO", OracleDbType.Varchar2).Value = param.REF_RO_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_REQUEST_DATE", OracleDbType.Varchar2).Value = param.REQUEST_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE064_M2_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE064_M2_GetDetail/{returnRequestNo}")]
        [Authorize]
        public IHttpActionResult SNSE064_M2_GetDetail(string returnRequestNo)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE064_M2_GET_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_REF_RO_NO", OracleDbType.Varchar2).Value = returnRequestNo;
            cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion  1

        #region SNPE034

        [HttpGet, Route("SNPE034_GetDocList/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE034_GetDocList(string docno, int noofshowrec)
        {
            DataSet ds = new DataSet();

            docno = ismoUtil.escapeCharacter(docno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_DOC_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = docno.ToUpper() == "NULL" ? (object)DBNull.Value : docno;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNPE034_GetInvList/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE034_GetInvList(string invoiceno, int noofshowrec)
        {
            DataSet ds = new DataSet();

            invoiceno = ismoUtil.escapeCharacter(invoiceno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_INV_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNPE034_GetDocumentList")]
        [Authorize]
        public IHttpActionResult SNPE034_GetDocumentList([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_DOCUMENTS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DocumentNo;
            cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = pData.DocumentDate != null ? Convert.ToDateTime(pData.DocumentDate) : (Object)DBNull.Value;
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.InvoiceNo;
            cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData.InvoiceDate != null ? Convert.ToDateTime(pData.InvoiceDate) : (Object)DBNull.Value;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNPE034_CheckDocument/{docno}")]
        [Authorize]
        public IHttpActionResult SNPE034_CheckDocument(string docno)
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_CHECK_DOCUMENT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = docno;
            cmd.Parameters.Add("C_RETURN_HEAD", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        [HttpPost, Route("SNPE034_CheckDocument")]
        [Authorize]
        public IHttpActionResult SNPE034_CheckDocument([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_CHECK_DOC_BYINV";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VendorCode;
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.InvoiceNo;
            cmd.Parameters.Add("C_RETURN_HEAD", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNPE034_GetBasePrice/{partno}")]
        [Authorize]
        public IHttpActionResult SNPE034_GetBasePrice(string partno)
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE034_GET_BASE_PRICE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partno;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE034_SaveDocument")]
        [Authorize]
        public IHttpActionResult SNPE034_SaveDocument([FromBody] dynamic[] pDetail)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            if (pDetail.Length > 0)
            {

                //Run insert detail
                foreach (var detail in pDetail)
                {


                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE034_SAVE_DETAIL_TEMP";
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pDetail[0]["INVOICE_NO"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDetail[0]["VENDOR_CODE"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = detail["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_QUANTITY", OracleDbType.Double).Value = detail["ITEM_QTY"];
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = detail["UNIT_PRICE"];

                    cmd.Parameters.Add("P_EXP_CLEANRANCE_AMT_B", OracleDbType.Double).Value = detail["EXP_CLEANRANCE_AMT_B"];
                    cmd.Parameters.Add("P_EXP_OTHER_AMT_B", OracleDbType.Double).Value = detail["EXP_OTHER_AMT_B"];
                    cmd.Parameters.Add("P_INCOME_TAX", OracleDbType.Double).Value = detail["INCOME_TAX"];
                    cmd.Parameters.Add("P_EXCISE_TAX", OracleDbType.Double).Value = detail["EXCISE"];
                    cmd.Parameters.Add("P_INTERIOR_TAX", OracleDbType.Double).Value = detail["INTERIOR"];
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Double).Value = detail["DUTY_RATE"];



                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE034_SAVE_TRANSACTION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pDetail[0]["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = DateTime.Parse(pDetail[0]["INVOICE_DATE"].ToString());
                cmd.Parameters.Add("P_BL_AWB_DATE", OracleDbType.Date).Value = DateTime.Parse(pDetail[0]["BL_AWB_DATE"].ToString());


                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDetail[0]["VENDOR_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pDetail[0]["SENT_BY"];
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDetail[0]["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDetail[0]["ORDER_TYPE"];

                cmd.Parameters.Add("P_EXP_CLEANRANCE_AMT_B", OracleDbType.Double).Value = pDetail[0]["TOTAL_EXP_CLEANRANCE_AMT_B"];
                cmd.Parameters.Add("P_EXP_OTHER_AMT_B", OracleDbType.Double).Value = pDetail[0]["TOTAL_EXP_OTHER_AMT_B"];
                cmd.Parameters.Add("P_INCOME_TAX", OracleDbType.Double).Value = pDetail[0]["TOTAL_INCOME_TAX"];

                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                return Json(new { DOCUMENT_NO = documentNo });
            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }

        [HttpPost, Route("SNPE034_DeleteDocument")]
        [Authorize]
        public IHttpActionResult SNPE034_DeleteDocument([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNPE034_DELETE_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DocumentNo;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VendorCode;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.InvoiceNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Json("Done");
            }
            catch (OracleException ex)
            {
                string err_status = string.Empty;

                if (ex.Number == 20001)
                {
                    err_status = "INVOICE_ERROR";
                }
                else if (ex.Number == 20002)
                {
                    err_status = "CONFIRMED";
                }
                else
                {
                    err_status = "CONTACT";
                }

                return Ok(err_status);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }
        }
        #endregion



        #region SNM110P
        [HttpGet, Route("SNM110P_GetCategory/{mainCategory}")]
        [Authorize]
        public IHttpActionResult SNM110P_GetCategory(string mainCategory)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM110P_GET_CATEGORY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_CATEGORY", OracleDbType.Varchar2).Value = mainCategory.ToUpper() == "NULL" ? (object)DBNull.Value : mainCategory;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM110P_GetOverlapLocation/{mainCategory}/{subCategory}/{fromLocation}/{toLocation}")]
        [Authorize]
        public IHttpActionResult SNM110P_GetOverlapLocation(string mainCategory, string subCategory, string fromLocation, string toLocation)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM110P_OVERLAP_LOCATION";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_CATEGORY", OracleDbType.Varchar2).Value = mainCategory;
            cmd.Parameters.Add("P_SUB_CATEGORY", OracleDbType.Varchar2).Value = subCategory;
            cmd.Parameters.Add("P_FROM_LOCATION", OracleDbType.Varchar2).Value = fromLocation;
            cmd.Parameters.Add("P_TO_LOCATION", OracleDbType.Varchar2).Value = toLocation;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM110P_AddCategoryLocation/{mainCategory}/{subCategory}/{fromLocation}/{toLocation}")]
        [Authorize]
        public IHttpActionResult SNM110P_AddCategoryLocation(string mainCategory, string subCategory, string fromLocation, string toLocation)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM110P_ADD";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_CATEGORY", OracleDbType.Varchar2).Value = mainCategory;
            cmd.Parameters.Add("P_SUB_CATEGORY", OracleDbType.Varchar2).Value = subCategory;
            cmd.Parameters.Add("P_FROM_LOCATION", OracleDbType.Varchar2).Value = fromLocation;
            cmd.Parameters.Add("P_TO_LOCATION", OracleDbType.Varchar2).Value = toLocation;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM110P_DeleteCategoryLocation/{mainCategory}/{subCategory}/{fromLocation}/{toLocation}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM110P_DeleteCategoryLocation(string mainCategory, string subCategory, string fromLocation, string toLocation)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM110P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_CATEGORY", OracleDbType.Varchar2).Value = mainCategory;
            cmd.Parameters.Add("P_SUB_CATEGORY", OracleDbType.Varchar2).Value = subCategory;
            cmd.Parameters.Add("P_FROM_LOCATION", OracleDbType.Varchar2).Value = fromLocation;
            cmd.Parameters.Add("P_TO_LOCATION", OracleDbType.Varchar2).Value = toLocation;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNSD162

        [HttpGet, Route("SNSD162_GetPartNo/{part_no=null}/{noofshowrec= null}")]
        [Authorize]
        public IHttpActionResult SNSD162_GetPartNo(string part_no = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSD162_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part_no != "null" ? part_no : "";
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNSE181

        [HttpPost, Route("SNSE181_UploadFiles")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE181_UploadFiles(HttpRequestMessage request)
        {
            DataSet ds = new DataSet();
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                    string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD181/Process/");
                    if (!Directory.Exists(gRootPath))
                    {
                        Directory.CreateDirectory(gRootPath);
                    }
                    string gFileLocation = System.IO.Path.Combine(gRootPath, gFileName);
                    File.WriteAllBytes(gFileLocation, buffer);
                    ds.Tables.Add();
                    ds.Tables[0].Columns.Add("SEQ");
                    ds.Tables[0].Columns.Add("PRINT_DATE");
                    ds.Tables[0].Columns.Add("PARTS_NO");
                    ds.Tables[0].Columns.Add("LOCATION_NO");
                    ds.Tables[0].Columns.Add("NEW_LOCATION");
                    ds.Tables[0].Columns.Add("LOC_DIV");
                    ds.Tables[0].Columns.Add("RELO_QTY");
                    ds.Tables[0].Columns.Add("PICK_QTY");
                    ds.Tables[0].Columns.Add("CASE_NO");
                    ds.Tables[0].Columns.Add("PICK_DATE");
                    ds.Tables[0].Columns.Add("PICK_HT_DATE");
                    ds.Tables[0].Columns.Add("PICK_HT_TIME");
                    ds.Tables[0].Columns.Add("SHELVE_QTY");
                    ds.Tables[0].Columns.Add("SHELVE_DATE");
                    ds.Tables[0].Columns.Add("SHELVE_HT_DATE");
                    ds.Tables[0].Columns.Add("SHELVE_HT_TIME");
                    ds.Tables[0].Columns.Add("STATUS_CODE");
                    string line;
                    System.IO.StreamReader fileRd = new System.IO.StreamReader(gFileLocation);
                    try
                    {
                        int rowCount = 0;
                        while ((line = fileRd.ReadLine()) != null)
                        {
                            if (rowCount > 0)
                            {
                                string[] temp = line.Split(',');

                                OracleCommand cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNSE181_SAVE_LOG";
                                cmd.Parameters.Add("P_SEQ_NO", OracleDbType.Varchar2).Value = temp[0];
                                cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = temp[1];
                                cmd.Parameters.Add("P_PARTS_NO", OracleDbType.Varchar2).Value = temp[2];
                                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = temp[3];
                                cmd.Parameters.Add("P_NEW_LOCATION", OracleDbType.Varchar2).Value = temp[4];
                                cmd.Parameters.Add("P_LOC_DIV", OracleDbType.Varchar2).Value = temp[5];
                                cmd.Parameters.Add("P_RELO_QTY", OracleDbType.Varchar2).Value = temp[6];
                                cmd.Parameters.Add("P_PICK_QTY", OracleDbType.Varchar2).Value = temp[7];
                                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = temp[8];
                                if (temp[9].ToString() != "")
                                {
                                    cmd.Parameters.Add("P_PICK_DATE", OracleDbType.Date).Value = Convert.ToDateTime(temp[9]);
                                }
                                else
                                {
                                    cmd.Parameters.Add("P_PICK_DATE", OracleDbType.Date).Value = DBNull.Value;
                                }
                                cmd.Parameters.Add("P_PICK_HT_DATE", OracleDbType.Varchar2).Value = temp[10];
                                cmd.Parameters.Add("P_PICK_HT_TIME", OracleDbType.Varchar2).Value = temp[11];
                                cmd.Parameters.Add("P_SHELVE_QTY", OracleDbType.Varchar2).Value = temp[12];
                                if (temp[13].ToString() != "")
                                {
                                    cmd.Parameters.Add("P_SHELVE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(temp[13]);// ;
                                }
                                else
                                {
                                    cmd.Parameters.Add("P_SHELVE_DATE", OracleDbType.Date).Value = DBNull.Value;
                                }
                                cmd.Parameters.Add("P_SHELVE_HT_DATE", OracleDbType.Varchar2).Value = temp[14];
                                cmd.Parameters.Add("P_SHELVE_HT_TIME", OracleDbType.Varchar2).Value = temp[15];
                                cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = temp[16];
                                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                                cmd.Parameters.Add("P_CREATE_PGM", OracleDbType.Varchar2).Value = "SNSE181";
                                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName.ToUpper().Replace(".CSV", "");
                                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                                DataSet dsRes = oOracledb.SqlQueryDS_CMD(cmd);

                                if (dsRes != null)
                                {
                                    if (dsRes.Tables.Count > 0 && dsRes.Tables[0].Rows.Count > 0)
                                    {
                                        DataRow dr = ds.Tables[0].NewRow();
                                        dr["SEQ"] = rowCount;
                                        dr["PRINT_DATE"] = temp[1];
                                        dr["PARTS_NO"] = temp[2];
                                        dr["LOCATION_NO"] = temp[3];
                                        dr["NEW_LOCATION"] = temp[4];
                                        dr["LOC_DIV"] = temp[5];
                                        dr["RELO_QTY"] = temp[6];
                                        dr["PICK_QTY"] = temp[7];
                                        dr["CASE_NO"] = temp[8];
                                        dr["PICK_DATE"] = temp[9];
                                        dr["PICK_HT_DATE"] = temp[10];
                                        dr["PICK_HT_TIME"] = temp[11];
                                        dr["SHELVE_QTY"] = temp[12];
                                        dr["SHELVE_DATE"] = temp[13];
                                        dr["SHELVE_HT_DATE"] = temp[14];
                                        dr["SHELVE_HT_TIME"] = temp[15];
                                        dr["STATUS_CODE"] = temp[16];
                                        ds.Tables[0].Rows.Add(dr);
                                    }
                                }
                            }

                            rowCount++;
                        }
                        fileRd.Close();
                    }
                    catch (Exception)
                    {
                        fileRd.Close();

                    }
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE181_UploadFiles");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE181_UpdateUpload")]
        [Authorize]
        public IHttpActionResult SNSE181_UpdateUpload([FromBody] dynamic[] pdata)
        {
            DataSet ds = new DataSet();
            string strFileName = pdata[0]["CREATE_FILE_NAME"];
            foreach (var item in pdata)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE181_SAVE_PART";
                cmd.Parameters.Add("P_SEQ_NO", OracleDbType.Varchar2).Value = item["SEQ"];
                cmd.Parameters.Add("P_PRINT_CODE", OracleDbType.Varchar2).Value = item["PRINT_DATE"];
                cmd.Parameters.Add("P_PARTS_NO", OracleDbType.Varchar2).Value = item["PARTS_NO"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = item["LOCATION_NO"];
                cmd.Parameters.Add("P_NEW_LOCATION", OracleDbType.Varchar2).Value = item["NEW_LOCATION"];
                cmd.Parameters.Add("P_LOC_DIV", OracleDbType.Varchar2).Value = item["LOC_DIV"];
                cmd.Parameters.Add("P_RELO_QTY", OracleDbType.Varchar2).Value = item["RELO_QTY"];
                cmd.Parameters.Add("P_PICK_QTY", OracleDbType.Varchar2).Value = item["PICK_QTY"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = item["CASE_NO"];
                if (item["PICK_DATE"].ToString() != "")
                {
                    cmd.Parameters.Add("P_PICK_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item["PICK_DATE"]);
                }
                else
                {
                    cmd.Parameters.Add("P_PICK_DATE", OracleDbType.Date).Value = DBNull.Value;
                }
                cmd.Parameters.Add("P_PICK_HT_DATE", OracleDbType.Varchar2).Value = item["PICK_HT_DATE"];
                cmd.Parameters.Add("P_PICK_HT_TIME", OracleDbType.Varchar2).Value = item["PICK_HT_TIME"];
                cmd.Parameters.Add("P_SHELVE_QTY", OracleDbType.Varchar2).Value = item["SHELVE_QTY"];
                if (item["SHELVE_DATE"].ToString() != "")
                {
                    cmd.Parameters.Add("P_SHELVE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item["SHELVE_DATE"]);
                }
                else
                {
                    cmd.Parameters.Add("P_SHELVE_DATE", OracleDbType.Date).Value = DBNull.Value;
                }
                cmd.Parameters.Add("P_SHELVE_HT_DATE", OracleDbType.Varchar2).Value = item["SHELVE_HT_DATE"];
                cmd.Parameters.Add("P_SHELVE_HT_TIME", OracleDbType.Varchar2).Value = item["SHELVE_HT_TIME"];
                cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = item["STATUS_CODE"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CREATE_PGM", OracleDbType.Varchar2).Value = "SNSE181";
                cmd.Parameters.Add("P_CREATE_FILE_NAME", OracleDbType.Varchar2).Value = item["CREATE_FILE_NAME"];
                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables[0].Rows[0][0].ToString() != "DONE")
                {
                    return Json(DTFM.convertToList(ds));
                }
            }

            if (ds.Tables[0].Rows[0][0].ToString() == "DONE")
            {
                string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD181/Process/");
                string gRootback = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD181/Backup/");
                if (!Directory.Exists(gRootback))
                {
                    Directory.CreateDirectory(gRootback);
                }
                if (File.Exists(gRootback + strFileName))
                    File.Delete(gRootback + strFileName);
                File.Move(gRootPath + strFileName, gRootback + strFileName.ToUpper().Replace(".CSV", "").Replace(".TSV", "") + DateTime.Now.ToString("yyyyMMddhhMMss") + ".CSV");
            }
            return Json(DTFM.convertToList(ds));
        }

        #endregion

        #region SNSE187

        [HttpGet, Route("SNSE187_GetLocation/{locationNo=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNSE187_GetLocation(string locationNo = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();

            locationNo = ismoUtil.escapeCharacter(locationNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE187_GET_LOC";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = locationNo == "null" ? "" : locationNo;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE187_GetPart/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE187_GetPart(string PartNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            ////locationFrom = ismoUtil.escapeCharacter(locationFrom);
            ////locationTo = ismoUtil.escapeCharacter(locationTo);
            PartNo = ismoUtil.escapeCharacter(PartNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE187_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = param.locationFrom == "null" ? "" : param.locationFrom;
            cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = param.locationTo == "null" ? "" : param.locationTo;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo == "null" ? "" : PartNo;
            cmd.Parameters.Add("P_SHOW_NUMBER", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNSE187_GetPrint/{locationFrom=null}/{locationTo=null}/{PartFrom=null}/{PartTo=null}/{PritNo=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNSE187_GetPrint(string locationFrom = null, string locationTo = null, string PartFrom = null, string PartTo = null, string PritNo = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();
            locationFrom = ismoUtil.escapeCharacter(locationFrom);
            locationTo = ismoUtil.escapeCharacter(locationTo);
            PartFrom = ismoUtil.escapeCharacter(PartFrom);
            PartTo = ismoUtil.escapeCharacter(PartTo);
            PritNo = ismoUtil.escapeCharacter(PritNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE187_GET_PRINT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = locationFrom == "null" ? "" : locationFrom;
            cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = locationTo == "null" ? "" : locationTo;
            cmd.Parameters.Add("P_PART_NO_FROM", OracleDbType.Varchar2).Value = PartFrom == "null" ? "" : PartFrom;
            cmd.Parameters.Add("P_PART_NO_TO", OracleDbType.Varchar2).Value = PartTo == "null" ? "" : PartTo;
            cmd.Parameters.Add("P_PRINT_NO", OracleDbType.Varchar2).Value = PritNo == "null" ? "" : PritNo;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE187_Search/{locationFrom=null}/{locationTo=null}/{PartFrom=null}/{PartTo=null}/{PrintFrom=null}/{PrintTo=null}")]
        [Authorize]
        public IHttpActionResult SNSE187_Search(string locationFrom, string locationTo, string PartFrom, string PartTo, string PrintFrom, string PrintTo)
        {
            DataSet ds = new DataSet();
            locationFrom = ismoUtil.escapeCharacter(locationFrom);
            locationTo = ismoUtil.escapeCharacter(locationTo);
            PartFrom = ismoUtil.escapeCharacter(PartFrom);
            PartTo = ismoUtil.escapeCharacter(PartTo);
            PrintFrom = ismoUtil.escapeCharacter(PrintFrom);
            PrintTo = ismoUtil.escapeCharacter(PrintTo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE187_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = locationFrom == "null" ? "" : locationFrom;
            cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = locationTo == "null" ? "" : locationTo;
            cmd.Parameters.Add("P_PART_NO_FROM", OracleDbType.Varchar2).Value = PartFrom == "null" ? "" : PartFrom;
            cmd.Parameters.Add("P_PART_NO_TO", OracleDbType.Varchar2).Value = PartTo == "null" ? "" : PartTo;
            cmd.Parameters.Add("P_PRINT_FROM", OracleDbType.Varchar2).Value = PrintFrom == "null" ? "" : PrintFrom;
            cmd.Parameters.Add("P_PRINT_TO", OracleDbType.Varchar2).Value = PrintTo == "null" ? "" : PrintTo;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNSE192

        [HttpPost, Route("SNSE192_UploadFiles")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE192_UploadFiles(HttpRequestMessage request)
        {
            DataSet ds = new DataSet();
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                    string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD192/Process/");
                    if (!Directory.Exists(gRootPath))
                    {
                        Directory.CreateDirectory(gRootPath);
                    }
                    string gFileLocation = System.IO.Path.Combine(gRootPath, gFileName);
                    File.WriteAllBytes(gFileLocation, buffer);
                    ds.Tables.Add();
                    ds.Tables[0].Columns.Add("SEQ");
                    ds.Tables[0].Columns.Add("STAFF_ID");
                    ds.Tables[0].Columns.Add("STAFF_NAME");
                    ds.Tables[0].Columns.Add("JOB_ID");
                    ds.Tables[0].Columns.Add("JOB_NAME");
                    ds.Tables[0].Columns.Add("REF_NO");
                    ds.Tables[0].Columns.Add("REF_NAME");
                    ds.Tables[0].Columns.Add("PROCESS_DATE");
                    ds.Tables[0].Columns.Add("STATUS_CODE");
                    ds.Tables[0].Columns.Add("ERROR_MESSAGE");
                    string line;
                    System.IO.StreamReader fileRd = new System.IO.StreamReader(gFileLocation);
                    try
                    {
                        int rowCount = 0;
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE192_CLEAR_HANDHELD";
                        cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                        DataSet dsc = oOracledb.SqlQueryDS_CMD(cmd);

                        while ((line = fileRd.ReadLine()) != null)
                        {
                            if (rowCount > 0)
                            {
                                string[] temp = line.Split(',');

                                cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNSE192_SAVE_HANDHELD";
                                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                                cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = temp[1];
                                cmd.Parameters.Add("P_PROCESS_ID", OracleDbType.Varchar2).Value = temp[3];
                                cmd.Parameters.Add("P_REF_NO", OracleDbType.Varchar2).Value = temp[5];
                                if (temp[7].ToString() != "")
                                {
                                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(temp[7]);
                                }
                                else
                                {
                                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = DBNull.Value;
                                }
                                cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = temp[8];
                                cmd.Parameters.Add("P_ERROR_MSG", OracleDbType.Varchar2).Value = temp[9];
                                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = temp[10];
                                cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = temp[11];
                                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                                cmd.Parameters.Add("P_CREATE_PGM", OracleDbType.Varchar2).Value = "SNSE192";
                                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                                DataSet dsRes = oOracledb.SqlQueryDS_CMD(cmd);

                                if (dsRes != null)
                                {
                                    if (dsRes.Tables.Count > 0 && dsRes.Tables[0].Rows.Count > 0)
                                    {
                                        DataRow dr = ds.Tables[0].NewRow();
                                        dr["SEQ"] = rowCount;
                                        dr["STAFF_ID"] = temp[1];
                                        dr["STAFF_NAME"] = temp[2];
                                        dr["JOB_ID"] = temp[3];
                                        dr["JOB_NAME"] = temp[4];
                                        dr["REF_NO"] = temp[5];
                                        dr["REF_NAME"] = temp[6];
                                        dr["PROCESS_DATE"] = temp[7];
                                        dr["STATUS_CODE"] = temp[8];
                                        dr["ERROR_MESSAGE"] = temp[9];
                                        ds.Tables[0].Rows.Add(dr);
                                    }
                                }
                            }

                            rowCount++;
                        }
                        fileRd.Close();
                    }
                    catch (Exception)
                    {
                        fileRd.Close();

                    }
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE192_UploadFiles");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE192_UpdateUpload")]
        [Authorize]
        public IHttpActionResult SNSE192_UpdateUpload([FromBody] dynamic[] pdata)
        {
            DataSet ds = new DataSet();
            string strFileName = pdata[0]["CREATE_FILE_NAME"];
            foreach (var item in pdata)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE192_SAVE_S_WOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = item["STAFF_ID"];
                cmd.Parameters.Add("P_PROCESS_ID", OracleDbType.Varchar2).Value = item["PROCESS_ID"];
                cmd.Parameters.Add("P_REF_NO", OracleDbType.Varchar2).Value = item["REF_NO"];
                if (item["PROCESS_DATE"].ToString() != "")
                {
                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item["PROCESS_DATE"]);
                }
                else
                {
                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = DBNull.Value;
                }
                cmd.Parameters.Add("P_STATUS_CODE", OracleDbType.Varchar2).Value = item["STATUS_CODE"];
                cmd.Parameters.Add("P_ERROR_MSG", OracleDbType.Varchar2).Value = item["ERROR_MSG"];
                cmd.Parameters.Add("P_PROCESS_TYPE", OracleDbType.Varchar2).Value = item["PROCESS_TYPE"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CREATE_PGM", OracleDbType.Varchar2).Value = "SNSE192";
                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables[0].Rows[0][0].ToString() != "DONE")
                {
                    return Json(DTFM.convertToList(ds));
                }
            }

            if (ds.Tables[0].Rows[0][0].ToString() == "DONE")
            {
                string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD192/Process/");
                string gRootback = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD192/Backup/");
                if (!Directory.Exists(gRootback))
                {
                    Directory.CreateDirectory(gRootback);
                }
                if (File.Exists(gRootback + strFileName))
                    File.Delete(gRootback + strFileName);
                File.Move(gRootPath + strFileName, gRootback + strFileName.ToUpper().Replace(".CSV", "").Replace(".TSV", "") + DateTime.Now.ToString("yyyyMMddhhMMss") + ".CSV");
            }
            return Json(DTFM.convertToList(ds));
        }

        #endregion

        #region SNSE193

        [HttpGet, Route("SNSE193_GetStaffList/{StaffId=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNSE193_GetStaffList(string StaffId = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();

            StaffId = ismoUtil.escapeCharacter(StaffId);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE193_GET_STAFF";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = StaffId == "null" ? "" : StaffId;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE193_SearchStaffList/{StaffId=null}")]
        [Authorize]
        public IHttpActionResult SNSE193_SearchStaffList(string StaffId = null)
        {
            DataSet ds = new DataSet();

            StaffId = ismoUtil.escapeCharacter(StaffId);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE193_SEARCH_STAFF";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = StaffId == "null" ? "" : StaffId;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion

        #region SNSE194

        [HttpPost, Route("SNSE194_UploadFiles")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE194_UploadFiles(HttpRequestMessage request)
        {
            DataSet ds = new DataSet();
            string gFileName = "";
            string gFileLocation = "";
            string line;
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                    string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD194/Process/");
                    if (!Directory.Exists(gRootPath))
                    {
                        Directory.CreateDirectory(gRootPath);
                    }
                    gFileLocation = System.IO.Path.Combine(gRootPath, gFileName);
                    File.WriteAllBytes(gFileLocation, buffer);

                    System.IO.StreamReader fileRd = new System.IO.StreamReader(gFileLocation);
                    try
                    {
                        int rowCount = 0;
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE194_CLEAR_HANDHELD";
                        cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                        DataSet dsc = oOracledb.SqlQueryDS_CMD(cmd);

                        while ((line = fileRd.ReadLine()) != null)
                        {
                            if (rowCount > 0)
                            {
                                string[] temp = line.Split(',');

                                cmd = new OracleCommand();
                                cmd.CommandText = "SP_NP_SNSE194_SAVE_HANDHELD";
                                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                                cmd.Parameters.Add("P_STAFF_ID", OracleDbType.Varchar2).Value = temp[1];
                                cmd.Parameters.Add("P_PROCESS_ID", OracleDbType.Varchar2).Value = temp[2];
                                cmd.Parameters.Add("P_REF_NO", OracleDbType.Varchar2).Value = temp[4];
                                if (temp[5].ToString() != "")
                                {
                                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(temp[5]);
                                }
                                else
                                {
                                    cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = DBNull.Value;
                                }
                                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                                cmd.Parameters.Add("P_CREATE_PGM", OracleDbType.Varchar2).Value = "SNSE194";
                                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                                DataSet dsRes = oOracledb.SqlQueryDS_CMD(cmd);
                            }

                            rowCount++;
                        }
                        fileRd.Close();
                    }
                    catch (Exception)
                    {
                        fileRd.Close();

                    }
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE194_UploadFiles");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            OracleCommand cmdr = new OracleCommand();
            cmdr.CommandText = "SP_NP_SNSE194_GET_HANDHELD";
            cmdr.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmdr.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
            cmdr.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            DataSet dsr = oOracledb.SqlQueryDS_CMD(cmdr);
            dsr.Tables[0].Columns.Add("SEQ");
            dsr.Tables[0].Columns.Add("JOB_NAME");
            int i = 0;
            foreach (DataRow dr in dsr.Tables[0].Rows)
            {
                i++;
                dr["SEQ"] = i;


                int rowCount = 0;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE194_CLEAR_HANDHELD";
                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet dsc = oOracledb.SqlQueryDS_CMD(cmd);
                System.IO.StreamReader fileRd = new System.IO.StreamReader(gFileLocation);
                try
                {

                    while ((line = fileRd.ReadLine()) != null)
                    {
                        if (rowCount > 0)
                        {
                            string[] temp = line.Split(',');
                            if (dr["STAFF_ID"].ToString() == temp[1] && dr["JOB_ID"].ToString() == temp[2] && dr["REF_NO"].ToString() == temp[4])
                            {
                                dr["JOB_NAME"] = temp[3];
                            }
                        }
                        rowCount++;
                    }
                }
                catch (Exception)
                {

                }
                finally
                {
                    fileRd.Close();
                }
            }
            return Json(DTFM.convertToList(dsr));
        }

        [HttpPost, Route("SNSE194_UpdateUpload")]
        [Authorize]
        public IHttpActionResult SNSE194_UpdateUpload([FromBody] dynamic[] pdata)
        {

            string strFileName = pdata[0]["CREATE_FILE_NAME"];


            string gRootPath = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD194/Process/");
            string gRootback = System.Web.HttpContext.Current.Server.MapPath("~/tempFile/SNSD194/Backup/");
            if (!Directory.Exists(gRootback))
            {
                Directory.CreateDirectory(gRootback);
            }
            if (File.Exists(gRootback + strFileName))
                File.Delete(gRootback + strFileName);
            if (System.IO.File.Exists(gRootPath + strFileName))
            {
                File.Move(gRootPath + strFileName, gRootback + strFileName.ToUpper().Replace(".CSV", "").Replace(".TSV", "") + DateTime.Now.ToString("yyyyMMddhhMMss") + ".CSV");
            }
            DataSet ds = new DataSet();
            ds.Tables.Add();
            ds.Tables[0].Columns.Add("STATUS");
            ds.Tables[0].Rows.Add(new Object[] { "DONE" });
            return Json(DTFM.convertToList(ds));
        }

        #endregion


        #region SNSE170

        [HttpPost, Route("SNSE170_GetData")]
        [Authorize]
        public IHttpActionResult SNSE170_GetData([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE170_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList["CUSTOMER_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE170_GetData");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE170_StandardWorking_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE170_StandardWorking_Save([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            var x = HttpContext.Current.Request.UserHostAddress.ToString();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE170_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList["CUSTOMER_CODE"];
            cmd.Parameters.Add("P_PICKING_CARD", OracleDbType.Varchar2).Value = pDataList["PICKING_CARD"];
            cmd.Parameters.Add("P_DEPARTURE_SEA", OracleDbType.Varchar2).Value = pDataList["DEPARTURE_SEA"];
            cmd.Parameters.Add("P_LEADTIME_SEA", OracleDbType.Varchar2).Value = pDataList["LEADTIME_SEA"];
            cmd.Parameters.Add("P_LEADTIME_AIR", OracleDbType.Varchar2).Value = pDataList["LEADTIME_AIR"];
            cmd.Parameters.Add("P_SHIPMENT_SEA", OracleDbType.Varchar2).Value = pDataList["SHIPMENT_SEA"];
            cmd.Parameters.Add("P_SHIPMENT_AIR", OracleDbType.Varchar2).Value = pDataList["SHIPMENT_AIR"];
            cmd.Parameters.Add("P_CDD_STATUS", OracleDbType.Varchar2).Value = pDataList["CDD_STATUS"];
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE170";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE170_StandardWorking_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE170_StandardWorking_Delete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            var x = HttpContext.Current.Request.UserHostAddress.ToString();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE170_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList["CUSTOMER_CODE"];
            cmd.Parameters.Add("P_PICKING_CARD", OracleDbType.Varchar2).Value = pDataList["PICKING_CARD"];
            cmd.Parameters.Add("P_DEPARTURE_SEA", OracleDbType.Varchar2).Value = pDataList["DEPARTURE_SEA"];
            cmd.Parameters.Add("P_LEADTIME_SEA", OracleDbType.Varchar2).Value = pDataList["LEADTIME_SEA"];
            cmd.Parameters.Add("P_LEADTIME_AIR", OracleDbType.Varchar2).Value = pDataList["LEADTIME_AIR"];
            cmd.Parameters.Add("P_SHIPMENT_SEA", OracleDbType.Varchar2).Value = pDataList["SHIPMENT_SEA"];
            cmd.Parameters.Add("P_SHIPMENT_AIR", OracleDbType.Varchar2).Value = pDataList["SHIPMENT_AIR"];
            cmd.Parameters.Add("P_CDD_STATUS", OracleDbType.Varchar2).Value = pDataList["CDD_STATUS"];
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE170";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("GetCustomerCodeSTD/{CustomerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetCustomerCodeSTD(string CustomerCode, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCode = ismoUtil.escapeCharacter(CustomerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE170_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCustomerCodeSTD");
                return Json(new { Status = "Error" });
            }
        }


        [HttpPost, Route("SNSE170_StandardWorking_GenDepartureDay")]
        [Authorize]
        public IHttpActionResult SNSE170_StandardWorking_GenDepartureDay([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE170_GEN_DEPARTURE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList["CUSTOMER_CODE"];
            cmd.Parameters.Add("P_TO_YEAR", OracleDbType.Varchar2).Value = pDataList["TO_YEAR"];
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE170";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion

        #region SNSE171

        [HttpPost, Route("SNSE171_GetPickingCard")]
        [Authorize]
        public IHttpActionResult SNSE171_GetPickingCard([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_GET_PICKING_CARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pParams.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pParams.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE171_GetPickingCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE171_GetData")]
        [Authorize]
        public IHttpActionResult SNSE171_GetData([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_CUSTOMER_PC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = (pDataList["CUSTOMER_CODE"] != null && pDataList["CUSTOMER_CODE"] == "null" ? "" : pDataList["CUSTOMER_CODE"]);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = (pDataList["PROCESS_YEAR"] != null && pDataList["PROCESS_YEAR"] == "null" ? "" : pDataList["PROCESS_YEAR"]);
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = (pDataList["PROCESS_MONTH"] != null && pDataList["PROCESS_MONTH"] == "null" ? "" : pDataList["PROCESS_MONTH"]);
                cmd.Parameters.Add("P_ISSUE_PICKING_CARD", OracleDbType.Varchar2).Value = (pDataList["ISSUE_PICKING_CARD"] != null && pDataList["ISSUE_PICKING_CARD"] == "null" ? "" : pDataList["ISSUE_PICKING_CARD"]);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE171_GetData");
                return Json(new { Status = "Error" });
            }
        }







        [HttpGet, Route("SNSE171_GetCalendar/{ProcessYear}/{ProcessMonth}")]
        [Authorize]
        public IHttpActionResult SNSE171_GetCalendar(string ProcessYear, string ProcessMonth)
        {
            try
            {
                DataSet ds = new DataSet();
                ProcessYear = ismoUtil.escapeCharacter(ProcessYear);
                ProcessMonth = ismoUtil.escapeCharacter(ProcessMonth);
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE171_DAY_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = ProcessYear;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = ProcessMonth;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE171_GetCalendar");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE171_GetCalendarWKP")]
        [Authorize]
        public IHttpActionResult SNSE171_GetCalendarWKP([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_CALENDAR_WKP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = (pDataList["CUSTOMER_CODE"]);
                cmd.Parameters.Add("P_MONTH", OracleDbType.Varchar2).Value = (pDataList["PROCESS_MONTH"]);
                cmd.Parameters.Add("P_YEAR", OracleDbType.Varchar2).Value = (pDataList["PROCESS_YEAR"]);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE171_GetCalendarWKP");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE171_PickingCard_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE171_PickingCard_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            foreach (var data in pDataList)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = data["PROCESS_YEAR"];
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = data["PROCESS_MONTH"];
                cmd.Parameters.Add("P_ISSUE_PICKING_CARD", OracleDbType.Varchar2).Value = data["ISSUE_PICKING_CARD"];
                cmd.Parameters.Add("P_WEEK1", OracleDbType.Varchar2).Value = data["WEEK1"];
                cmd.Parameters.Add("P_WEEK2", OracleDbType.Varchar2).Value = data["WEEK2"];
                cmd.Parameters.Add("P_WEEK3", OracleDbType.Varchar2).Value = data["WEEK3"];
                cmd.Parameters.Add("P_WEEK4", OracleDbType.Varchar2).Value = data["WEEK4"];
                cmd.Parameters.Add("P_WEEK5", OracleDbType.Varchar2).Value = data["WEEK5"];
                cmd.Parameters.Add("P_CIPCD", OracleDbType.Varchar2).Value = data["CIPCD"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE171";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSE171_CreateIssuePicking")]
        [Authorize]
        public IHttpActionResult SNSE171_CreateIssuePicking([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            foreach (var data in pDataList)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_GEN_PICKING_CARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = data["PROCESS_YEAR"];
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = data["PROCESS_MONTH"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE171";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE171_AddProcessYear")]
        [Authorize]
        public IHttpActionResult SNSE171_AddProcessYear([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE171_ADD_PROCESS_YEAR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pData["PROCESS_YEAR"];
                cmd.Parameters.Add("P_OLD_PROCESS_YEAR", OracleDbType.Varchar2).Value = pData["OLD_PROCESS_YEAR"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("Done");
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Ok("Unable to insert.");
                }
                else if (ex.Number == 20002)
                {
                    return Ok("Please setup picking card day.");
                }
                else if (ex.Number == 20003)
                {
                    return Ok("Process year already exists.");
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                throw (ex);
            }
        }
        #endregion


        #region SNSE157

        [HttpGet, Route("SNSE157_GetData")]
        [Authorize]
        public IHttpActionResult SNSE157_GetData()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE157_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE157_GetData");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE157_UpdateData")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE157_UpdateData([FromBody] dynamic pDataList)
        {

            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE157_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_WH_SHELVE", OracleDbType.Int64).Value = pDataList["WH_SHELVE"];
            cmd.Parameters.Add("P_WH_RECEIVING", OracleDbType.Int64).Value = pDataList["WH_RECEIVING"];
            cmd.Parameters.Add("P_WH_ALLOCATE", OracleDbType.Int64).Value = pDataList["WH_ALLOCATE"];
            cmd.Parameters.Add("P_WH_PICKING", OracleDbType.Int64).Value = pDataList["WH_PICKING"];
            cmd.Parameters.Add("P_TOTAL_WH", OracleDbType.Int64).Value = pDataList["TOTAL_WH"];
            cmd.Parameters.Add("P_EXP_RECEIVING", OracleDbType.Int64).Value = pDataList["EXP_RECEIVING"];
            cmd.Parameters.Add("P_EXP_PACKING", OracleDbType.Int64).Value = pDataList["EXP_PACKING"];
            cmd.Parameters.Add("P_EXP_PRE_INV", OracleDbType.Int64).Value = pDataList["EXP_PRE_INV"];
            cmd.Parameters.Add("P_EXP_BOOKING", OracleDbType.Int64).Value = pDataList["EXP_BOOKING"];
            cmd.Parameters.Add("P_EXP_INVOICE", OracleDbType.Int64).Value = pDataList["EXP_INVOICE"];
            cmd.Parameters.Add("P_EXP_LOADING", OracleDbType.Int64).Value = pDataList["EXP_LOADING"];
            cmd.Parameters.Add("P_TOTAL_EXP", OracleDbType.Int64).Value = pDataList["TOTAL_EXP"];
            cmd.Parameters.Add("P_TOTAL_ALL", OracleDbType.Int64).Value = pDataList["TOTAL_ALL"];
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE157";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        #endregion

        #region SNPE015_P


        [HttpPost, Route("SNPE015_P_GetVendorCode/{vendorcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE015_P_GetVendorCode(string vendorcode, int noofshowrec, [FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            vendorcode = ismoUtil.escapeCharacter(vendorcode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE015_P_VENDOR_CODE";
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorcode != null && vendorcode == "null" ? "" : vendorcode;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("P_INV_TYPE", OracleDbType.Varchar2).Value = pParams["INV_TYPE"];
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE015_P_GetInvList/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE015_P_GetInvList(string invoiceno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                invoiceno = ismoUtil.escapeCharacter(invoiceno.ToUpper());

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_GetInvList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNPE015_R_GetPre_ReceivingList/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE015_R_GetPre_ReceivingList(string docno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                docno = ismoUtil.escapeCharacter(docno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_R_GET_PRE_REV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = docno.ToUpper() == "NULL" ? (object)DBNull.Value : docno;
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetPre_ReceivingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNPE015_P_GetData")]
        [Authorize]
        public IHttpActionResult SNPE015_P_GetData([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_P_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"] != null ? Convert.ToDateTime(pData["INVOICE_DATE"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE015_P_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNPE015_P_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_P_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE015_P_CheckDuplicateInvoice")]
        [Authorize]
        public IHttpActionResult SNPE015_P_CheckDuplicateInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_P_CHECK_DUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"];

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Ok("Failed");
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_CheckDuplicateInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok("Done");
        }

        [HttpPost, Route("SNPE015_P_CheckPOByPartNo")]
        [Authorize]
        public IHttpActionResult SNPE015_P_CheckPOByPartNo([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_CHECK_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_CheckPOByPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_P_CheckWOPartNo")]
        [Authorize]
        public IHttpActionResult SNPE015_P_CheckWOPartNo([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_CHECK_WO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_CheckWOPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNPE015_P_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_P_Save([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            DataSet ds;
            DataTable dtHeader, dtDetail;
            string gGUID, gDocNo;

            List<dynamic> partList = new List<dynamic>(pData.PART_LIST);
            int partCount = 0;

            //Insert to pre-receive
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                foreach (var part in partList)
                {
                    partCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_P_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_RNO", OracleDbType.Decimal).Value = partCount;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_UNIT_TYPE", OracleDbType.Varchar2).Value = part["UNIT_TYPE"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Decimal).Value = part["QTY"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = part["PO_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = part["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = part["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_PO_CLASS", OracleDbType.Varchar2).Value = part["ORDER_CLASS"];
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = part["ORDER_TYPE"];
                    cmd.Parameters.Add("P_RECEIVE_PACKAGE", OracleDbType.Decimal).Value = part["RECEIVE_PACKAGE"];
                    cmd.Parameters.Add("P_PENDING_TYPE", OracleDbType.Varchar2).Value = part["PENDING_TYPE"];

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_RECEIVE_DATE", OracleDbType.Date).Value = pData["ATTRIBUTE1"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_GUID", OracleDbType.Varchar2, 32).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                gGUID = cmd.Parameters["R_GUID"].Value.ToString();
                gDocNo = cmd.Parameters["R_DOC_NO"].Value.ToString();

            }
            catch (Exception ex)
            {

                if (ex.Message.IndexOf("Found invoice") > 0)
                {
                    return Json(new { Result = "Duplicate invoice", DocNo = "" });
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_P_Save");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }


            }

            //Update key to pre-receive
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_GET_REC_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                dtHeader = ds.Tables[0];
                dtDetail = ds.Tables[1];

                ReceiveTokenService receiveToken = new ReceiveTokenService();
                string tokenH = receiveToken.GenerateKeyHeader(gGUID, dtHeader);

                //Update Header
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_H";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenH;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                //Update Detail
                foreach (DataRow dataDetail in dtDetail.Rows)
                {
                    string tokenD = receiveToken.GenerateKeyDetail(gGUID, dtHeader, dataDetail);

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_P_UPD_KEY_D";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = dataDetail["SEQ"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = dataDetail["PART_NO"];
                    cmd.Parameters.Add("P_KEY", OracleDbType.Varchar2).Value = tokenD;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(new { Result = "Done", DocNo = gDocNo });

        }

        [HttpPost, Route("SNPE015_P_Cancel")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_P_Cancel([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_P_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_Cancel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_P_PendingComplete")]
        [Authorize]
        public IHttpActionResult SNPE015_P_PendingComplete([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                dynamic pDataHeader = pData.header;
                List<dynamic> pPendingPO = new List<dynamic>(pData.pendingPO);

                foreach (var pendingPO in pPendingPO)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_P_PENDING_PO_UPD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDataHeader["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pDataHeader["INVOICE_NO"];
                    cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pDataHeader["PRE_RECEIVE_DOC_NO"];
                    cmd.Parameters.Add("P_TRAN_ITEM_NO", OracleDbType.Decimal).Value = pendingPO["TRAN_ITEM_NO"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = pendingPO["PO_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = pendingPO["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pendingPO["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_PENDING_COMPLETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDataHeader["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pDataHeader["INVOICE_NO"];
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pDataHeader["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20008) return Ok("Cannot found item.");
                if (ex.Number == 20009) return Ok("Cannot found PO.");
                if (ex.Number == 20010) return Ok("Unmatch Qty.");
                if (ex.Number == 20011) return Ok("Used PO.");

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_PendingComplete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNPE015_P_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                OracleCommand cmd = new OracleCommand();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNPE015_P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer, gUploadLication, gFileLocation);

                SystemApiController sys = new SystemApiController();
                dt_file = sys.ReadDataExcel(gFileLocation);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TEMP_NP_SNPE015_P_UPLOAD_PART_FILE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_VALIDATE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
                //return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        [HttpPost, Route("SNPE015_P_Confirm_Upload")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_P_Confirm_Upload([FromBody] dynamic pData)
        {

            OracleCommand cmd2;
            try
            {
                cmd2 = new OracleCommand();
                cmd2.CommandText = "SP_NP_SNPE015_P_DEL_UPLOAD_PART_TEMP";
                cmd2.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd2.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd2 = oOracledb.SqlQueryReturnValue(cmd2);


                int iCount = 0;
                OracleCommand cmd1 = new OracleCommand();

                foreach (var detail in pData)
                {
                    iCount++;

                    cmd1 = new OracleCommand();
                    cmd1.CommandText = "SP_NP_SNPE015_P_INSERT_UPLOAD_PART_TEMP";
                    cmd1.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd1.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_NO"];
                    cmd1.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = detail["QTY"];
                    cmd1.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd1.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd1.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd1.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd1 = oOracledb.SqlQueryReturnValue(cmd1);
                }
                DataSet ds = new DataSet();
                DataSet ds2 = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt2 = new DataTable();
                DataSet ds3 = new DataSet();
                DataSet ds4 = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_GET_UPLOAD_PART_TEMP";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);


                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_P_Confirm_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        #endregion

        #region SNPE015_R


        [HttpPost, Route("SNPE015_R_GetDocNoList")]
        [Authorize]
        public IHttpActionResult SNPE015_R_GetDocNoList([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_P_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"] != null ? Convert.ToDateTime(pData["INVOICE_DATE"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetDocNoList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE015_R_GetDetailByItem")]
        [Authorize]
        public IHttpActionResult SNPE015_R_GetDetailByItem([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_SCAN_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SEQ", OracleDbType.Decimal).Value = pData["SEQ"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetDetailByItem");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_R_WEB_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNPE015_R_WEB_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_WEB_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_I", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_WEB_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpGet, Route("SNPE015_R_GetInvList/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE015_R_GetInvList(string invoiceno, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                invoiceno = ismoUtil.escapeCharacter(invoiceno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_R_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetInvList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_R_GetData")]
        [Authorize]
        public IHttpActionResult SNPE015_R_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_R_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNPE015_R_SaveDocument")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_R_SaveDocument([FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_R_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                oOracledb.SqlQueryReturnValue(cmd);

                if (partList.Count > 0)
                {

                    //Run insert detail
                    foreach (var part in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPE015_R_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                        cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                        cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                        cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                        cmd.Parameters.Add("P_RECEIVE_FLAG", OracleDbType.Varchar2).Value = part["RECEIVE_FLAG"];
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_R_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_MANUAL_RECEIVE", OracleDbType.Varchar2).Value = pData["MANUAL_RECEIVE"] ?? "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                    return Json(new { Result = "Done", DocumentNo = documentNo });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_SaveDocument");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: items are not equally." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_SaveDocument");
                    return Json(new { Result = "Fail", Message = "Cannot confirm invoice: status code." });
                }
                else
                {
                    throw (ex);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_R_SaveDocument");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion


        #region SNPE015_S

        [HttpPost, Route("SNPE015_S_GetData")]
        [Authorize]
        public IHttpActionResult SNPE015_S_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNPE015_S_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNPE015_S_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_S_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_L", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE015_S_SaveDocument")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_S_SaveDocument([FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                List<dynamic> partList = new List<dynamic>(pData.PART_LIST);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_S_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                //Run insert detail
                foreach (var part in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_S_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = part["SEQ"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = part["QTY"];
                    cmd.Parameters.Add("P_SHELVE_FLAG", OracleDbType.Varchar2).Value = part["SHELVE_FLAG"];
                    cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = part["LOCATION"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_S_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new { Result = "Done" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_SaveDocument");
                return Json(new { Result = "Fail", Message = ex.Message });
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNPE015_S_GetReceivingList/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE015_S_GetReceivingList(string docno, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                docno = ismoUtil.escapeCharacter(docno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_S_GET_REV_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = docno.ToUpper() == "NULL" ? (object)DBNull.Value : docno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_S_GetReceivingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNPE015_W_GetData")]
        [Authorize]
        public IHttpActionResult SNPE015_W_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_W_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNPE015_W_GetData_Comfirmed")]
        [Authorize]
        public IHttpActionResult SNPE015_W_GetData_Comfirmed([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_W_GET_CONFIRM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_GetData_Comfirmed");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNPE015_W_GetData_OH")]
        [Authorize]
        public IHttpActionResult SNPE015_W_GetData_OH([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE015_W_GET_OH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_GetData_OH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE015_W_Cancel")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_W_Cancel([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_W_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Json(new { Result = "Done" });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Cancel");
                    return Json(new { Result = "Fail", Message = "Cannot cancel invoice: No receive data found." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Cancel");
                    return Json(new { Result = "Fail", Message = "Cannot cancel invoice: Invalid document type." });
                }
                else if (ex.Number == 20003)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Cancel");
                    return Json(new { Result = "Fail", Message = "Cannot cancel invoice: No document data found." });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Cancel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNPE015_W_ConfirmShelve")]
        [Authorize]
        [RunBatchGuard, DateControlGuard]
        public IHttpActionResult SNPE015_W_ConfirmShelve([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_W_CONFIRM_SHELVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);


                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                return Json(new { DOCUMENT_NO = documentNo });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: items are not equally." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: status code." });
                }
                else if (ex.Number == 20003)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: non-received." });
                }
                else if (ex.Number == 20004)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: PENDING." });
                }
                else
                {
                    throw (ex);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_ConfirmShelve");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        [HttpPost, Route("SNPE015_W_Update_PO")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_W_Update_PO([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            string gDocNo;

            List<dynamic> partList = new List<dynamic>(pData.PART_LIST);
            int partCount = 0;

            //Insert to pre-receive
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_P_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                foreach (var part in partList)
                {
                    partCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE015_P_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_RNO", OracleDbType.Decimal).Value = part["ITEM"]; // Note item mean current order
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_UNIT_TYPE", OracleDbType.Varchar2).Value = part["UNIT_TYPE"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Decimal).Value = part["QTY"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = part["PO_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = part["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = part["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_PO_CLASS", OracleDbType.Varchar2).Value = part["ORDER_CLASS"];
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = part["ORDER_TYPE"];

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_W_UPDATE_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_GUID", OracleDbType.Varchar2).Value = pData["GUID"];
                cmd.Parameters.Add("R_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                gDocNo = cmd.Parameters["R_DOC_NO"].Value.ToString();

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(new { Result = "Done", DocNo = gDocNo });

        }

        [HttpPost, Route("SNPE015_W_Update_Invoice")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_W_Update_Invoice([FromBody] dynamic pData)
        {
            OracleCommand cmd;

            //Insert to pre-receive
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_W_UPDATE_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("P_UPDATE_INVOICE_NO", OracleDbType.Varchar2).Value = pData["UPDATE_INVOICE_NO"];
                cmd.Parameters.Add("P_UPDATE_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["UPDATE_INVOICE_DATE"]);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_GUID", OracleDbType.Varchar2).Value = pData["GUID"];
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Update_Invoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(new { Result = "Done" });

        }


        [HttpPost, Route("SNPE015_W_Update_Invoice_Date")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE015_W_Update_Invoice_Date([FromBody] dynamic pData)
        {
            OracleCommand cmd;

            //Insert to pre-receive
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE015_W_UPD_INV_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("P_UPDATE_INVOICE_NO", OracleDbType.Varchar2).Value = pData["UPDATE_INVOICE_NO"];
                cmd.Parameters.Add("P_UPDATE_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["UPDATE_INVOICE_DATE"]);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_GUID", OracleDbType.Varchar2).Value = pData["GUID"];
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE015_W_Update_Invoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(new { Result = "Done" });

        }


        #endregion

        #region SNPE047
        [HttpPost, Route("SNPE047_GetList")]
        [Authorize]
        public IHttpActionResult SNPE047_GetList([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pParam["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_GetList");
                return Json(new { Status = "Error" });

            }
        }
        [HttpPost, Route("SNPE047_GetList_Confirmed")]
        [Authorize]
        public IHttpActionResult SNPE047_GetList_Confirmed([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pParam["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = "55";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_GetList");
                return Json(new { Status = "Error" });

            }
        }
        [HttpPost, Route("SNPE047_GetList_OH")]
        [Authorize]
        public IHttpActionResult SNPE047_GetList_OH([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pParam["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pParam["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pParam["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = "60";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_GetList");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047_GetDetail")]
        [Authorize]
        public IHttpActionResult SNPE047_GetDetail([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_GetDetail");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNPE047_Save")]
        [Authorize]
        [RunBatchGuard, DateControlGuard]
        public IHttpActionResult SNPE047_Save([FromBody] dynamic pData)
        {


            int iCount = 0;
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_DELETE_TEMP";
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];

                oOracledb.SqlQueryReturnValue(cmd);

                foreach (var item in pData.PART_LIST)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE047_SAVE_TEMP";
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = item["ORDER_NO"];
                    cmd.Parameters.Add("P_ORDER_ITEM_NO", OracleDbType.Int16).Value = item["ORDER_ITEM_NO"];
                    cmd.Parameters.Add("P_ORDER_SUB_ITEM_NO", OracleDbType.Varchar2).Value = item["ORDER_SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Double).Value = item["QTY"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                //Insert transaction
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE047_SAVE_TRANS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = (pData._INVOICE_DATE != null) ? Convert.ToDateTime(pData._INVOICE_DATE) : null;
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                    return Json(new { result = "Done", DOCUMENT_NO = documentNo });
                }
                catch (OracleException ex)
                {
                    if (ex.Number == 20001)
                    {
                        return Json(new { result = "Fail", MESSAGE = oOracledb.GetErrorMessage(ex) });
                    }

                    throw;
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_Save");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));
        }

        [HttpPost, Route("SNPE047_Cancel_Invoice")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNPE047_Cancel_Invoice([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_CANCEL_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData.PRE_RECEIVE_DOC_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                ds = oOracledb.SqlQueryDS_CMD(cmd);


                return Json(new { result = "Done" });
            }
            catch (OracleException ex)
            {

                if (ex.Number == 20001)
                {
                    return Json(new { result = "Fail", message = oOracledb.GetErrorMessage(ex) });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_Cancel_Invoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

        }

        [HttpPost, Route("SNPE047_Cancel")]
        [Authorize]
        [RunBatchGuard, DateControlGuard]
        public IHttpActionResult SNPE047_Cancel([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { result = "Done" });
            }
            catch (OracleException ex)
            {

                if (ex.Number == 20001)
                {
                    return Json(new { result = "Fail", message = oOracledb.GetErrorMessage(ex) });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_Cancel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

        }
        #endregion


        #region SNPE047P

        [HttpGet, Route("SNPE047P_GetDoc/{DocNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetDoc(string DocNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMEN_NO", OracleDbType.Varchar2).Value = DocNo != "null" ? DocNo : "";
                cmd.Parameters.Add("P_DOCUMEN_TYPE", OracleDbType.Varchar2).Value = "RI";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_GetDoc");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNPE047P_GetVendor/{VendorCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetVendor(string VendorCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                VendorCode = ismoUtil.escapeCharacter(VendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = VendorCode != "null" ? VendorCode : "";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_GetVendor");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047P_GetInv/{InvNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetInv(string InvNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                InvNo = ismoUtil.escapeCharacter(InvNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = InvNo != "null" ? InvNo : "";
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE != "null" ? pData.VENDOR_CODE : "";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_GetInv");
                return Json(new { Status = "Error" });

            }
        }

        //[HttpPost, Route("SNPE047P_GetInvP")]
        //[Authorize]
        //public IHttpActionResult SNPE047P_GetInvP([FromBody] dynamic pData)
        //{
        //    try
        //    {
        //        DataSet ds = new DataSet();

        //        OracleCommand cmd = new OracleCommand();
        //        cmd.CommandText = "SP_NP_SNPE047P_GET_INV";
        //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = pData.REF_INV_NO != "null" ? pData.REF_INV_NO : "";
        //        cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE != "null" ? pData.VENDOR_CODE : "";
        //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        ds = oOracledb.SqlQueryDS_CMD(cmd);
        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        return Json(new { Status = "Error" });

        //    }
        //}

        [HttpPost, Route("SNPE047P_GetList")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetList([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_DOC_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam["DOCUMENT_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_REF_INV_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_GetList");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047P_GetPart/{PartNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetPart(string PartNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = pData["REF_INV_NO"].Value;
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = (object)DBNull.Value;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo != "null" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_GetPart");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047P_GetPart_M/{PartNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetPart_M(string PartNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_PART_M";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo != "null" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_GetPart_M");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047P_GetPart_M2/{PartNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetPart_M2(string PartNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_PART_M2";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo != "null" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_GetPart_M2");
                return Json(new { Status = "Error" });

            }
        }


        [HttpGet, Route("SNPE047P_CheckPo/{vendor}/{invoice}/{part}")]
        [Authorize]
        public IHttpActionResult SNPE047P_CheckPo(string vendor, string invoice, string part)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_GET_PART_M";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoice;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047P_CheckPo");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNPE047P_GetDetail")]
        [Authorize]
        public IHttpActionResult SNPE047P_GetDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE047_P_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData["DOCUMENT_NO"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = "ALL";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_GetPart");
                return Json(new { Status = "Error" });

            }
        }



        [HttpPost, Route("SNPE047P_Save")]
        [Authorize]
        public IHttpActionResult SNPE047P_Save([FromBody] dynamic pData)
        {


            int iCount = 0;
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {
                foreach (var item in pData.PART_LIST)
                {
                    if (item.EDIT == "EDIT")
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPE047_P_SAVE_DET_TMP";
                        cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = item["PO_NO"];
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = item["ITEM_NO"];
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = item["SUB_ITEM_NO"];
                        cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = item["ACCEPT_PART_NO"];
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = item["ORDER_PART_NO"];
                        cmd.Parameters.Add("P_QUANTITY", OracleDbType.Double).Value = item["QUANTITY"];
                        cmd.Parameters.Add("P_REMARK_FLAG", OracleDbType.Varchar2).Value = item["REMARK_FLAG"] ?? "N";
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                }

                //Insert transaction
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE047_P_SAVE_TRANS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.REF_INV_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = (pData._REF_INV_DATE != null) ? Convert.ToDateTime(pData._REF_INV_DATE) : null;
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();
                    string status = cmd.Parameters["R_STATUS"].Value.ToString();

                    return Json(new { DOCUMENT_NO = documentNo, STATUS = status });
                }
                catch (OracleException ex)
                {
                    if (ex.Number == 20001)
                    {
                        return Json(new { STATUS = "Error", MESSAGE = oOracledb.GetErrorMessage(ex) });
                    }

                    throw;
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_Save");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));
        }

        [HttpPost, Route("SNPE047P_Cancel")]
        [Authorize]
        public IHttpActionResult SNPE047P_Cancel([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_P_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO != "null" ? pData.DOCUMENT_NO : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = pData.LAST_UPD_PGM != "null" ? pData.LAST_UPD_PGM : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                string status_text = ds.Tables[0].Rows[0]["STATUS"].ToString();
                return Ok(status_text);
            }
            catch (OracleException ex)
            {
                string err_status = string.Empty;

                if (ex.Number == 20002)
                {
                    err_status = "CONFIRMED";
                }
                else
                {
                    err_status = "CONTACT";
                }

                return Ok(err_status);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_P_Cancel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

        }


        #endregion

        #region SNPE047I

        [HttpGet, Route("SNPE047_I_GetDoc/{DocNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetDoc(string DocNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMEN_NO", OracleDbType.Varchar2).Value = DocNo != "null" ? DocNo : "";
                cmd.Parameters.Add("P_DOCUMEN_TYPE", OracleDbType.Varchar2).Value = "RI";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetDoc");
                return Json(new { Status = "Error" });
            }
        }
        [HttpGet, Route("SNPE047_I_GetVendor/{VendorCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetVendor(string VendorCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                VendorCode = ismoUtil.escapeCharacter(VendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = VendorCode != "null" ? VendorCode : "";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetVendor");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNPE047_I_GetInv/{InvNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetInv(string InvNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                InvNo = ismoUtil.escapeCharacter(InvNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = InvNo != "null" ? InvNo : "";
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetInv");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNPE047_I_GetDetail")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetDetail([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetInv");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNPE047_I_GetList")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetList([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_DOC_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam["DOCUMENT_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_REF_INV_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetList");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNPE047_I_GetPart/{InvNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047_I_GetPart(string InvNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                InvNo = ismoUtil.escapeCharacter(InvNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = pData["REF_INV_NO"].Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_GetPart");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNPE047_I_Save")]
        [Authorize]
        public IHttpActionResult SNPE047_I_Save([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_SAVE_TRANS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.REF_INV_NO;
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = (pData._REF_INV_DATE != null) ? Convert.ToDateTime(pData._REF_INV_DATE) : null;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("P_REMARK_FLAG", OracleDbType.Varchar2).Value = pData.REMARK_FLAG ?? "N";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();
                string status = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { DOCUMENT_NO = documentNo, STATUS = status });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { STATUS = "Error", MESSAGE = oOracledb.GetErrorMessage(ex) });
                }

                throw;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_I_Save");
                return Json(new { STATUS = "Error" });
            }
        }
        [HttpPost, Route("SNPE047_I_Cancel")]
        [Authorize]
        public IHttpActionResult SNPE047_I_Cancel([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_I_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO != "null" ? pData.DOCUMENT_NO : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = pData.LAST_UPD_PGM != "null" ? pData.LAST_UPD_PGM : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                string status_text = ds.Tables[0].Rows[0]["STATUS"].ToString();
                return Ok(status_text);
            }
            catch (OracleException ex)
            {
                string err_status = string.Empty;
                if (ex.Number == 20002)
                {
                    err_status = "CONFIRMED";
                }
                else
                {
                    err_status = "CONTACT";
                }
                return Ok(err_status);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }
        }
        #endregion

        #region SNPE047C
        [HttpGet, Route("SNPE047C_GetDoc/{DocNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetDoc(string DocNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMEN_NO", OracleDbType.Varchar2).Value = DocNo != "null" ? DocNo : "";
                cmd.Parameters.Add("P_DOCUMEN_TYPE", OracleDbType.Varchar2).Value = "RI";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047_C_GetDoc");
                return Json(new { Status = "Error" });
            }
        }
        [HttpGet, Route("SNPE047C_GetVendor/{VendorCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetVendor(string VendorCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                VendorCode = ismoUtil.escapeCharacter(VendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = VendorCode != "null" ? VendorCode : "";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetVendor");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNPE047C_GetInv/{InvNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetInv(string InvNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                InvNo = ismoUtil.escapeCharacter(InvNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = InvNo != "null" ? InvNo : "";
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE != "null" ? pData.VENDOR_CODE : "";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetInv");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNPE047C_GetList")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetList([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_DOC_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam["DOCUMENT_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE"];
                cmd.Parameters.Add("P_REF_INV_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetList");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNPE047C_GetCase/{CaseNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetCase(string CaseNo, string NoOfShowRecords, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                CaseNo = ismoUtil.escapeCharacter(CaseNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = pData["REF_INV_NO"].Value;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = CaseNo != "null" ? CaseNo : (object)DBNull.Value;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"].Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetCase");
                return Json(new { Status = "Error" });
            }
        }
        [HttpGet, Route("SNPE047C_GetCaseG/{documentNo}/{invoNo}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetCaseG(string documentNo, string invoNo)
        {
            try
            {
                DataSet ds = new DataSet();
                documentNo = ismoUtil.escapeCharacter(documentNo);
                invoNo = ismoUtil.escapeCharacter(invoNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_CASE_RES";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = invoNo;
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = documentNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetCaseG");
                return Json(new { Status = "Error" });
            }
        }
        [HttpGet, Route("SNPE047C_GetDiffCost/{invoice}/{case_no}/{vendor}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetDiffCost(string invoice, string case_no, string vendor)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_DIFF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = invoice;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = case_no;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetDiffCost");
                return Json(new { Status = "Error" });
            }
        }
        [HttpGet, Route("SNPE047C_GetPart/{InvNo}")]
        [Authorize]
        public IHttpActionResult SNPE047C_GetPart(string InvNo, string DocNo)
        {
            try
            {
                DataSet ds = new DataSet();
                InvNo = ismoUtil.escapeCharacter(InvNo);
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = InvNo;
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocNo;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = null;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_GetPart");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNPE047C_Save")]
        [Authorize]
        public IHttpActionResult SNPE047C_Save([FromBody] dynamic pParam)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();
            foreach (var detail in pParam.CASE_LIST)
            {
                if (detail.EDIT != null && detail.EDIT == "DEL")
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPE047_C_SAVE_CASE_TEMP";
                    cmd.Parameters.Add("P_ITEM_QTY", OracleDbType.Int32).Value = detail["ITEM_QTY"];
                    cmd.Parameters.Add("P_INV_NO", OracleDbType.Varchar2).Value = null;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = detail["CASE_NO"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = null;
                    cmd.Parameters.Add("P_REMARK_FLAG", OracleDbType.Varchar2).Value = detail["REMARK_FLAG"] ?? "N";
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
            }
            //Insert transaction
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_SAVE_TRANS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam.REF_INV_NO;
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = (pParam._REF_INV_DATE != null) ? Convert.ToDateTime(pParam._REF_INV_DATE) : null;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParam.VENDOR_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();
                string status = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { DOCUMENT_NO = documentNo, STATUS = status });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { STATUS = "Error", MESSAGE = oOracledb.GetErrorMessage(ex) });
                }

                throw;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNPE047C_Cancel")]
        [Authorize]
        public IHttpActionResult SNPE047C_Cancel([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE047_C_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO != "null" ? pData.DOCUMENT_NO : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = pData.LAST_UPD_PGM != "null" ? pData.LAST_UPD_PGM : DBNull.Value;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE047C_Cancel");
                return Json(new { Status = "Error" });
            }
        }
        #endregion

        #region SNPE046_

        [HttpPost, Route("SNPE046_P_GetInvList/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE046_P_GetInvList(string invoiceno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                invoiceno = ismoUtil.escapeCharacter(invoiceno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE046_P_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_P_GetInvList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNPE046_P_GetExistInvoice/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE046_P_GetExistInvoice(string invoiceno, string noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                invoiceno = ismoUtil.escapeCharacter(invoiceno);

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_P_GET_INVOICE_L";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pParams["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;

                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_P_GetInvList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE046_P_ValidateInvoice")]
        [Authorize]
        public IHttpActionResult SNPE046_P_ValidateInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE046_P_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 150).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();


                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_P_ValidateInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE046_P_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNPE046_P_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_P_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_P_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE046_R_GetData_Import")]
        [Authorize]
        public IHttpActionResult SNPE046_R_GetData_Import([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_R_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_R_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNPE046_P_GetData")]
        [Authorize]
        public IHttpActionResult SNPE046_P_GetData([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_P_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"] != null ? Convert.ToDateTime(pData["INVOICE_DATE"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE046_R_GetPre_ReceivingList/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE046_R_GetPre_ReceivingList(string docno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                docno = ismoUtil.escapeCharacter(docno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE046_R_GET_PRE_REV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = docno.ToUpper() == "NULL" ? (object)DBNull.Value : docno;
                cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_R_GetPre_ReceivingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNPE046_S_GetData")]
        [Authorize]
        public IHttpActionResult SNPE046_S_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_S_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_S_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE046_W_GetData")]
        [Authorize]
        public IHttpActionResult SNPE046_W_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNPE046_W_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_W_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNPE046_W_ConfirmShelve")]
        [Authorize]
        public IHttpActionResult SNPE046_W_ConfirmShelve([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE046_W_CONFIRM_SHELVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);


                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                return Json(new { DOCUMENT_NO = documentNo });

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: items are not equally." });
                }
                else if (ex.Number == 20002)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: status code." });
                }
                else if (ex.Number == 20003)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_W_ConfirmShelve");
                    return Json(new { Result = "Fail", Message = "Cannot shelve invoice: non-received." });
                }
                else
                {
                    throw (ex);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE046_W_ConfirmShelve");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        #endregion

        #region SNWR001
        [HttpPost, Route("SNWR001_InsertTemp")]
        [Authorize]
        public IHttpActionResult SNWR001_InsertTemp([FromBody] dynamic[] pDataList)
        {
            int iCount = 0;
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR001_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlQueryReturnValue(cmd);
                foreach (var data in pDataList)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR001_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data.LOCATION_NO;
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Int16).Value = iCount;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR001_InsertTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR001_GetLocation/{locationNo=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNWR001_GetLocation(string locationNo = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();

            locationNo = ismoUtil.escapeCharacter(locationNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR001_GET_LOC";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = locationNo == "null" ? "" : locationNo;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNWR001_GetPart/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR001_GetPart(string PartNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            ////locationFrom = ismoUtil.escapeCharacter(locationFrom);
            ////locationTo = ismoUtil.escapeCharacter(locationTo);
            PartNo = ismoUtil.escapeCharacter(PartNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR001_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = param.locationFrom == "null" ? "" : param.locationFrom;
            cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = param.locationTo == "null" ? "" : param.locationTo;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo == "null" ? "" : PartNo;
            cmd.Parameters.Add("P_SHOW_NUMBER", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNWR001_GetPrint/{locationFrom=null}/{locationTo=null}/{PartFrom=null}/{PartTo=null}/{PritNo=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNWR001_GetPrint(string locationFrom = null, string locationTo = null, string PartFrom = null, string PartTo = null, string PritNo = null, string noofshowrec = null)
        {
            DataSet ds = new DataSet();
            locationFrom = ismoUtil.escapeCharacter(locationFrom);
            locationTo = ismoUtil.escapeCharacter(locationTo);
            PartFrom = ismoUtil.escapeCharacter(PartFrom);
            PartTo = ismoUtil.escapeCharacter(PartTo);
            PritNo = ismoUtil.escapeCharacter(PritNo);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR001_GET_PRINT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = locationFrom == "null" ? "" : locationFrom;
            cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = locationTo == "null" ? "" : locationTo;
            cmd.Parameters.Add("P_PART_NO_FROM", OracleDbType.Varchar2).Value = PartFrom == "null" ? "" : PartFrom;
            cmd.Parameters.Add("P_PART_NO_TO", OracleDbType.Varchar2).Value = PartTo == "null" ? "" : PartTo;
            cmd.Parameters.Add("P_PRINT_NO", OracleDbType.Varchar2).Value = PritNo == "null" ? "" : PritNo;
            cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNWR001_Search/{locationFrom=null}/{locationTo=null}/{PartFrom=null}/{PartTo=null}")]
        [Authorize]
        public IHttpActionResult SNWR001_Search(string locationFrom, string locationTo, string PartFrom, string PartTo)
        {
            DataSet ds = new DataSet();

            try
            {
                locationFrom = ismoUtil.escapeCharacter(locationFrom);
                locationTo = ismoUtil.escapeCharacter(locationTo);
                PartFrom = ismoUtil.escapeCharacter(PartFrom);
                PartTo = ismoUtil.escapeCharacter(PartTo);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR001_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = locationFrom == "null" ? "" : locationFrom;
                cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = locationTo == "null" ? "" : locationTo;
                cmd.Parameters.Add("P_PART_NO_FROM", OracleDbType.Varchar2).Value = PartFrom == "null" ? "" : PartFrom;
                cmd.Parameters.Add("P_PART_NO_TO", OracleDbType.Varchar2).Value = PartTo == "null" ? "" : PartTo;

                cmd.Parameters.Add("c_return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR001_Search");
                return Json(new { Status = "Error" });
            }
        }


        #endregion

        #region SNWR002


        [HttpGet, Route("SNWR002_GetPartNonLocation/{partNo}")]
        [Authorize]
        public IHttpActionResult SNWR002_GetPartNonLocation(string partNo)
        {
            DataSet ds = new DataSet();
            try
            {
                partNo = ismoUtil.escapeCharacter(partNo);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_GET_PART_NOLO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_GetPartNonLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR002_GetPONonLocation")]
        [Authorize]
        public IHttpActionResult SNWR002_GetPONonLocation()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_GET_PO_NOLO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_GetPONonLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR002_GetIncomingPart")]
        [Authorize]
        public IHttpActionResult SNWR002_GetIncomingPart([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_GET_INCOM_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RANGE_NEW_PART", OracleDbType.Int16).Value = pDataList.COMING_IN_DAY.ToString();
                cmd.Parameters.Add("P_RANGE_ETD_INCOMING", OracleDbType.Int16).Value = pDataList.ETD_DAY.ToString();
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_GetIncomingPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR002_ProcessSuggest")]
        [Authorize]
        public IHttpActionResult SNWR002_ProcessSuggest([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_PROCESS_SUGGEST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                cmd.Parameters.Add("P_DIMENSION_MARGIN_RATE", OracleDbType.Varchar2).Value = pDataList.M3_MAGIN_RATE.ToString();
                cmd.Parameters.Add("P_DIMENSION_WEIGHT_RATE", OracleDbType.Varchar2).Value = pDataList.WEIGHT_MAGIN_RATE.ToString();

                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_ProcessSuggest");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        //[HttpGet, Route("SNWR002_GetLocationNonPart/{pPartNo=null}/{pPartCat=null}/{PartFrom=null}/{PartTo=null}")]
        //[Authorize]
        //public IHttpActionResult SNWR002_GetLocationNonPart(string pPartNo, string pPartCat, string PartFrom, string PartTo)
        //{
        //    DataSet ds = new DataSet();
        //    try
        //    {
        //        pPartNo = ismoUtil.escapeCharacter(pPartNo);
        //        pPartCat = ismoUtil.escapeCharacter(pPartCat);

        //        OracleCommand cmd = new OracleCommand();
        //        cmd.CommandText = "SP_NP_SNWR002_GET_LOC_NOPART";
        //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pPartNo == "null" ? "" : pPartNo;
        //        cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pPartCat == "null" ? "" : pPartCat;

        //        cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        cmd.Parameters.Add("C_RETURN4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        ds = oOracledb.SqlQueryDS_CMD(cmd);

        //        return Json(DTFM.convertToList(ds));

        //    }
        //    catch (Exception ex)
        //    {
        //        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_GetPartNonLocation");
        //        return Json(new { Status = "Error" });
        //    }
        //}


        [HttpPost, Route("SNWR002_GetLocationSuggest")]
        [Authorize]
        public IHttpActionResult SNWR002_GetLocationSuggest([FromBody] dynamic pDataList)


        //[HttpGet, Route("SNWR002_GetLocationSuggest/{pPartNo=null}/{pPartCat=null}/{pLocationNo=null}/{PartTo=null}")]
        //[Authorize]
        //public IHttpActionResult SNWR002_GetLocationSuggest(string pPartNo, string pPartCat, string pLocationNo, string PartTo)
        //
        {
            DataSet ds = new DataSet();
            try
            {


                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_GET_LOCATION_SUG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO == "null" ? (object)DBNull.Value : pDataList.PART_NO;
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY.ToString() == "null" ? (object)DBNull.Value : pDataList.PART_CATEGORY.ToString();
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO == "null" ? (object)DBNull.Value : pDataList.LOCATION_NO;
                //   PartNo != "null" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN5", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN6", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR002_GetLocationSuggest");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR002_UpdateLocation")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNWR002_UpdateLocation([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR002_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR002";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_UpdateLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNWR003
        [HttpPost, Route("SNWR003_GetPre_ReceivingList/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR003_GetPre_ReceivingList(string docno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                docno = ismoUtil.escapeCharacter(docno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR003_R_GET_PRE_REV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = docno.ToUpper() == "NULL" ? (object)DBNull.Value : docno;
                // cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR003_GetPre_ReceivingList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR003_P_GetInvList/{invoiceno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR003_P_GetInvList(string invoiceno, int noofshowrec, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                invoiceno = ismoUtil.escapeCharacter(invoiceno);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR003_P_GET_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceno.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceno;
                //  cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = pParams["INVOICE_STATUS"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR003_P_GetInvList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR003_GetData")]
        [Authorize]
        public IHttpActionResult SNWR003_GetData([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR003_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOC_DATE_FROM", OracleDbType.Date).Value = pData["DOCUMENT_DATE_FROM"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_FROM"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_DOC_DATE_TO", OracleDbType.Date).Value = pData["DOCUMENT_DATE_TO"] != null ? Convert.ToDateTime(pData["DOCUMENT_DATE_TO"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("P_PRE_RECEIVING_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVING_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR003_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }



        [HttpPost, Route("SNWR003_GetDataByPart")]
        [Authorize]
        public IHttpActionResult SNWR003_GetDataByPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR003_GET_LIST_BY_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR003_GetDataByPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNWR003_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNWR003_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR003_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_I", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR003_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpGet, Route("SNWR004_GetReLocationList")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationList()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR004_GetReLocationStatus")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationStatus()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_ST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR004_GetReLocationL1/{location?}")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationL1(string location = null)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_L1";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationL1");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR004_GetReLocationL2")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationL2()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_L2";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationL2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR004_GetReLocationL3")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationL3()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_L3";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationL3");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNWR004_GetReLocationL4")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationL4()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_L4";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationL4");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR004_GetReLocationL5")]
        [Authorize]
        public IHttpActionResult SNWR004_GetReLocationL5([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_RE_LOCATION_L5";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pParam.LOCATION_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetReLocationL5");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR004_GetLocationSuggest")]
        [Authorize]
        public IHttpActionResult SNWR004_GetLocationSuggest([FromBody] dynamic pDataList)
        {
            //    [HttpGet, Route("SNWR004_GetLocationSuggest/{pPartNo=null}/{pPartCat=null}/{pLocationNo=null}/{PartTo=null}")]
            //[Authorize]
            //public IHttpActionResult SNWR004_GetLocationSuggest(string pPartNo, string pPartCat, string pLocationNo, string PartTo)  
            DataSet ds = new DataSet();
            try
            {
                //pPartNo = ismoUtil.escapeCharacter(pPartNo);
                //pPartCat = ismoUtil.escapeCharacter(pPartCat);
                //pLocationNo = ismoUtil.escapeCharacter(pLocationNo);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_GET_LOCATION_SUG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList.PART_CATEGORY;
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO;

                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN5", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetLocationSuggest");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR004_ReLocation")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNWR004_ReLocation([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO_OLD.ToString();
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();

                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_UpdateLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR004_ConfirmReLocation")]
        [Authorize]
        public IHttpActionResult SNWR004_ConfirmReLocation([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_CONFIRM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO_OLD", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO_OLD.ToString();
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList.LOCATION_NO.ToString();

                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO.ToString();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR004";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM033P_UpdateLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNWR004_GetTempLocation/{location}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR004_GetTempLocation(string location, int noofshowrec)
        {
            DataSet ds = new DataSet();
            location = ismoUtil.escapeCharacter(location);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_GET_TEMP_LOCAT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = location.ToUpper() == "NULL" ? (object)DBNull.Value : location; ;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_GetTempLocation");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR004_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR004_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNWR004");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                SystemApiController sys = new SystemApiController();
                //dt = sys.ReadDataExcel(gFileLocation);
                dt = SNWR004_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }


        public DataTable SNWR004_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("MSG");
            dt.Columns.Add("ERR_FLAG");
            Hashtable hTable = new Hashtable();
            int iCount = 0;

            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["NEW_LOCATION"].ToString()))
                {
                    iCount++;

                    row["SEQ"] = iCount;
                    row["ERR_FLAG"] = "N";
                    if (row["NEW_LOCATION"].ToString().Length > 10)
                    {
                        row["MSG"] = "Location No cannot mare than 10 charactor";
                        row["ERR_FLAG"] = "Y";
                    }


                    if (hTable.Contains(row["NEW_LOCATION"].ToString().Trim()))
                    {
                        //row["SEQ"] = iCount;
                        row["MSG"] = "Duplicate Location No at Seq " + hTable[row["NEW_LOCATION"]];
                        row["ERR_FLAG"] = "Y";
                    }
                    else
                    {
                        hTable.Add(row["NEW_LOCATION"].ToString().Trim(), iCount);
                    }

                }
            }

            return dt;
        }


        [HttpPost, Route("SNWR004_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNWR004_SaveByUpload([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {
                var IP_ADDR = HttpContext.Current.Request.UserHostAddress.ToString();
                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR004_SAVE_TEMP";
                    cmd.Parameters.Add("P_FORMER_LOC", OracleDbType.Varchar2).Value = d["FORMER_LOCATION"];
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_NEW_LOC", OracleDbType.Varchar2).Value = d["NEW_LOCATION"];
                    cmd.Parameters.Add("P_TOTAL_OH", OracleDbType.Varchar2).Value = d["TOTAL_OH"];
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = d["STATUS"];
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = IP_ADDR;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR004_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = IP_ADDR;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNWR004_SaveByUpload");
                }

                return Json(new { STATUS = "DONE", MESSAGE = "COMPLETE" });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR004_SaveByUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }
        #endregion

        #region SNWR005

        [HttpPost, Route("SNWR005_GetPicking")]
        [Authorize]
        public IHttpActionResult SNWR005_GetPicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_GET_PICKING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData["ORDER_TYPE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GetPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNWR005_GeneratePicking")]
        [Authorize]
        public IHttpActionResult SNWR005_GeneratePicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_GEN_PICKING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.orderType;
                cmd.Parameters.Add("P_ZONE", OracleDbType.Varchar2).Value = pData.zone;
                if (pData.class4 == true)
                {
                    cmd.Parameters.Add("P_CLASS", OracleDbType.Varchar2).Value = "1";
                }
                else
                {
                    cmd.Parameters.Add("P_CLASS", OracleDbType.Varchar2).Value = "0";
                }

                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_PICKING_LIST_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_PICKING_LIST_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string result = result = cmd.Parameters["R_STATUS"].Value.ToString();
                string pickingListFrom = cmd.Parameters["R_PICKING_LIST_FROM"].Value.ToString();
                string pickingListTo = cmd.Parameters["R_PICKING_LIST_TO"].Value.ToString();

                return Ok(new { result = result, pickingListFrom = pickingListFrom, pickingListTo = pickingListTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GeneratePicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNWR005_GeneratePicking_kit")]
        [Authorize]
        public IHttpActionResult SNWR005_GeneratePicking_kit([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_GEN_PICKING_KIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.orderType;
                cmd.Parameters.Add("P_ZONE", OracleDbType.Varchar2).Value = pData.zone;
                if (pData.class4 == true)
                {
                    cmd.Parameters.Add("P_CLASS", OracleDbType.Varchar2).Value = "1";
                }
                else
                {
                    cmd.Parameters.Add("P_CLASS", OracleDbType.Varchar2).Value = "0";
                }
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_PICKING_LIST_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_PICKING_LIST_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string result = result = cmd.Parameters["R_STATUS"].Value.ToString();
                string pickingListFrom = cmd.Parameters["R_PICKING_LIST_FROM"].Value.ToString();
                string pickingListTo = cmd.Parameters["R_PICKING_LIST_TO"].Value.ToString();
                return Ok(new { result = result, pickingListFrom = pickingListFrom, pickingListTo = pickingListTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GeneratePicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR005_GetPickingList")]
        [Authorize]
        public IHttpActionResult SNWR005_GetPickingList([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICKING_LIST", OracleDbType.Varchar2).Value = pData["PICKING_LIST_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GeneratePicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }




        [HttpPost, Route("SNWR005_GetSearchCust/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR005_GetSearchCust(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GetSearchCust");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR005_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR005_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR005_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR005_PrintPicking")]
        [Authorize]
        public IHttpActionResult SNWR005_PrintPicking([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPickingListNo = "";
            string gTracking = "";
            try
            {
                cmd.CommandText = "SP_NP_SNWR005_NEW_PICKING_LIST_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status == "DONE")
                {
                    gPickingListNo = r_msg;
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNWR005_PrintPicking");
                    //return Ok("Error : " + r_msg);
                    return Json(new { STATUS = r_status, MESSAGE = r_msg });
                }


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR005_PrintPicking");
                // return Ok("Error : " + ex.Message);
                return Json(new { STATUS = "ERROR", MESSAGE = ex.Message });
            }

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR005_SAVE_PRINT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = gPickingListNo;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                    gTracking = data.INSTRUCTION_NO;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                    string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                    if (r_status != "DONE")
                    {
                        oOracledb.WriteLog("ErrorHandle", r_msg, "SNWR005_PrintPicking");
                        //return Ok("Error : " + r_msg);
                        return Json(new { STATUS = r_status, MESSAGE = r_msg });
                    }

                };

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Instruction no. : " + gTracking + " - " + ex.Message, "SNWR005_PrintPicking");

                return Json(new { STATUS = "ERROR", MESSAGE = ex.Message });
            }


            try
            {
                cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR005_SAVE_HEADER";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = gPickingListNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = dataList[0].BRANCH_CODE;

                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNWR005_PrintPicking");
                    return Json(new { STATUS = r_status, MESSAGE = r_msg });
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Instruction no. : " + gTracking + " - " + ex.Message, "SNWR005_PrintPicking");
                return Json(new { STATUS = "ERROR", MESSAGE = ex.Message });
            }

            return Json(new { STATUS = "DONE", MESSAGE = gPickingListNo });
        }

        [HttpPost, Route("SNWR005_GeneratePickingListPDF")]
        [Authorize]
        public async Task<HttpResponseMessage> SNWR005_GeneratePickingListPDF([FromBody] dynamic param)
        {
            try
            {
                //Set
                string reportName = "";
                if (param.SHOWQR == "Y")
                {
                    reportName = "PMS_RPT_WMS_SNWR005_PICKING_WithQR";
                }
                else
                {
                    reportName = "PMS_RPT_WMS_SNWR005_PICKING";
                }

                string reportType = "PDF";
                string fileName = param.PICKING_LIST_FROM + "_" + param.PICKING_LIST_FROM + ".pdf";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Name = "P_COMPANY_CODE", Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), Label = "P_COMPANY_CODE" });
                reportParams.Add(new ParameterValue { Name = "P_PICKING_LIST_FROM", Value = param.PICKING_LIST_FROM, Label = "P_PICKING_LIST_FROM" });
                reportParams.Add(new ParameterValue { Name = "P_PICKING_LIST_TO", Value = param.PICKING_LIST_TO, Label = "P_PICKING_LIST_TO" });
                reportParams.Add(new ParameterValue { Name = "P_INSTRUCTION_NO", Value = param.INSTRUCTION_NO, Label = "P_INSTRUCTION_NO" });

                byte[] reportBytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                string mimeType = MimeMapping.GetMimeMapping(fileName);
                response.Content = new ByteArrayContent(reportBytes.ToArray());
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                return response;
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }


        }
        [HttpPost, Route("SNWR005_GeneratePickingListPDF_KIT")]
        [Authorize]
        public async Task<HttpResponseMessage> SNWR005_GeneratePickingListPDF_KIT([FromBody] dynamic param)
        {
            try
            {
                string reportName = "PMS_RPT_WMS_SNWR005_PICKING_KIT_LIST";
                string reportType = "PDF";
                string fileName = param.PICKING_LIST_FROM + "_" + param.PICKING_LIST_FROM + "_KIT.pdf";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Name = "P_COMPANY_CODE", Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), Label = "P_COMPANY_CODE" });
                reportParams.Add(new ParameterValue { Name = "P_PICKING_LIST_FROM", Value = param.PICKING_LIST_FROM, Label = "P_PICKING_LIST_FROM" });
                reportParams.Add(new ParameterValue { Name = "P_PICKING_LIST_TO", Value = param.PICKING_LIST_TO, Label = "P_PICKING_LIST_TO" });
                reportParams.Add(new ParameterValue { Name = "P_INSTRUCTION_NO", Value = param.INSTRUCTION_NO, Label = "P_INSTRUCTION_NO" });
                byte[] reportBytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                string mimeType = MimeMapping.GetMimeMapping(fileName);
                response.Content = new ByteArrayContent(reportBytes.ToArray());
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                return response;
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }
        #endregion SNWR005

        #region SNWR006
        [HttpPost, Route("SNWR006_getPickListNo/{pickListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR006_getPickListNo(string pickListNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            pickListNo = ismoUtil.escapeCharacter(pickListNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickListNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_getPickListNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR006_GetCustCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR006_GetCustCode(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_GetCustCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR006_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR006_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR006_GetPickingHeader")]
        [Authorize]
        public IHttpActionResult SNWR006_GetPickingHeader([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_GET_PICKING_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pData["PICKING_LIST_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_GetPickingHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNWR006_GetPickingDetail")]
        [Authorize]
        public IHttpActionResult SNWR006_GetPickingDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR006_GET_PICKING_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pData["PICKING_LIST_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_GetPickingDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR006_Save")]
        [Authorize]
        public IHttpActionResult SaveProgressPicking([FromBody] dynamic[] pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                if (pData.Length > 0)
                {
                    foreach (var item in pData)
                    {
                        if (item.QTY != null && Convert.ToInt16(item.QTY.ToString()) > 0)
                        {
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNWR006_SAVE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = item.PICKING_LIST_NO.ToString();
                            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PICK_LOT_DATE.ToString());
                            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Int16).Value = Convert.ToInt16(item.PICK_LOT_SEQ.ToString());
                            cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = item.INSTRUCTION_NO.ToString();
                            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO.ToString();
                            cmd.Parameters.Add("P_QTY", OracleDbType.Int16).Value = Convert.ToInt16(item.QTY.ToString());
                            cmd.Parameters.Add("P_IS_UNLOCK", OracleDbType.Varchar2).Value = "Y";
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                            oOracledb.SqlExecute_CMD(cmd);
                        }
                    }

                    return Json(new { Result = "Done" });
                }

                return Json(new { Result = "Fail", Message = "NO_ROW_TO_PROCESS" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SaveProgressPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR006_SaveReportProblemPart")]
        [Authorize]
        public IHttpActionResult SNWR006_SaveReportProblemPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_CREATE_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pData.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pData.SUB_ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.MISSING_QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR006_SaveReportProblemPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR007

        [HttpGet, Route("SNWR007_GetReasonCode/{reasoncode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetReasonCodeByJobType(string reasoncode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            reasoncode = ismoUtil.escapeCharacter(reasoncode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR007_REASON_CODE";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = reasoncode.ToUpper() == "NULL" ? (object)DBNull.Value : reasoncode;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR007_GetIncompleteList")]
        [Authorize]
        public IHttpActionResult SNWR007_GetIncompleteList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_GetIncompleteList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR007_Submit/{reasonCode}")]
        [Authorize]
        public IHttpActionResult SNWR007_Submit([FromBody] dynamic pData, string reasonCode)
        {
            OracleCommand cmd = new OracleCommand();
            List<dynamic> dataList = new List<dynamic>(pData.data);

            try
            {
                string submitId = DateTime.Now.ToString("yyyyMMddHHmmss");
                foreach (dynamic data in dataList)
                {
                    if (data.STATUS == "Send")
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR007_SUBMIT_DETAIL";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                        cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                        cmd.Parameters.Add("P_PARTIAL_NO", OracleDbType.Int32).Value = data.PARTIAL_NO;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = reasonCode;
                        cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.remarks;
                        cmd.Parameters.Add("P_SUBMIT_ID", OracleDbType.Varchar2).Value = submitId;
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlExecute_CMD(cmd);
                    }

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_PROCESS_SUBMIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUBMIT_ID", OracleDbType.Varchar2).Value = submitId;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_Submit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR007_ReturnItems")]
        [Authorize]
        public IHttpActionResult SNWR007_ReturnItems([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR007_RETURN";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    cmd.Parameters.Add("P_PARTIAL_NO", OracleDbType.Int32).Value = data.PARTIAL_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);





                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_ReturnItems");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion

        #region SNWR007_M
        [HttpGet, Route("SNWR007_GetSubmitList")]
        [Authorize]
        public IHttpActionResult SNWR007_GetSubmitList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_GET_SUBMIT_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_GetIncompleteList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR007_ProcessApprove/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR007_ProcessApprove([FromBody] dynamic pData, string refDocNo)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_PROCESS_APPROVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_ProcessApprove");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR007_A
        [HttpGet, Route("SNWR007_GetApproveList")]
        [Authorize]
        public IHttpActionResult SNWR007_GetApproveList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_GET_APPROVE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_GetApproveList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR007_GetSubmitDetail/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR007_GetSubmitDetail(string refDocNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_GET_SUBMIT_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_GetSubmitDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR007_ProcessSuspend/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR007_ProcessSuspend([FromBody] dynamic pData, string refDocNo)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR007_PROCESS_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR007_ProcessSuspend");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNWR008
        [HttpPost, Route("SNWR022_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR022_GetData")]
        [Authorize]
        public IHttpActionResult SNWR022_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR022_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR022_GetPickLotSeq/{PickLotSeq}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetPickLotSeq(string PickLotSeq, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            PickLotSeq = ismoUtil.escapeCharacter(PickLotSeq);
            var pData = pParam["DATA"];
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_PICK_SEQ";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pData.PICK_LOT_DATE;
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = PickLotSeq.ToUpper() == "NULL" ? (object)DBNull.Value : PickLotSeq;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR022_GetPartConf/{partNo}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetPartConf(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_PART_CONF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];

                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetPartConf");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNWR009

        [HttpPost, Route("SNWR009_GetDaily")]
        [Authorize]
        public IHttpActionResult SNWR009_GetDaily([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_DAILY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.END_DATE;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_ID);
                //cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetDaily");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR009_GetDailyChart")]
        [Authorize]
        public IHttpActionResult SNWR009_GetDailyChart([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_DAILY_CHART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = pDataList.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = pDataList.END_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetDaily");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR009_GetPerformance")]
        [Authorize]
        public IHttpActionResult SNWR009_GetPerformance([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_GET_PERFORMANCE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_ID);
                cmd.Parameters.Add("P_VIEW_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = pDataList.PIC;
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetPerformance");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR009_GetTimeUsagePartReceive")]
        [Authorize]
        public IHttpActionResult SNWR009_GetTimeUsagePartReceive([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_GET_TIME_USAGE_PART_RECEIVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_ID);
                cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = pDataList.PIC_RECEIVE;
                cmd.Parameters.Add("P_VIEW_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetTimeUsagePartReceive");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR009_GetTimeUsagePartShelve")]
        [Authorize]
        public IHttpActionResult SNWR009_GetTimeUsagePartShelve([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_GET_TIME_USAGE_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_ID);
                cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = pDataList.PIC_RECEIVE;
                cmd.Parameters.Add("P_VIEW_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetTimeUsagePart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR009_GetTimeUsageInvoice")]
        [Authorize]
        public IHttpActionResult SNWR009_GetTimeUsageInvoice([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_GET_TIME_USAGE_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_ID);
                cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = pDataList.PIC_RECEIVE;
                cmd.Parameters.Add("P_VIEW_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pDataList.INVOICE_NO;
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.VIEW_DATE;
                //cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetTimeUsageInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR009_GetReceiveMonitor")]
        [Authorize]
        public IHttpActionResult SNWR009_GetReceiveMonitor([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_RECEIVE_MONITOR_R";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.START_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.END_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetReceiveMonitor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR009_GetReceiveMonitorPlus")]
        [Authorize]
        public IHttpActionResult SNWR009_GetReceiveMonitorPlus([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR009_RECEIVE_MONITOR_S_PLUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pDataList.START_DATE;
                //cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pDataList.END_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR009_GetReceiveMonitorPlus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNWR016



        //--------------------------------------------------------------------------------------------------------------

        [HttpPost, Route("SNWR016_GetDataAllocated")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataAllocated([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetAllocated");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetDetailAllocated")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDetailAllocated([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDetailAllocated");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNWR016_GetDataNotPick")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataNotPick([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_NOT_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDataNotPick");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetDetailNotPick")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDetailNotPick([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_NOT_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDetailNotPick");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNWR016_GetDataPicking")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataPicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_PICKING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDataPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetDetailPicking")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDetailPicking([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_PICKING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "L";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDetailPicking");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNWR016_GetDataPicked")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataPicked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_PICKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDataPicked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetDetailPicked")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDetailPicked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_PICKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDetailPicked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }





        [HttpPost, Route("SNWR016_GetDataPacked")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataPacked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_PACKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDataPacked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetPackedDetail")]
        [Authorize]
        public IHttpActionResult SNWR016_GetPackedDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_PACKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = pData.COMPLETE_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetPackedDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR016_GetDataInvoice")]
        [Authorize]
        public IHttpActionResult SNWR016_GetDataInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DATA_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetDataInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNWR016_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_DETAIL_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Date).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = pData.COMPLETE_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR016_FindInstruction")]
        [Authorize]
        public IHttpActionResult SNWR016_FindInstruction([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_FIND_BY_INSTRUCTION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_FindInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR016_GetCutOffBKK")]
        [Authorize]
        public IHttpActionResult SNWR016_GetCutOffBKK([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_CUTOFF_BBK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetCutOffBKK");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR016_GetCutOffUPC")]
        [Authorize]
        public IHttpActionResult SNWR016_GetCutOffUPC([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_GET_CUTOFF_UPC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_GetCutOffUPC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR016_CancelPick")]
        [Authorize]
        public IHttpActionResult SNWR016_CancelPick([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR016_Cancel_Pick";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                // cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR016_CancelPick");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }







        #endregion
        [HttpPost, Route("PrintLabelBKK")]
        public async Task<HttpResponseMessage> PrintLabelBKK([FromBody] dynamic pParams)
        {
            try
            {

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();


                //  this.global.GoToReport("PMS_RPT_WH_SNSR041_OUM", {
                //    P_CUSTOMER_CODE: this.ListDataReportCondition.CUSTOMER_CODE,
                //    P_CASHEER_NO: this.ListDataReportCondition.CASHEER_NO,
                //    P_DOCUMENT_NO: this.ListDataReportCondition.DOCUMENT_NO,
                //    P_TOTAL_PAGE: this.ListDataReportCondition.C_TOTAL_PAGE

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintLabelBKK", "/");

                string reportName = "PMS_RPT_WH_SNSR041_OUM";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CASHEER_NO", Name = "P_CASHEER_NO", Value = pParams.CASHEER_NO });
                reportParams.Add(new ParameterValue { Label = "P_DOCUMENT_NO", Name = "P_DOCUMENT_NO", Value = pParams.DOCUMENT_NO });
                reportParams.Add(new ParameterValue { Label = "P_TOTAL_PAGE", Name = "P_TOTAL_PAGE", Value = pParams.TOTAL_PAGE });

                //Render reports
                //ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams);

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }

        [HttpPost, Route("PrintLabelUpCountry")]
        public async Task<HttpResponseMessage> PrintLabelUpCountry([FromBody] dynamic pParams)
        {
            try
            {

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                //this.global.GoToReport("PMS_RPT_WH_SNSR040_PIK", {
                //  P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
                //  P_IP: this.global.GetTokenDetail("ssIPAddress"),
                //  P_CUSTOMER_CODE: this.ListDataReportCondition.CUSTOMER_CODE,
                //  P_BRANCH_CODE: this.ListDataReportCondition.BRANCH_CODE,
                //  P_CASHEER_NO: this.ListDataReportCondition.CASHEER_NO,
                //  P_DOCUMENT_NO: this.ListDataReportCondition.DOCUMENT_NO


                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintLabelUpCountry", "/");

                string reportName = "PMS_RPT_WH_SNSR040_PIK";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.CUSTOMER_CODE });

                reportParams.Add(new ParameterValue { Label = "P_IP", Name = "P_IP", Value = pParams.IP });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_BRANCH_CODE", Name = "P_BRANCH_CODE", Value = pParams.BRANCH_CODE });

                reportParams.Add(new ParameterValue { Label = "P_CASHEER_NO", Name = "P_CASHEER_NO", Value = pParams.CASHEER_NO });
                reportParams.Add(new ParameterValue { Label = "P_DOCUMENT_NO", Name = "P_DOCUMENT_NO", Value = pParams.DOCUMENT_NO });


                //Render reports
                //ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams);

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }

        #region SNWR015
        [HttpPost, Route("SNWR015_CheckDuplicateInvoice")]
        [Authorize]
        public IHttpActionResult SNWR015_CheckDuplicateInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_CHECK_DUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"];

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Ok("Failed");
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_CheckDuplicateInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok("Done");
        }

        [HttpPost, Route("SNWR015_CheckWOPartNo")]
        [Authorize]
        public IHttpActionResult SNWR015_CheckWOPartNo([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR015_CHECK_WO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_CheckWOPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR015_GetData")]
        [Authorize]
        public IHttpActionResult SNWR015_GetData([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"] != null ? Convert.ToDateTime(pData["INVOICE_DATE"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNWR015_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNWR015_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNWR015_Save")]
        [Authorize]
        public IHttpActionResult SNWR015_Save([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            string gDocNo;

            List<dynamic> partList = new List<dynamic>(pData.PART_LIST);
            int partCount = 0;

            //Insert to pre-receive
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR015_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                foreach (var part in partList)
                {
                    partCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR015_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                    cmd.Parameters.Add("P_RNO", OracleDbType.Decimal).Value = partCount;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Decimal).Value = part["QTY"];
                    cmd.Parameters.Add("P_RECEIVE_PACKAGE", OracleDbType.Decimal).Value = part["RECEIVE_PACKAGE"];

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR015_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["INVOICE_DATE"]);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                gDocNo = cmd.Parameters["R_DOC_NO"].Value.ToString();

            }
            catch (Exception ex)
            {

                if (ex.Message.IndexOf("Found invoice") > 0)
                {
                    return Json(new { Result = "Duplicate invoice", DocNo = "" });
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_Save");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }


            }

            return Json(new { Result = "Done", DocNo = gDocNo });

        }

        [HttpPost, Route("SNWR015_Cancel")]
        [Authorize]
        public IHttpActionResult SNWR015_Cancel([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData["INVOICE_TYPE"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_Cancel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR015_MapInvoice")]
        [Authorize]
        public IHttpActionResult SNWR015_MapInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_MAP_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["PRE_RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_RECEIVE_DATE", OracleDbType.Date).Value = pData["RECEIVE_DATE"] == null ? DBNull.Value : Convert.ToDateTime(pData["RECEIVE_DATE"]);
                cmd.Parameters.Add("P_RECEIVE_DOC_NO", OracleDbType.Varchar2).Value = pData["RECEIVE_DOC_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_MapInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion SNWR015

        #region SNWR015_RO

        [HttpPost, Route("SNWR015_RO_GetData")]
        [Authorize]
        public IHttpActionResult SNWR015_RO_GetData([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR015_RO_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pData["INVOICE_DATE"] != null ? Convert.ToDateTime(pData["INVOICE_DATE"].ToString()) : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

                //DataTable dt = ds.Tables[0];
                //dt = dt.AsEnumerable().Where(x => x.Field<string>("INVOICE_TYPE") == "TEMP").CopyToDataTable();
                //return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR015_RO_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }



        #endregion
        #region SNWR018
        [HttpPost, Route("SNWR018_UpdateInvoiceTime")]
        [Authorize]
        public IHttpActionResult SNWR018_UpdateInvoiceTime([FromBody] dynamic pData)
        {
            string returnMessage;
            string status;
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR018_UPD_INVOICE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dtProcessStatus = ds.Tables[0];
            returnMessage = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
            status = dtProcessStatus.Rows[0]["STATUS"].ToString();
            if (status == "FAIL")
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, returnMessage));
            }
            else if (status == "DONE")
            {
                return Ok(new { STATUS = "DONE" });
            }
            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "Something wrong in process."));
        }
        #endregion

        #region SNWR019
        [HttpPost, Route("SNWR019_INIT")]
        [Authorize]
        public IHttpActionResult SNWR019_CreateCheckStockInit([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_INIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHECK_DATE", OracleDbType.Date).Value = data["CHECK_DATE"];
                cmd.Parameters.Add("P_CHECK_TYPE", OracleDbType.Varchar2).Value = data["CHECK_TYPE"];
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = data["REMARKS"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_CreateCheckStockInit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_GetData")]
        [Authorize]
        public IHttpActionResult SNWR019_GetData([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHECK_DATE", OracleDbType.Date).Value = data["CHECK_DATE"] == null ? DBNull.Value : Convert.ToDateTime(data["CHECK_DATE"]);
                cmd.Parameters.Add("P_CHECK_TYPE", OracleDbType.Varchar2).Value = data["CHECK_TYPE"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_Save/{pHeaderId}")]
        [Authorize]
        public IHttpActionResult SNWR019_Save(string pHeaderId, [FromBody] dynamic[] pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                foreach (var data in pData)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR019_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                    //cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data["LOCATION_NO"];
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data["LOCATION_NO"];
                    cmd.Parameters.Add("P_COUNT_QTY", OracleDbType.Varchar2).Value = data["COUNT_QTY"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR019_GetData_ByID/{headerId}")]
        [Authorize]
        public IHttpActionResult SNWR019_GetData_ByID(int headerId)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_GET_DATA_BY_ID";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int16).Value = headerId;
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_GetData_ByID");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_Send")]
        [Authorize]
        public IHttpActionResult SNWR019_Send([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_SEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int16).Value = data["HEADER_ID"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_Send");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_Submit")]
        [Authorize]
        public IHttpActionResult SNWR019_Submit([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_SUBMIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int16).Value = data["HEADER_ID"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_Submit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_A_GetData")]
        [Authorize]
        public IHttpActionResult SNWR019_A_GetData([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_A_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHECK_DATE", OracleDbType.Date).Value = data["CHECK_DATE"] == null ? DBNull.Value : Convert.ToDateTime(data["CHECK_DATE"]);
                cmd.Parameters.Add("P_CHECK_TYPE", OracleDbType.Varchar2).Value = data["CHECK_TYPE"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_A_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR019_Confirm/{headerId}")]
        [Authorize]
        public IHttpActionResult SNWR019_Confirm(int headerId, [FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                foreach (var d in data)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR019_CONFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int16).Value = headerId;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_CONFIRM");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR019P_UploadExcel/{pHeaderId}")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR019P_UploadExcel(HttpRequestMessage request, string pHeaderId)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle", "UnsupportedMediaType", "SNWR019P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();

                MemoryStream stream = new MemoryStream(buffer);
                DataTable dt = new DataTable();
                SystemApiController sys = new SystemApiController();
                dt = sys.ReadDataExcel(stream);

                OracleCommand cmd;

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                dt = dt.AsEnumerable().Where(data => !string.IsNullOrWhiteSpace(data["LOCATION_NO"].ToString())).CopyToDataTable();
                foreach (DataRow data in dt.Rows)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR019_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                    //cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data["LOCATION_NO"];
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data["LOCATION_NO"];
                    cmd.Parameters.Add("P_COUNT_QTY", OracleDbType.Varchar2).Value = data["COUNT_QTY"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pHeaderId);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable validationResult = ds.Tables[0];

                return Json(DTFM.convertToList(validationResult));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpGet, Route("SNWR019_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR019_GetPartNo(string partNo, int noofshowrec)
        {
            try
            {
                partNo = ismoUtil.escapeCharacter(partNo);
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "sp_NP_SNWR019_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNWR019_GetLocationNo/{locationNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR019_GetLocationNo(string locationNo, int noofshowrec)
        {
            try
            {
                locationNo = ismoUtil.escapeCharacter(locationNo);
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_GET_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = (locationNo != null && locationNo == "null" ? "" : locationNo);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_GetLocationNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR019_SaveHeader")]
        [Authorize]
        public IHttpActionResult SNWR019_SaveHeader([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_SAVE_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pData["HEADER_ID"]);
                cmd.Parameters.Add("P_CHECK_DATE", OracleDbType.Date).Value = pData["CHECK_DATE"];
                cmd.Parameters.Add("P_CHECK_TYPE", OracleDbType.Varchar2).Value = pData["CHECK_TYPE"];
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData["REMARKS"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_SaveHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR019_DeleteDetail")]
        [Authorize]
        public IHttpActionResult SNWR019_DeleteDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR019_DELETE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_ID", OracleDbType.Int32).Value = Convert.ToInt32(pData["HEADER_ID"]);
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pData["LOCATION_NO"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR019_DeleteDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        [HttpPost, Route("SNWR017_Print_Case_Number")]
        [Authorize]
        public IHttpActionResult SNWR017_Print_Case_Number([FromBody] dynamic data)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                int printNo = data["PRINT_NUMBER"];
                int iCount = 0;
                string caseNoFrom = String.Empty;
                string caseNoTo = String.Empty;

                for (int i = 0; i < printNo; i++)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_CASE_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = "DOMESTIC";
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR017";
                    cmd.Parameters.Add("R_CASE_NO", OracleDbType.Varchar2, 6).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    if (iCount == 1)
                    {
                        caseNoFrom = cmd.Parameters["R_CASE_NO"].Value.ToString();
                    }

                    caseNoTo = cmd.Parameters["R_CASE_NO"].Value.ToString();
                }

                return Json(new { Result = "Done", CaseNoFrom = caseNoFrom, CaseNoTo = caseNoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR017_Print_Case_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR017_Get_Case_Number")]
        [Authorize]
        public IHttpActionResult SNWR017_Get_Case_Number([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_CASE_RUNNING_NUMBER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = "DOMESTIC";
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data["CASE_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR017_Print_Case_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM141_Get_Data")]
        [Authorize]
        public IHttpActionResult SNM141_Get_Data([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM141_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM141_Get_Data");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM141_Insert")]
        [Authorize]
        public IHttpActionResult SNM141_Insert([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM141_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_PACKAGE_QTY", OracleDbType.Varchar2).Value = pData["PACKAGE_QTY"];
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM141";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM141_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM141_Update")]
        [Authorize]
        public IHttpActionResult SNM141_Update([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM141_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_PACKAGE_QTY", OracleDbType.Varchar2).Value = pData["PACKAGE_QTY"];
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM141";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM141_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM142_Get_Data")]
        [Authorize]
        public IHttpActionResult SNM142_Get_Data([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142_Get_Data");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM142_Insert")]
        [Authorize]
        public IHttpActionResult SNM142_Insert([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM141";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM142_Delete")]
        [Authorize]
        public IHttpActionResult SNM142_Delete([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #region SNWR010
        [HttpGet, Route("SNWR010_GetBKKDeliveryList/{date_string}")]
        [Authorize]
        public IHttpActionResult SNWR010_GetBKKDeliveryList(string date_string)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_DELIVERY_LIST_BKK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DELIVERY_DATE", OracleDbType.Varchar2).Value = date_string;
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetBKKDeliveryList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR010_GetTempDeliveryList")]
        [Authorize]
        public IHttpActionResult SNWR010_GetTempDeliveryList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_DELIVERY_LIST_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetBKKDeliveryList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_SaveUnSentList")]
        [Authorize]
        public IHttpActionResult SNWR010_SaveUnSentList([FromBody] dynamic pList)
        {
            try
            {
                OracleCommand cmd;

                foreach (var item in pList.ListUnSent)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_HH_UNSENT_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = item.INVOICE_NO;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = item.CASE_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_SaveUnSentList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_SaveSentList")]
        [Authorize]
        public IHttpActionResult SNWR010_SaveSentList([FromBody] dynamic pList)
        {
            try
            {
                OracleCommand cmd;

                foreach (var item in pList.ListSent)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_HH_SENT_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = item.INVOICE_NO;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = item.CASE_NO;
                    cmd.Parameters.Add("P_DELIVERY_DATE", OracleDbType.Varchar2).Value = pList.DateString;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_SaveSentList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_SaveTempRouteList/{routeNo}")]
        [Authorize]
        public IHttpActionResult SNWR010_SaveTempRouteList(string routeNo, [FromBody] dynamic pData)
        {
            try
            {
                List<dynamic> selectedList = new List<dynamic>(pData.selectedList);
                OracleCommand cmd;

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_DEL_ROUTE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROUTE_NO", OracleDbType.Varchar2).Value = routeNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                foreach (var item in selectedList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_SAVE_ROUTE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ROUTE_NO", OracleDbType.Varchar2).Value = routeNo;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = item.INVOICE_NO;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = item.CASE_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_SAVE_ROUTE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROUTE_NO", OracleDbType.Int32).Value = routeNo;
                cmd.Parameters.Add("P_CAR_NO", OracleDbType.Varchar2).Value = pData.CAR_NO;
                cmd.Parameters.Add("P_LICENSE_NO", OracleDbType.Varchar2).Value = pData.LICENSE_NO;
                cmd.Parameters.Add("P_DRIVER_NAME", OracleDbType.Varchar2).Value = pData.DRIVER_NAME;
                cmd.Parameters.Add("P_CAR_TYPE", OracleDbType.Varchar2).Value = pData.CAR_TYPE;
                cmd.Parameters.Add("P_DRIVER_TEL", OracleDbType.Varchar2).Value = pData.DRIVER_TEL;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_SaveTempRouteList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_ConfirmBKK")]
        public IHttpActionResult SNWR010_ConfirmBKK([FromBody] dynamic param)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_CONFIRM_DELIVERY_BKK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_DELIVERY_DATE", OracleDbType.Varchar2).Value = param.DATE_STRING;

                oOracledb.SqlExecute_CMD(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_ConfirmBKK");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR010_GetPDIDeliveryList/{date_string}")]
        [Authorize]
        public IHttpActionResult SNWR010_GetPDIDeliveryList(string date_string)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_DELIVERY_LIST_PDI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DELIVERY_DATE", OracleDbType.Varchar2).Value = date_string;
                cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetBKKDeliveryList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_ConfirmPDI")]
        public IHttpActionResult SNWR010_ConfirmPDI([FromBody] dynamic param)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_CONFIRM_DELIVERY_PDI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_DELIVERY_DATE", OracleDbType.Varchar2).Value = param.DATE_STRING;

                oOracledb.SqlExecute_CMD(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_ConfirmPDI");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR010_UploadFile(HttpRequestMessage request)
        {
            try
            {
                if (!request.Content.IsMimeMultipartContent())
                {
                    throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
                }
                var reqParts = await request.Content.ReadAsMultipartAsync();
                byte[] buffer = null;
                foreach (var file in reqParts.Contents)
                {
                    buffer = file.ReadAsByteArrayAsync().Result;
                }
                MemoryStream stream = new MemoryStream(buffer);
                DataTable dt = new DataTable();
                SystemApiController sys = new SystemApiController();
                dt = sys.ReadDataExcel(stream);

                OracleCommand cmd;
                String uploadId = DateTime.Now.ToString("yyyyMMddHHmmss");

                foreach (DataRow row in dt.Rows)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_UPLOAD_DELIVERY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = row["Document No."];
                    cmd.Parameters.Add("P_PACKAGE_NO", OracleDbType.Varchar2).Value = row["Package No."];
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = row["Box No."];
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = row["Unit Name"];
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["Dealer Code"];
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["Branch Code"];
                    cmd.Parameters.Add("P_DATE", OracleDbType.Varchar2).Value = row["Date"];
                    cmd.Parameters.Add("P_TIME", OracleDbType.Varchar2).Value = row["Time"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = row["User"];
                    cmd.Parameters.Add("P_UPLOAD_ID", OracleDbType.Varchar2).Value = uploadId;

                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_GET_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_UPLOAD_ID", OracleDbType.Varchar2).Value = uploadId;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable deliveryList = ds.Tables[0];

                return Json(DTFM.convertToList(deliveryList));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        [HttpGet, Route("SNWR010_GetRoute/{routeType}")]
        [Authorize]
        public IHttpActionResult SNWR010_GetRoute(string routeType)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_DELIVERY_ROUTE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROUTE_TYPE", OracleDbType.Varchar2).Value = routeType;
                cmd.Parameters.Add("C_RETURN_ROUTE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_CAR", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetRoute");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR010_GetBKKCustomers")]
        [Authorize]
        public IHttpActionResult SNWR010_GetBKKCustomers()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_BKK_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds.Tables[0]));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetBKKCustomers");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR010_GetPDICustomers")]
        [Authorize]
        public IHttpActionResult SNWR010_GetPDICustomers()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR010_PDI_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds.Tables[0]));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR010_GetPDICustomers");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR010_SaveRoute")]
        [Authorize]
        public IHttpActionResult SNWR010_SaveRoute([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            try
            {
                List<dynamic> routes = new List<dynamic>(pData.routes);
                List<dynamic> customers = new List<dynamic>(pData.customers);


                foreach (var data in routes)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_SAVE_ROUTE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ROUTE_NO", OracleDbType.Int32).Value = data.ROUTE_NO;
                    cmd.Parameters.Add("P_CAR_NO", OracleDbType.Varchar2).Value = data.CAR_NO;
                    cmd.Parameters.Add("P_LICENSE_NO", OracleDbType.Varchar2).Value = data.LICENSE_NO;
                    cmd.Parameters.Add("P_DRIVER_NAME", OracleDbType.Varchar2).Value = data.DRIVER_NAME;
                    cmd.Parameters.Add("P_CAR_TYPE", OracleDbType.Varchar2).Value = data.CAR_TYPE;
                    cmd.Parameters.Add("P_DRIVER_TEL", OracleDbType.Varchar2).Value = data.DRIVER_TEL;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                foreach (var data in customers)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR010_SAVE_CUSTOMER";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ROUTE_NO", OracleDbType.Int32).Value = data.ROUTE_NO == null ? DBNull.Value : data.ROUTE_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_DELIVERY_TIME", OracleDbType.Varchar2).Value = data.DELIVERY_TIME;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SavePackedData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion

        #region SNWR021
        [HttpGet, Route("SNWR021_GetData")]
        [Authorize]
        public IHttpActionResult SNWR021_GetData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR021_GET_DATA";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR021_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR021_UpdateStatus")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNWR021_UpdateStatus([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR021_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParams.CONFIG_VALUE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR021";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR021_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR022
        [HttpGet, Route("SNWR022_GetPickListNo/{pickListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetPickListNo(string pickListNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            pickListNo = ismoUtil.escapeCharacter(pickListNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickListNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetPickListNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR022_IssueCase")]
        [Authorize]
        public IHttpActionResult SNWR022_IssueCase([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            int deleteCount = 0;
            string caseNo = "";
            string customerCode = "";
            string branchCode = "";
            try
            {
                //cmd = new OracleCommand();
                //cmd.CommandText = "SP_NP_SNWR022_DEL_TEMP";
                //cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                //oOracledb.SqlQueryReturnValue(cmd);
                List<string> caseList = new List<string>();
                foreach (var data in pDataList)
                {
                    if (!caseList.Contains(data.CASE_NO.ToString()))
                    {
                        caseList.Add(data.CASE_NO.ToString());

                        customerCode = data.CUSTOMER_CODE;
                        branchCode = data.BRANCH_CODE;
                        caseNo = data.CASE_NO;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR022_DELETE_CASE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                        oOracledb.SqlExecute_CMD(cmd);

                        deleteCount++;
                    }
                }
                int i = 0;
                customerCode = "";
                foreach (var data in pDataList)
                {
                    i = i + 1;
                    customerCode = data.CUSTOMER_CODE;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR022_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                return Ok(new { status = "Done" });
            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {

                    if (deleteCount > 0)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR022_REVERT_CASE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                        oOracledb.SqlExecute_CMD(cmd);
                    }

                    err_message = "Found an error, please contract support team";
                }
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_IssueCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpGet, Route("SNWR022_GetAvailableCase/{caseGroup}/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetAvailableCase(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("R_PRINT_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String printDate = cmd.Parameters["R_PRINT_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, PrintDate = printDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Failed", Message = "Unavailable case number." });

                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Exist", Message = "Case is exist, do you want to unpack." });

                }
                else
                {
                    throw (ex);
                }


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR022_GetCaseInfo/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR022_GetCaseInfo(string caseNo)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR022_GET_Case_Info";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR022_GetCaseInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion


        #region SNWR023
        [HttpGet, Route("SNWR023_GetPickListNo/{pickListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR023_GetPickListNo(string pickListNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            pickListNo = ismoUtil.escapeCharacter(pickListNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickListNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetPickListNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR023_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR023_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR023_GetCaseAll")]
        [Authorize]
        public IHttpActionResult SNWR023_GetCaseAll([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_ALL_CASE_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = pData.PACK_BY;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;

                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetCaseAll");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR023_GetCaseAllKit")]
        [Authorize]
        public IHttpActionResult SNWR023_GetCaseAllKit([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_ALL_CASE_CUST_KIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = pData.PACK_BY;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;

                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetCaseAllKit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR023_GetPack")]
        [Authorize]
        public IHttpActionResult SNWR023_GetPack([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_PACK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_Return1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_Return2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_Return3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_Return4", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_Return5", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_Return6", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                //cmd.Parameters.Add("C_Return7", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetPack");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR023_HoldCase")]
        [Authorize]
        public IHttpActionResult SNWR023_HoldCase([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_UPDATE_CASE_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.caseNo;
                cmd.Parameters.Add("P_CASE_STATUS", OracleDbType.Varchar2).Value = "H";
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Ok(new { status = "Done" });


            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {
                    err_message = "Found an error, please contract support team";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Message, "SNWR023_HoldCase");
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_HoldCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("SNWR023_ActiveCase")]
        [Authorize]
        public IHttpActionResult SNWR023_ActiveCase([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_UPDATE_CASE_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.caseNo;
                cmd.Parameters.Add("P_CASE_STATUS", OracleDbType.Varchar2).Value = "A";
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Ok(new { status = "Done" });


            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {
                    err_message = "Found an error, please contract support team";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Message, "SNWR023_ActiveCase");
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_ActiveCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("SNWR023_IssueInvoice/{packBy}")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNWR023_IssueInvoice([FromBody] dynamic[] pDataList, string packBy)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_LOCK_ISSUE_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = packBy;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string gCheckMessage = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (gCheckMessage != "OK")
                {
                    //---- Wait
                    System.Threading.Thread.Sleep(10000);
                    return Ok(new { status = "Wait", message = "มีการออกอินวอยด์พร้อมกันของ " + packBy + " กรุณารอสักครู่" });
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_DEL_TEMP";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = packBy;
                oOracledb.SqlQueryReturnValue(cmd);


                int i = 0;
                string customerCode = "";

                Hashtable hTable = new Hashtable();

                foreach (var data in pDataList)
                {
                    if (data.CASE_STATUS != "H")
                    {

                        if (hTable.Contains(data["CASE_NO"].ToString().Trim()))
                        {
                            continue;
                        }
                        else
                        {
                            hTable.Add(data["CASE_NO"].ToString().Trim(), i);

                            i = i + 1;
                            customerCode = data.CUSTOMER_CODE;
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNWR023_INSERT_TEMP_CASE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                            //cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                            cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                            cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = packBy;
                            ds = oOracledb.SqlQueryDS_CMD(cmd);
                        }

                    }

                }

                if (i > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "PK_NP_SNWR023_INVOICE.ISSUE_INVOICES";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = packBy;
                    cmd.Parameters.Add("R_INVOICE_NO_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_INVOICE_NO_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string invoiceNoFrom = cmd.Parameters["R_INVOICE_NO_FROM"].Value.ToString();
                    string invoiceNoTo = cmd.Parameters["R_INVOICE_NO_TO"].Value.ToString();

                    return Ok(new { status = "Done", invoiceNoFrom = invoiceNoFrom, invoiceNoTo = invoiceNoTo });
                }
                else
                {
                    return Ok(new { status = "Fail", message = "No data to be processed." });
                }

            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {
                    err_message = "Found an error, please contract support team";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Message, "SNWR023_IssueInvoice");
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_IssueInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("SNWR023_IssueInvoiceKit/{packBy}")]
        [Authorize]
        public IHttpActionResult SNWR023_IssueInvoiceKit([FromBody] dynamic[] pDataList, string packBy)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_LOCK_ISSUE_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = packBy;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string gCheckMessage = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (gCheckMessage != "OK")
                {
                    //---- Wait
                    System.Threading.Thread.Sleep(10000);
                    return Ok(new { status = "Wait", message = "มีการออกอินวอยด์พร้อมกันของ " + packBy + " กรุณารอสักครู่" });
                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_DEL_TEMP";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = packBy;
                oOracledb.SqlQueryReturnValue(cmd);
                int i = 0;
                string customerCode = "";
                foreach (var data in pDataList)
                {
                    if (data.CASE_STATUS != "H")
                    {
                        i = i + 1;
                        customerCode = data.CUSTOMER_CODE;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR023_INSERT_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                        cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                        cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                        cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                        cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                        cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                        cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                        cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                        cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                        cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);
                        cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                        cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = packBy;
                        ds = oOracledb.SqlQueryDS_CMD(cmd);
                    }
                }
                if (i > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "PK_NP_SNWR023_INVOICE_KIT.ISSUE_INVOICES";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = packBy;
                    cmd.Parameters.Add("R_INVOICE_NO_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_INVOICE_NO_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string invoiceNoFrom = cmd.Parameters["R_INVOICE_NO_FROM"].Value.ToString();
                    string invoiceNoTo = cmd.Parameters["R_INVOICE_NO_TO"].Value.ToString();
                    return Ok(new { status = "Done", invoiceNoFrom = invoiceNoFrom, invoiceNoTo = invoiceNoTo });
                }
                else
                {
                    return Ok(new { status = "Fail", message = "No data to be processed." });
                }
            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {
                    err_message = "Found an error, please contract support team";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Message, "SNWR023_IssueInvoice");
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_IssueInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        [HttpGet, Route("SNWR023_GetAvailableCase/{caseGroup}/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR023_GetAvailableCase(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("R_PRINT_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String printDate = cmd.Parameters["R_PRINT_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, PrintDate = printDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Failed", Message = "Unavailable case number." });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR023_GetPickLotSeq/{PickLotSeq}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR023_GetPickLotSeq(string PickLotSeq, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            PickLotSeq = ismoUtil.escapeCharacter(PickLotSeq);
            var pData = pParam["DATA"];
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR023_GET_PICK_SEQ";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pData.PICK_LOT_DATE;
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = PickLotSeq.ToUpper() == "NULL" ? (object)DBNull.Value : PickLotSeq;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR023_GetInvoiceList")]
        [Authorize]
        public IHttpActionResult SNWR023_GetInvoiceList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR023_GET_INVOICE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetInvoiceList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR023_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNWR023_GetInvoiceDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR023_GET_INVOICE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR023_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("PrintInvoice")]
        public async Task<HttpResponseMessage> PrintInvoice([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();


                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDO", "/");

                string reportName = "PMS_RPT_WMS_SNWR023_INVOICE";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_FROM", Name = "P_INVOICE_NO_FROM", Value = pParams.P_INVOICE_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_TO", Name = "P_INVOICE_NO_TO", Value = pParams.P_INVOICE_NO_TO });

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                UpdatePrintFlag(pParams.P_INVOICE_NO_FROM.ToString(), pParams.P_INVOICE_NO_TO.ToString());
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        public void UpdatePrintFlag(string invoice_from, string invoice_to)
        {
            OracleCommand cmd = new OracleCommand();
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR023_UPDATE_PRINT_FLAG";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_FROM", OracleDbType.Varchar2).Value = invoice_from;
            cmd.Parameters.Add("P_INVOICE_TO", OracleDbType.Varchar2).Value = invoice_to;
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR023";
            oOracledb.SqlQueryReturnValue(cmd);
        }

        #endregion

        #region SNWR024
        [HttpPost, Route("SNWR024_GetInvoiceForPrint/{pCustomerZone}")]
        [Authorize]
        public IHttpActionResult SNWR024_GetInvoiceForPrint([FromBody] dynamic pParam, string pCustomerZone)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_GET_INVOICE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pCustomerZone;
                cmd.Parameters.Add("R_INVOICE_NO_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_NO_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string invoiceNoFrom = cmd.Parameters["R_INVOICE_NO_FROM"].Value.ToString();
                string invoiceNoTo = cmd.Parameters["R_INVOICE_NO_TO"].Value.ToString();

                return Ok(new { status = "Done", invoiceNoFrom = invoiceNoFrom, invoiceNoTo = invoiceNoTo });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_GetInvoiceForPrint");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR024_GetInvoiceLast/{pCustomerZone}")]
        [Authorize]
        public IHttpActionResult SNWR024_GetInvoiceLast([FromBody] dynamic pParam, string pCustomerZone)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_GET_INVOICE_LAST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pCustomerZone;
                // cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_GetInvoiceLast");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("PrintDeliveryLabel")]
        public async Task<HttpResponseMessage> PrintDeliveryLabel([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();


                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");

                string reportName = "PMS_RPT_WMS_SNWR023_DELIVERY";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_FROM", Name = "P_INVOICE_NO_FROM", Value = pParams.P_INVOICE_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_TO", Name = "P_INVOICE_NO_TO", Value = pParams.P_INVOICE_NO_TO });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_ZONE", Name = "P_CUSTOMER_ZONE", Value = pParams.P_CUSTOMER_ZONE });

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion

        #region SNWR024_UPC
        [HttpPost, Route("SNWR024_UPC_GetInvoiceForPrint")]
        [Authorize]
        public IHttpActionResult SNWR024_UPC_GetInvoiceForPrint([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_UPC_GET_INVOICE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("R_INVOICE_NO_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_NO_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string invoiceNoFrom = cmd.Parameters["R_INVOICE_NO_FROM"].Value.ToString();
                string invoiceNoTo = cmd.Parameters["R_INVOICE_NO_TO"].Value.ToString();
                return Ok(new { status = "Done", invoiceNoFrom = invoiceNoFrom, invoiceNoTo = invoiceNoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_UPC_GetInvoiceForPrint");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR024_UPC_GetInvoiceLast")]
        [Authorize]
        public IHttpActionResult SNWR024_UPC_GetInvoiceLast([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_UPC_GET_INVOICE_LAST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_UPC_GetInvoiceLast");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("PrintDeliveryLabelUPC")]
        public async Task<HttpResponseMessage> PrintDeliveryLabelUPC([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR024_UPC_DELIVERY";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_FROM", Name = "P_INVOICE_NO_FROM", Value = pParams.P_INVOICE_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_TO", Name = "P_INVOICE_NO_TO", Value = pParams.P_INVOICE_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                UpdatePrintFlagLable(pParams.P_INVOICE_NO_FROM.ToString(), pParams.P_INVOICE_NO_TO.ToString());
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        #region SNWR028
        [HttpGet, Route("SNWR028_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR028_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR028_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR028_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR028_Print_Case_Number")]
        [Authorize]
        public IHttpActionResult SNWR028_Print_Case_Number([FromBody] dynamic data)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                int printNo = data["PRINT_NUMBER"];
                int iCount = 0;
                string customerCode = data["CUSTOMER_CODE"];
                string branchCode = data["BRANCH_CODE"];
                string orderType = data["ORDER_TYPE"];
                string caseNoFrom = String.Empty;
                string caseNoTo = String.Empty;
                for (int i = 0; i < printNo; i++)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_CASE_EXPORT_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR028";
                    cmd.Parameters.Add("R_CASE_NO", OracleDbType.Varchar2, 6).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    if (iCount == 1)
                    {
                        caseNoFrom = cmd.Parameters["R_CASE_NO"].Value.ToString();
                    }
                    caseNoTo = cmd.Parameters["R_CASE_NO"].Value.ToString();
                }
                return Json(new { Result = "Done", CustomerCode = customerCode, BranchCode = branchCode, OrderType = orderType, CaseNoFrom = caseNoFrom, CaseNoTo = caseNoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR028_Print_Case_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR028_Get_Case_Number")]
        [Authorize]
        public IHttpActionResult SNWR028_Get_Case_Number([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_CASE_RUNNING_NUMBER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = "EXPORT";
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data["CASE_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR028_Print_Case_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR028_Preview_Case")]
        public async Task<HttpResponseMessage> SNWR028_Preview_Case([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR028_CASE";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.P_CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_BRANCH_CODE", Name = "P_BRANCH_CODE", Value = pParams.P_BRANCH_CODE });
                reportParams.Add(new ParameterValue { Label = "P_ORDER_TYPE", Name = "P_ORDER_TYPE", Value = pParams.P_ORDER_TYPE });
                reportParams.Add(new ParameterValue { Label = "P_CASE_NO_FROM", Name = "P_CASE_NO_FROM", Value = pParams.P_CASE_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_CASE_NO_TO", Name = "P_CASE_NO_TO", Value = pParams.P_CASE_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        #region SNWR031
        [HttpGet, Route("SNWR031_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR031_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR031_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR031_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR031_Print_Babycard_Number")]
        [Authorize]
        public IHttpActionResult SNWR031_Print_Babycard_Number([FromBody] dynamic parms)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                int printNo = parms["PRINT_NUMBER"];
                int iCount = 0;
                string CustomerCode = parms["CUSTOMER_CODE"];
                string BabyCardGroup = parms["BABYCARD_GROUP"];
                string NoFrom = String.Empty;
                string NoTo = String.Empty;
                for (int i = 0; i < printNo; i++)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_BABYCARD_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode;
                    cmd.Parameters.Add("P_BABYCARD_GROUP", OracleDbType.Varchar2).Value = BabyCardGroup;
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR031";
                    cmd.Parameters.Add("R_BABYCARD_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    if (iCount == 1)
                    {
                        NoFrom = cmd.Parameters["R_BABYCARD_NO"].Value.ToString();
                    }
                    NoTo = cmd.Parameters["R_BABYCARD_NO"].Value.ToString();
                }
                return Json(new { Result = "Done", CustomerCode = CustomerCode, BabyCardGroup = BabyCardGroup, BabyCardNoFrom = NoFrom, BabyCardNoTo = NoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR031_Print_Babycard_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR031_Get_Babycard_Number")]
        [Authorize]
        public IHttpActionResult SNWR031_Get_Babycard_Number([FromBody] dynamic parms)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_BABYCARD_RUNNING_NUMBER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = parms["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BABYCARD_GROUP", OracleDbType.Varchar2).Value = parms["BABYCARD_GROUP"];
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = parms["BABYCARD_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR031_Get_Babycard_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR031_Preview_BabyCard")]
        public async Task<HttpResponseMessage> SNWR031_Preview_BabyCard([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR031_BABYCARD";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.P_CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_GROUP", Name = "P_BABYCARD_GROUP", Value = pParams.P_BABYCARD_GROUP });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_NO_FROM", Name = "P_BABYCARD_NO_FROM", Value = pParams.P_BABYCARD_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_NO_TO", Name = "P_BABYCARD_NO_TO", Value = pParams.P_BABYCARD_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        public void UpdatePrintFlagLable(string invoice_from, string invoice_to)
        {
            OracleCommand cmd = new OracleCommand();
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR024_UPDATE_PRINT_FLAG_LABLE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_INVOICE_FROM", OracleDbType.Varchar2).Value = invoice_from;
            cmd.Parameters.Add("P_INVOICE_TO", OracleDbType.Varchar2).Value = invoice_to;
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR023";
            oOracledb.SqlQueryReturnValue(cmd);
        }
        #region SNWR024_BKK
        [HttpPost, Route("SNWR024_BKK_GetInvoiceForPrint")]
        [Authorize]
        public IHttpActionResult SNWR024_BKK_GetInvoiceForPrint([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_BKK_GET_INVOICE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("R_INVOICE_NO_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_NO_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string invoiceNoFrom = cmd.Parameters["R_INVOICE_NO_FROM"].Value.ToString();
                string invoiceNoTo = cmd.Parameters["R_INVOICE_NO_TO"].Value.ToString();
                return Ok(new { status = "Done", invoiceNoFrom = invoiceNoFrom, invoiceNoTo = invoiceNoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_BKK_GetInvoiceForPrint");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR024_BKK_GetInvoiceLast")]
        [Authorize]
        public IHttpActionResult SNWR024_BKK_GetInvoiceLast([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR024_BKK_GET_INVOICE_LAST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR024_BKK_GetInvoiceLast");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("PrintDeliveryLabelBKK")]
        public async Task<HttpResponseMessage> PrintDeliveryLabelBKK([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR024_BKK_DELIVERY";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_FROM", Name = "P_INVOICE_NO_FROM", Value = pParams.P_INVOICE_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_INVOICE_NO_TO", Name = "P_INVOICE_NO_TO", Value = pParams.P_INVOICE_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                UpdatePrintFlagLable(pParams.P_INVOICE_NO_FROM.ToString(), pParams.P_INVOICE_NO_TO.ToString());
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        #region SNSE032_W

        [HttpPost, Route("SNSE032_W_GetCaseDetail")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetCaseDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_W_GET_CASE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetCaseDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE032_W_GetVanNo/{van}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetVanNo(string van, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            van = ismoUtil.escapeCharacter(van);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_W_GET_VAN_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = van;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE032_W_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE032_W_GetPreInvoiceAir/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetPreInvoiceAir(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_W_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetPreInvoiceAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE032_W_GetPreInvoiceNoAir/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetPreInvoiceNoAir(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_W_GET_PRE_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetPreInvoiceAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE032_W_GetVanData")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetVanData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_W_GET_VAN_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetVanData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE032_W_GetData")]
        [Authorize]
        public IHttpActionResult SNSE032_W_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pParam.VAN_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE032_W_ModifyPre")]
        [Authorize]
        public IHttpActionResult SNSE032_W_ModifyPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            String pPInvoiceDate = "";
            try
            {
                pPInvoiceDate = DateTime.Now.ToString("MM-dd-yyyy HH:mm:ss");
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_W_MODIFY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = pPInvoiceDate;
                    cmd.Parameters.Add("P_CHK_BOX", OracleDbType.Varchar2).Value = data.CHK_BOX;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                };
                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE032_W_GetDataModify")]
        [Authorize]
        public IHttpActionResult SNSE032_GetDataModify([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_W_GET_DATA_MODIFY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam.PINVOICE_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetDataModify");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE032_W_ConfirmPre")]
        [Authorize]
        public IHttpActionResult SNSE032_W_ConfirmPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPreInvoice = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE032_W_GEN_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gPreInvoice = ds.Tables[0].Rows[0]["PRE_INV_NO"].ToString();

                }
            }
            catch (Exception ex)
            {
                gPreInvoice = "";
                return Ok("Error : " + ex.Message);
            }

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_W_CONFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = gPreInvoice;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    //cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    //cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    //cmd.Parameters.Add("P_GROSS_WEIGHT", OracleDbType.Varchar2).Value = data.GROSS_WEIGHT;
                    //cmd.Parameters.Add("P_GROSS_MM", OracleDbType.Varchar2).Value = data.GROSS_MM;
                    //cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = data.CASE_WIDTH;
                    //cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = data.CASE_DEPTH;
                    //cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = data.CASE_HEIGHT;
                    //cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = dataList.VAN_NO;
                    cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = data.VAN_GROUP;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE-" + gPreInvoice);
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE032_W_ConfirmDelete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE032_W_ConfirmDelete([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_W_DELETE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                    // cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }


        [HttpPost, Route("SNSE032_W_Load_report")]
        public async Task<HttpResponseMessage> SNSE032_W_Load_report([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_SALES_SNSE032_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUST_S", Name = "P_CUST_S", Value = pParams.P_CUST_S });
                reportParams.Add(new ParameterValue { Label = "P_CUST_E", Name = "P_CUST_E", Value = pParams.P_CUST_E });

                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_START", Name = "P_PINV_NO_START", Value = pParams.P_PINV_NO_START });
                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_END", Name = "P_PINV_NO_END", Value = pParams.P_PINV_NO_END });

                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_S", Name = "P_PINV_DATE_S", Value = pParams.P_PINV_DATE_S });
                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_E", Name = "P_PINV_DATE_E", Value = pParams.P_PINV_DATE_E });
                reportParams.Add(new ParameterValue { Label = "P_SENT_BY", Name = "P_SENT_BY", Value = pParams.P_SENT_BY });

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion SNSE032_W



        #region SNSE053_W

        [HttpPost, Route("SNSE053_W_GetCaseDetail")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetCaseDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE053_W_GET_CASE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetCaseDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }


        [HttpGet, Route("SNSE053_W_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE053_W_GetVanData")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetVanData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_W_GET_VAN_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_W_GetVanData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }




        [HttpPost, Route("SNSE053_W_GetData")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pParam.VAN_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE053_W_GetDataModify")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetDataModify([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_W_GET_DATA_MODIFY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam.PINVOICE_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetDataModify");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE053_W_ConfirmPre")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE053_W_ConfirmPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPreInvoice = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE053_W_GEN_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gPreInvoice = ds.Tables[0].Rows[0]["PRE_INV_NO"].ToString();

                }
            }
            catch (Exception ex)
            {
                gPreInvoice = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_ConfirmPre");
                return Ok("Error : " + ex.Message);
            }

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE053_W_CONFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = gPreInvoice;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    //cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    //cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    //cmd.Parameters.Add("P_GROSS_WEIGHT", OracleDbType.Varchar2).Value = data.GROSS_WEIGHT;
                    //cmd.Parameters.Add("P_GROSS_MM", OracleDbType.Varchar2).Value = data.GROSS_MM;
                    //cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = data.CASE_WIDTH;
                    //cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = data.CASE_DEPTH;
                    //cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = data.CASE_HEIGHT;
                    //cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data.VAN_NO;
                    cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = data.VAN_GROUP;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE053_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE-" + gPreInvoice);
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }


        [HttpPost, Route("SNSE053_W_ConfirmDelete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE053_ConfirmDelete([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE053_W_DELETE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                    // cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE053_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_ConfirmDelete");
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE053_W_GetPreInvoiceSea/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetPreInvoiceSea(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_W_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetPreInvoiceSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE053_W_ModifyPre")]
        [Authorize]
        public IHttpActionResult SNSE053_W_ModifyPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            String pPInvoiceDate = "";
            try
            {
                pPInvoiceDate = DateTime.Now.ToString("MM-dd-yyyy HH:mm:ss");
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE053_MODIFY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = pPInvoiceDate;
                    cmd.Parameters.Add("P_CHK_BOX", OracleDbType.Varchar2).Value = data.CHK_BOX;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE053_W";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                };
                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE053_W_Load_report")]
        public async Task<HttpResponseMessage> SNSE053_W_Load_report([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_SALES_SNSE053_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUST_S", Name = "P_CUST_S", Value = pParams.P_CUST_S });
                reportParams.Add(new ParameterValue { Label = "P_CUST_E", Name = "P_CUST_E", Value = pParams.P_CUST_E });

                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_START", Name = "P_PINV_NO_START", Value = pParams.P_PINV_NO_START });
                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_END", Name = "P_PINV_NO_END", Value = pParams.P_PINV_NO_END });

                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_S", Name = "P_PINV_DATE_S", Value = pParams.P_PINV_DATE_S });
                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_E", Name = "P_PINV_DATE_E", Value = pParams.P_PINV_DATE_E });
                reportParams.Add(new ParameterValue { Label = "P_SENT_BY", Name = "P_SENT_BY", Value = pParams.P_SENT_BY });

                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("SNSE053_W_GetPreInvoiceNoSea/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE053_W_GetPreInvoiceNoSea(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_W_GET_PRE_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_W_GetPreInvoiceNoSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion SNSE053_W

        #region SNSE033_W

        [HttpGet, Route("SNSE033_W_GetCustomerCodeAir/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetCustomerCodeAir(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetCustomerCodeAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE033_W_GetCustomerCodeSea/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetCustomerCodeSea(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetCustomerCodeSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE033_W_GetPreInvoiceAir/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetPreInvoiceAir(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_W_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetPreInvoiceAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE033_W_GetPreInvoiceSea/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetPreInvoiceSea(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_W_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetPreInvoiceSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE033_W_GetDataAir")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetDataAir([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam["PINVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetDataAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE033_W_GetDataSea")]
        [Authorize]
        public IHttpActionResult SNSE033_W_GetDataSea([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam["PINVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_GetDataSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE033_W_ConfirmAir")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE033_W_ConfirmAir([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gInvoiceNo = "";
            string gTracking = "";
            //string gDocType = "";
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE033_W_TEMP_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            oOracledb.SqlQueryReturnValue(cmd);
            try
            {
                foreach (var data in dataList)
                {
                    gTracking = data.CASE_NO;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE033_W_TEMP_INSERT_AIR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = data.PINVOICE_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data.VAN_NO;
                    cmd.Parameters.Add("P_AWB_NO", OracleDbType.Varchar2).Value = dataList[0].AWB_NO;
                    cmd.Parameters.Add("P_AWB_DATE", OracleDbType.Date).Value = dataList[0].AWB_DATE;
                    cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
                    cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
                    cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
                    cmd.Parameters.Add("P_FLIGHT_NO", OracleDbType.Varchar2).Value = dataList[0].FLIGHT_NO;
                    cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);

                };
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_W_ConfirmAir");
                return Ok("Error: " + ex.Message);
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_W_START_CONFIRM_AIR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = dataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() == "DONE")
                    {
                        gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
                        return Ok("DONE-" + gInvoiceNo);
                    }
                    else
                    {
                        string errString = "Error: " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    return Ok("Error: " + "No status return");
                }
            }
            catch (Exception ex)
            {
                gInvoiceNo = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmAir");
                return Ok("Error: " + ex.Message);
            }

            //try
            //{
            //    cmd.CommandText = "SP_NP_SNSE033_W_GEN_INVOICE_NO";
            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            //    ds = oOracledb.SqlQueryDS_CMD(cmd);

            //    if (ds.Tables.Count > 0)
            //    {
            //        gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
            //        gDocType = gInvoiceNo.Substring(0, 2);
            //    }
            //}
            //catch (Exception ex)
            //{
            //    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmAir");
            //    return Ok("Error : " + ex.Message);
            //}

            //try
            //{
            //    cmd = new OracleCommand();
            //    cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_INV";
            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //    cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "START";
            //    oOracledb.SqlQueryReturnValue(cmd);
            //    foreach (var data in dataList)
            //    {
            //        cmd = new OracleCommand();
            //        cmd.CommandText = "SP_NP_SNSE033_W_CONFIRM_AIR";
            //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
            //        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
            //        cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;

            //        cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
            //        cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
            //        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
            //        cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
            //        cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
            //        cmd.Parameters.Add("P_AWB_NO", OracleDbType.Varchar2).Value = dataList[0].AWB_NO;

            //        cmd.Parameters.Add("P_AWB_DATE", OracleDbType.Date).Value = dataList[0].AWB_DATE;
            //        cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
            //        cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
            //        cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
            //        cmd.Parameters.Add("P_FLIGHT_NO", OracleDbType.Varchar2).Value = dataList[0].FLIGHT_NO;
            //        cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;

            //        cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //        cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033_W";
            //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            //        gTracking = data.CASE_NO;
            //        ds = oOracledb.SqlQueryDS_CMD(cmd);

            //        if (ds.Tables.Count > 0)
            //        {
            //            if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
            //            {
            //                string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
            //                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            //            }
            //        }

            //    };

            //}
            //catch (Exception ex)
            //{
            //    oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_W_ConfirmAir");
            //    return Ok("Error : " + ex.Message);
            //}

            //try
            //{
            //    cmd = new OracleCommand();

            //    cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_TRANS_H";

            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
            //    cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = gDocType;

            //    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033_W";
            //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            //    ds = oOracledb.SqlQueryDS_CMD(cmd);

            //    if (ds.Tables.Count > 0)
            //    {
            //        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
            //        {
            //            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
            //            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            //        }
            //    }
            //}
            //catch (Exception ex)
            //{
            //    gInvoiceNo = "";
            //    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmAir");
            //    return Ok("Error : " + ex.Message);
            //}
            //cmd = new OracleCommand();
            //cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_INV";
            //cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "END";
            //oOracledb.SqlQueryReturnValue(cmd);
            //return Ok("DONE-" + gInvoiceNo);
        }

        [HttpPost, Route("SNSE033_W_ConfirmSea")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE033_W_ConfirmSea([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gInvoiceNo = "";
            string gTracking = "";
            //string gDocType = "";
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE033_W_TEMP_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            oOracledb.SqlQueryReturnValue(cmd);
            try
            {
                foreach (var data in dataList)
                {
                    gTracking = data.CASE_NO;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE033_W_TEMP_INSERT_SEA";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = data.PINVOICE_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data.VAN_NO;
                    cmd.Parameters.Add("P_BL_NO", OracleDbType.Varchar2).Value = dataList[0].BL_NO;
                    cmd.Parameters.Add("P_BL_DATE", OracleDbType.Date).Value = dataList[0].BL_DATE;
                    cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
                    cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
                    cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
                    cmd.Parameters.Add("P_VESSEL", OracleDbType.Varchar2).Value = dataList[0].VESSEL;
                    cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                };
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_W_ConfirmSea");
                return Ok("Error: " + ex.Message);
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_W_START_CONFIRM_SEA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = dataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() == "DONE")
                    {
                        gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
                        return Ok("DONE-" + gInvoiceNo);
                    }
                    else
                    {
                        string errString = "Error: " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    return Ok("Error: " + "No status return");
                }
            }
            catch (Exception ex)
            {
                gInvoiceNo = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmSea");
                return Ok("Error: " + ex.Message);
            }
            //try
            //{
            //    cmd.CommandText = "SP_NP_SNSE033_W_GEN_INVOICE_NO";
            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            //    ds = oOracledb.SqlQueryDS_CMD(cmd);

            //    if (ds.Tables.Count > 0)
            //    {
            //        gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
            //        gDocType = gInvoiceNo.Substring(0, 2);
            //    }
            //}
            //catch (Exception ex)
            //{
            //    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmSea");
            //    return Ok("Error : " + ex.Message);
            //}

            //try
            //{
            //    cmd = new OracleCommand();
            //    cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_INV";
            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //    cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "START";
            //    oOracledb.SqlQueryReturnValue(cmd);
            //    foreach (var data in dataList)
            //    {
            //        cmd = new OracleCommand();
            //        cmd.CommandText = "SP_NP_SNSE033_W_CONFIRM_SEA";
            //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
            //        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
            //        cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;

            //        cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
            //        cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
            //        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
            //        cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
            //        cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
            //        cmd.Parameters.Add("P_BL_NO", OracleDbType.Varchar2).Value = dataList[0].BL_NO;

            //        cmd.Parameters.Add("P_BL_DATE", OracleDbType.Date).Value = dataList[0].BL_DATE;
            //        cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
            //        cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
            //        cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
            //        cmd.Parameters.Add("P_VESSEL", OracleDbType.Varchar2).Value = dataList[0].VESSEL;
            //        cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;

            //        cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //        cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033_W";
            //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            //        gTracking = data.CASE_NO;
            //        ds = oOracledb.SqlQueryDS_CMD(cmd);

            //        if (ds.Tables.Count > 0)
            //        {
            //            if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
            //            {
            //                string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
            //                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            //            }
            //        }

            //    };

            //}
            //catch (Exception ex)
            //{
            //    oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_W_ConfirmSea");
            //    return Ok("Error : " + ex.Message);
            //}


            //try
            //{
            //    cmd = new OracleCommand();

            //    cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_TRANS_H";

            //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
            //    cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = gDocType;

            //    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033_W";
            //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            //    ds = oOracledb.SqlQueryDS_CMD(cmd);

            //    if (ds.Tables.Count > 0)
            //    {
            //        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
            //        {
            //            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
            //            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            //        }
            //    }
            //}
            //catch (Exception ex)
            //{
            //    gInvoiceNo = "";
            //    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_W_ConfirmSea");
            //    return Ok("Error : " + ex.Message);
            //}
            //cmd = new OracleCommand();
            //cmd.CommandText = "SP_NP_SNSE033_W_UPDATE_INV";
            //cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            //cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            //cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "END";
            //oOracledb.SqlQueryReturnValue(cmd);
            //return Ok("DONE-" + gInvoiceNo);
        }

        [HttpPost, Route("SNSE033_W_PDF_INVOICE")]
        public async Task<HttpResponseMessage> SNSE033_W_PDF_INVOICE([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_SALES_SNSE033_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_DOC_TYPE", Name = "P_DOC_TYPE", Value = pParams.P_DOC_TYPE });
                reportParams.Add(new ParameterValue { Label = "P_DOC_NO", Name = "P_DOC_NO", Value = pParams.P_DOC_NO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion SNSE033_W
        #region SNWR026

        [HttpPost, Route("SNWR026_getPickListNo/{pickListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR026_getPickListNo(string pickListNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            pickListNo = ismoUtil.escapeCharacter(pickListNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickListNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_getPickListNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR026_GetCustCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR026_GetCustCode(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_GetCustCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR026_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR026_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR026_GetPickingHeader")]
        [Authorize]
        public IHttpActionResult SNWR026_GetPickingHeader([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_GET_PICKING_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pData["PICKING_LIST_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_GetPickingHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNWR026_GetPickingDetail")]
        [Authorize]
        public IHttpActionResult SNWR026_GetPickingDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_GET_PICKING_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pData["PICKING_LIST_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_GetPickingDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR026_Save")]
        [Authorize]
        public IHttpActionResult SNWR026_Save([FromBody] dynamic[] pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                if (pData.Length > 0)
                {
                    foreach (var item in pData)
                    {
                        if (item.QTY != null && Convert.ToInt16(item.QTY.ToString()) > 0)
                        {
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNWR026_SAVE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = item.PICKING_LIST_NO.ToString();
                            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(item.PICK_LOT_DATE.ToString());
                            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Int16).Value = Convert.ToInt16(item.PICK_LOT_SEQ.ToString());
                            cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = item.INSTRUCTION_NO.ToString();
                            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item.PART_NO.ToString();
                            cmd.Parameters.Add("P_QTY", OracleDbType.Int16).Value = Convert.ToInt16(item.QTY.ToString());
                            cmd.Parameters.Add("P_IS_UNLOCK", OracleDbType.Varchar2).Value = "Y";
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                            oOracledb.SqlExecute_CMD(cmd);
                        }
                    }

                    return Json(new { Result = "Done" });
                }

                return Json(new { Result = "Fail", Message = "NO_ROW_TO_PROCESS" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR026_SaveReportProblemPart")]
        [Authorize]
        public IHttpActionResult SNWR026_SaveReportProblemPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR026_CREATE_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pData.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = pData.SUB_ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = pData.MISSING_QTY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Json(new { Result = "Done" });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR026_SaveReportProblemPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        #endregion

        #region SNWR027

        [HttpPost, Route("SNWR027_CheckPackBabyCard")]
        [Authorize]
        public IHttpActionResult SNWR027_CheckPackBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CHECK_PACK_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_CheckPackBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR027_CheckPackInnerCase")]
        [Authorize]
        public IHttpActionResult SNWR027_CheckPackInnerCase([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CHECK_PACK_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INNERCASE_NO", OracleDbType.Varchar2).Value = pParam.INNERCASE_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_CheckPackInnerCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR027_GetPickListNo/{pickListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetPickListNo(string pickListNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            pickListNo = ismoUtil.escapeCharacter(pickListNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_PICKING_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickListNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetPickListNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR027_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR027_GetPickLotSeq/{PickLotSeq}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetPickLotSeq(string PickLotSeq, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            PickLotSeq = ismoUtil.escapeCharacter(PickLotSeq);
            var pData = pParam["DATA"];
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_PICK_SEQ";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pData.PICK_LOT_DATE;
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = PickLotSeq.ToUpper() == "NULL" ? (object)DBNull.Value : PickLotSeq;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR027_GetPartConf/{partNo}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetPartConf(string partNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_PART_CONF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];

                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetPartConf");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR027_GetData/{baby_no}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetData(string baby_no)
        {
            DataSet ds = new DataSet();
            baby_no = ismoUtil.escapeCharacter(baby_no);
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR027_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = baby_no;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR027_IssueCase")]
        [Authorize]
        public IHttpActionResult SNWR027_IssueCase([FromBody] dynamic pDataList)
        {
            OracleCommand cmd;
            int deleteCount = 0;
            try
            {
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_DELETE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//Date for delete current case no (protect 
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                deleteCount++;
                int i = 0;
                foreach (var data in packingList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR027_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);//Date for new case (But not used ToDateTime Function)
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_INNER_CASE_NO", OracleDbType.Varchar2).Value = data.INNER_CASE_NO;
                    cmd.Parameters.Add("P_INSERT_TYPE", OracleDbType.Varchar2).Value = "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_SAVE_PACKING_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                return Ok(new { status = "Done", message = "Done" });
            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is issue invoive, please try again later!";
                }
                else
                {
                    if (deleteCount > 0)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR027_REVERT_CASE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                        cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                        cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo; ;
                        oOracledb.SqlExecute_CMD(cmd);
                    }

                    err_message = "Found an error, please contract support team";
                }
                return Ok(new { status = "Fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_IssueCase");
                if (deleteCount > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR027_REVERT_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("SNWR027_ClearCaseExport")]
        [Authorize]
        public IHttpActionResult SNWR027_ClearCaseExport([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CLEAR_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pParam.scannedCase.caseDate);//for not delete old case no
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_ClearCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpGet, Route("SNWR027_GetAvailableCase/{customerCode}/{branchCode}/{orderType}/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetAvailableCase(string customerCode, string branchCode, string orderType, string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_AVAILABLE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable caseData = ds.Tables[0];

                return Json(new { Result = "Done", Data = DTFM.convertToList(caseData) });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Failed", Message = "Unavailable case number." });
                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Exist", Message = "Case is exist, do you want to unpack." });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { Result = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                else if (ex.Number == 20000)
                {
                    return Json(new { Result = "Notfound", Message = "Case is not found, Please generate case first." });
                }
                else
                {
                    throw (ex);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNWR027_GetCaseInfo/{customerCode}/{branchCode}/{orderType}/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetCaseInfo(string customerCode, string branchCode, string orderType, string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetCaseInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR027_GetAvailableInnerCase/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR027_GetAvailableInnerCase(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_AVAILABLE_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetAvailableInnerCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR027_GetCaseTypeAll")]
        [Authorize]
        public IHttpActionResult SNWR027_GetCaseTypeAll(string caseType = null)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_GET_CASE_TYPE_ALL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_GetCaseTypeAll");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR027_ClearCaseExportTemp")]
        [Authorize]
        public IHttpActionResult SNWR027_ClearCaseExportTemp([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR027_CLEAR_CASE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam["ORDER_TYPE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("P_DELETE_BY_USER", OracleDbType.Varchar2).Value = pParam["DELETE_BY_USER"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR027_ClearCaseExportTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion




        #region SNWR030

        [HttpPost, Route("SNWR030_Print_Van_Number")]
        [Authorize]
        public IHttpActionResult SNWR030_Print_Van_Number([FromBody] dynamic data)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();

                int printNo = data["PRINT_NUMBER"];
                int iCount = 0;
                string vanNoFrom = String.Empty;
                string vanNoTo = String.Empty;

                for (int i = 0; i < printNo; i++)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_VAN_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = "EXPORT";
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR030";
                    cmd.Parameters.Add("R_VAN_NO", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    if (iCount == 1)
                    {
                        vanNoFrom = cmd.Parameters["R_VAN_NO"].Value.ToString();
                    }

                    vanNoTo = cmd.Parameters["R_VAN_NO"].Value.ToString();
                }

                return Json(new { Result = "Done", VanNoFrom = vanNoFrom, VanNoTo = vanNoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR030_Print_Van_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR030_Get_Van_Number")]
        [Authorize]
        public IHttpActionResult SNWR030_Get_Van_Number([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_VAN_RUNNING_NUMBER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = "EXPORT";
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data["VAN_NO"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR030_Get_Van_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR030_Preview_Vannumber")]
        public async Task<HttpResponseMessage> SNWR030_Preview_Vannumber([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR030_VAN";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_VAN_NO_FROM", Name = "P_VAN_NO_FROM", Value = pParams.P_VAN_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_VAN_NO_TO", Name = "P_VAN_NO_TO", Value = pParams.P_VAN_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("SNWR030_N_GetVanNo/{VanNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR030_N_GetVanNo(string VanNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            VanNo = ismoUtil.escapeCharacter(VanNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR030_N_GET_VAN_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = VanNo.ToUpper() == "NULL" ? (object)DBNull.Value : VanNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR030_N_GET_VAN_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        #endregion

        #region SNWR032
        [HttpGet, Route("SNWR032_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR032_GetPartNo(string partNo = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                partNo = ismoUtil.escapeCharacter(partNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_GetPartNo");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNWR032_GetBabyCardLevelMaster")]
        [Authorize]
        public IHttpActionResult SNWR032_GetBabyCardLevelMaster([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_GET_BABY_CARD_LEVEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { result = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_GetBabyCardLevel");
                return Json(new { result = "Error", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR032_InsertBabyCardLevelMaster")]
        [Authorize]
        public IHttpActionResult SNWR032_InsertBabyCardLevelMaster([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_INSERT_BABYCARD_LEVEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LEVEL_1", OracleDbType.Varchar2).Value = pData.LEVEL_1;
                cmd.Parameters.Add("P_LEVEL_2", OracleDbType.Varchar2).Value = pData.LEVEL_2;
                cmd.Parameters.Add("P_LEVEL_3", OracleDbType.Varchar2).Value = pData.LEVEL_3;
                cmd.Parameters.Add("P_LEVEL_4", OracleDbType.Varchar2).Value = pData.LEVEL_4;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_InsertBabyCardLevelMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR032_UpdateBabyCardLevelMaster")]
        [Authorize]
        public IHttpActionResult SNWR032_UpdateBabyCardLevelMaster([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_UPDATE_BABYCARD_LEVEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LEVEL_1", OracleDbType.Varchar2).Value = pData.LEVEL_1;
                cmd.Parameters.Add("P_LEVEL_2", OracleDbType.Varchar2).Value = pData.LEVEL_2;
                cmd.Parameters.Add("P_LEVEL_3", OracleDbType.Varchar2).Value = pData.LEVEL_3;
                cmd.Parameters.Add("P_LEVEL_4", OracleDbType.Varchar2).Value = pData.LEVEL_4;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_UpdateBabyCardLevelMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR032_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR032_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT PART_NO,LEVEL_2 AS TYPE_A,LEVEL_3 AS TYPE_B,LEVEL_1 AS TYPE_C,UPLOAD_USER,COMPANY_CODE,SEQ FROM TPM_NP_BABYCARD_LEVEL_MASTER_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        [HttpPost, Route("SNWR032_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNWR032_SaveByUpload([FromBody] dynamic[] pData)
        {
            try
            {
                var json = JsonConvert.SerializeObject(pData);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNWR032_InsertBabyCardTemporary")]
        [Authorize]
        public IHttpActionResult SNWR032_InsertBabyCardTemporary([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_INSERT_BABYCARD_TEMPORARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LEVEL_1", OracleDbType.Varchar2).Value = pData.LEVEL_1;
                cmd.Parameters.Add("P_LEVEL_2", OracleDbType.Varchar2).Value = pData.LEVEL_2;
                cmd.Parameters.Add("P_LEVEL_3", OracleDbType.Varchar2).Value = pData.LEVEL_3;
                cmd.Parameters.Add("P_LEVEL_4", OracleDbType.Varchar2).Value = pData.LEVEL_4;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_InsertBabyCardTemporary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR032_UpdateBabyCardTemporary")]
        [Authorize]
        public IHttpActionResult SNWR032_UpdateBabyCardTemporary([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_UPDATE_BABYCARD_TEMPORARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LEVEL_1", OracleDbType.Varchar2).Value = pData.LEVEL_1;
                cmd.Parameters.Add("P_LEVEL_2", OracleDbType.Varchar2).Value = pData.LEVEL_2;
                cmd.Parameters.Add("P_LEVEL_3", OracleDbType.Varchar2).Value = pData.LEVEL_3;
                cmd.Parameters.Add("P_LEVEL_4", OracleDbType.Varchar2).Value = pData.LEVEL_4;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_UpdateBabyCardTemporary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR032_DeleteBabyCardTemporary")]
        [Authorize]
        public IHttpActionResult SNWR032_DeleteBabyCardTemporary([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_DELETE_BABYCARD_TEMPORARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pData.ID;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_DeleteBabyCardTemporary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR032_ConfirmBabyCardTemporary")]
        [Authorize]
        public IHttpActionResult SNWR032_ConfirmBabyCardTemporary([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR032_CONFIRM_BABYCARD_TEMPORARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pData.ID;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR032_ConfirmBabyCardTemporary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNJF003_W

        [HttpGet, Route("SNJF003_W_GetCustomerCode/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNJF003_W_GetCustomerCode(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_W_getInvoiceNo/{invoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNJF003_W_getInvoiceNo(string invoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            invoiceNo = ismoUtil.escapeCharacter(invoiceNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_INOVICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_getInvoiceNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_W_getPreInvoice")]
        [Authorize]
        public IHttpActionResult SNJF003_W_getPreInvoice([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_PRE_INOVICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pParam["INVOICE_DATE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam["SENT_BY"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_getPreInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_W_getData")]
        [Authorize]
        public IHttpActionResult SNJF003_W_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam["SENT_BY"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNJF003_W_getDataDetail")]
        [Authorize]
        public IHttpActionResult SNJF003_W_getDataDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_DATA_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pParam["PACK_DATE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_getDataDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNJF003_W_getFileName")]
        [Authorize]
        public IHttpActionResult SNJF003_W_getFileName()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_GET_FILE_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNJF003_W";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_getFileName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_W_UpdateInvoiceHeader")]
        [Authorize]
        public IHttpActionResult SNJF003_W_UpdateInvoiceHeader([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_W_UPD_TRANS_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam.INVOICE_NO;
                cmd.Parameters.Add("P_BL_AWB_DATE", OracleDbType.Date).Value = pParam.BL_AWB_DATE;
                cmd.Parameters.Add("P_BL_AWB_NO", OracleDbType.Varchar2).Value = pParam.BL_AWB_NO;
                cmd.Parameters.Add("P_VESSEL_FLIGHT_N", OracleDbType.Varchar2).Value = pParam.VESSEL_FLIGHT_N;
                cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Varchar2).Value = pParam.EXCHANGE_RATE;
                cmd.Parameters.Add("P_INSURANCE_C", OracleDbType.Varchar2).Value = pParam.INSURANCE_C;
                cmd.Parameters.Add("P_FWD_CHARGE_C", OracleDbType.Varchar2).Value = pParam.FWD_CHARGE_C;
                cmd.Parameters.Add("P_FREIGH_CHARGE_C", OracleDbType.Varchar2).Value = pParam.FREIGH_CHARGE_C;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_UpdateInvoiceHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_W_UpdateInvoiceDetail")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNJF003_W_UpdateInvoiceDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNJF003_W_PROCESS_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = data.INVOICE_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_W_UpdateInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_W_Dragon_Generate")]
        [Authorize]
        public IHttpActionResult SNJF003_W_Dragon_Generate([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNJF003_W_INS_DRAGON";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_HEADER", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dtProcessStatus = ds.Tables[0];

            if (dtProcessStatus.Rows[0]["STATUS"].ToString() == "DONE")
            {
                //Set data table for generate order file
                DataTable dtDragonFile = ds.Tables[1];
                DataTable dtDragonData = ds.Tables[2];

                string gFileDetailName = dtDragonFile.Rows[0]["FILE_NAME"].ToString();

                string gCategory = "SNJF003_W/DRG";

                //Write and get file path.
                string gDetailPath = DataService.WriteFile(dtDragonData, "TEXT", gCategory, gFileDetailName);

                var fileDetail = DataService.GetFileStream(gDetailPath, gFileDetailName);

                return Ok(new { Detail = new { filename = gFileDetailName, file = fileDetail } });
            }
            else
            {
                string errString = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            }
        }

        [HttpPost, Route("SNJF003_W_PDF_INVOICE")]
        public async Task<HttpResponseMessage> SNJF003_W_PDF_INVOICE([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_SALES_SNJF003_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_DOC_TYPE", Name = "P_DOC_TYPE", Value = pParams.P_DOC_TYPE });
                reportParams.Add(new ParameterValue { Label = "P_DOC_NO", Name = "P_DOC_NO", Value = pParams.P_DOC_NO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion

        #region SNSE034_W

        [HttpGet, Route("SNSE034_W_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE034_W_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_W_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //[HttpPost, Route("SNSE034_W_GetBranchCode/{branchCode}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE034_W_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        //{
        //    DataSet ds = new DataSet();

        //    branchCode = ismoUtil.escapeCharacter(branchCode);
        //    var pData = pParam["DATA"];

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNSE034_W_BRANCH";
        //    cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
        //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
        //    cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //    ds = oOracledb.SqlQueryDS_CMD(cmd);

        //    return Json(DTFM.convertToList(ds));
        //}

        [HttpPost, Route("SNSE034_W_getData")]
        [Authorize]
        public IHttpActionResult SNSE034_W_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_W_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE034_W_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE034_W_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE034_W_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE034_W";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_W_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE034_W_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE034W_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_W_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param.CASE_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_W_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSE026_W

        [HttpPost, Route("SNSE026W_getAllocateFromDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_getAllocateFromDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "FROM";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_getAllocateFromDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026W_getAllocateToDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_getAllocateToDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "TO";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_getAllocateToDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026W_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_getCustomerCode(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026W_getCustomerCodeReprint/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_getCustomerCodeReprint(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_CUSTOMER_CODE_REPRINT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_getCustomerCodeReprint");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026W_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026W_GetAcceptPartNo/{acceptPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026W_GetAcceptPartNo(string acceptPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            acceptPartNo = ismoUtil.escapeCharacter(acceptPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_ACCEPT_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = acceptPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : acceptPartNo;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_GetAcceptPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNSE026W_GetPickLotSeq")]
        [Authorize]
        public IHttpActionResult SNSE026W_GetPickLotSeq()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_GET_PICKLOT_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE026W_FirstLast_Date")]
        [Authorize]
        public IHttpActionResult SNSE026W_FirstLast_Date([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_FIRST_LAST_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = pParam.ALLOCATE_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_FirstLast_Date");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }






        [HttpPost, Route("SNSE026W_getData")]
        [Authorize]
        public IHttpActionResult SNSE026W_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pParam.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_DG", OracleDbType.Varchar2).Value = pParam.DG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNSE026W_GetPickingNo")]
        [Authorize]
        public IHttpActionResult SNSE026W_GetPickingNo()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_GET_PICKING_NO";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_GetPickingNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE026W_Process")]
        [Authorize]
        //[IssueCardGuard]
        public IHttpActionResult SNSE026W_Process([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_DEL_TEMP";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlQueryReturnValue(cmd);

                int i = 0;
                foreach (var data in pDataList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE026W_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OLD_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.OLD_SUB_ITEM_NO;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = data.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = data.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = data.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = data.ZONE_AREA;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE;
                    cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = data.ALLOCATE_DATE;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = data.INVOICE_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pDataList[0].PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pDataList[0].PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pDataList[0].PICKING_LIST_NO;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE026_W";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Date).Value = data.LAST_UPD_DATE;
                    cmd.Parameters.Add("P_CHK_PROCESS", OracleDbType.Varchar2).Value = i;
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_START_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pDataList[0].PICK_LOT_DATE;
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pDataList[0].PICK_LOT_SEQ;
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pDataList[0].PICKING_LIST_NO;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "DONE" });
            }
            catch (OracleException oex)
            {
                string err_message = string.Empty;
                if (oex.Number == 20002)
                {
                    err_message = "Someone is printing picking card, please try again later!";
                }
                else
                {
                    err_message = oex.Message;
                }
                oOracledb.WriteLog("ErrorHandle", err_message, "SNSE026W_Process");
                return Ok(new { status = "fail", message = err_message, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("SNSE026W_UpdateStatus")]
        [Authorize]
        public IHttpActionResult SNSE026W_UpdateStatus([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026W_UPDATE_STATUS";
                cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = pParam.LOCATION;
                cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = pParam.LOCATION_FROM;
                cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = pParam.LOCATION_TO;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026W_UpdateStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026_W_Load_Report")]
        public async Task<HttpResponseMessage> SNSE026_W_Load_Report([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNSE026_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUST_CODE", Name = "P_CUST_CODE", Value = pParams.P_CUST_CODE });
                reportParams.Add(new ParameterValue { Label = "P_ALLOCATE_DATE_S", Name = "P_ALLOCATE_DATE_S", Value = pParams.P_ALLOCATE_DATE_S });
                reportParams.Add(new ParameterValue { Label = "P_PICK_LIST_NO_S", Name = "P_PICK_LIST_NO_S", Value = pParams.P_PICK_LIST_NO_S });
                reportParams.Add(new ParameterValue { Label = "P_CREATE_BY", Name = "P_CREATE_BY", Value = pParams.P_CREATE_BY });
                reportParams.Add(new ParameterValue { Label = "P_IP", Name = "P_IP", Value = pParams.P_IP });
                reportParams.Add(new ParameterValue { Label = "P_TYPE", Name = "P_TYPE", Value = pParams.P_TYPE });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion

        #region SNSE043_W

        [HttpGet, Route("SNSE043_W_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043_W_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043_W_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043_W_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043_W_getInvoiceDate/{invoiceDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043_W_getInvoiceDate(string invoiceDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            invoiceDate = ismoUtil.escapeCharacter(invoiceDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043_W_INVOICE_DATE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = invoiceDate.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceDate;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043_W_getInvoiceDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043_W_getDocNo/{DocNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043_W_getDocNo(string DocNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            DocNo = ismoUtil.escapeCharacter(DocNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043_W_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                //cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"];
                //cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"];

                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"].ToString("dd-MMM-yyyy");
                cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"].ToString("dd-MMM-yyyy");


                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = DocNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043_W_getDocNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE43_W_getData")]
        [Authorize]
        public IHttpActionResult SNSE43_W_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pParam.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pParam.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pParam["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pParam["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE43_W_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043_W_ExportExcel")]
        public HttpResponseMessage SNSE043_W_ExportExcel(List<DataModel.SNSE043_W> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSE043_W.csv";
            string floderLocation = "SNSE043_W";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Pre-Invoice Attached Sheet\"");
            dt.Rows.Add("C;X1;Y2;K\"Pre-Invoice No\"");
            dt.Rows.Add("C;X2;Y2;K\"Buyer Code\"");
            dt.Rows.Add("C;X3;Y2;K\"Destination\"");
            dt.Rows.Add("C;X4;Y2;K\"Date (yyyy-mm-dd)\"");
            dt.Rows.Add("C;X11;Y2;K" + '"' + dModel[0].COMPANY_NAME + '"');
            dt.Rows.Add("C;X1;Y3;K" + '"' + dModel[0].PINVOICE_NO + '"');
            dt.Rows.Add("C;X2;Y3;K" + '"' + dModel[0].CUSTOMER_CODE + '"');
            dt.Rows.Add("C;X3;Y3;K" + '"' + dModel[0].BRANCH_NAME_EN + '"');
            dt.Rows.Add("C;X4;Y3;K" + '"' + Convert.ToDateTime(dModel[0].PINVOICE_DATE).ToString("dd-MM-yyyy") + '"');
            dt.Rows.Add("C;X1;Y4;K\"Container\"");
            dt.Rows.Add("C;X3;Y4;K\"Serial\"");
            dt.Rows.Add("C;X11;Y4;K" + '"' + dModel[0].CURRENCY_CODE + '"');
            dt.Rows.Add("C;X13;Y4;K\"Total\"");
            dt.Rows.Add("C;X13;Y5;K\"N/WT\"");
            dt.Rows.Add("C;X14;Y5;K\"M3\"");
            dt.Rows.Add("C;X15;Y5;K\"G/W\"");
            dt.Rows.Add("C;X1;Y6;K\"Case No.\"");
            dt.Rows.Add("C;X2;Y6;K\"Sub No.\"");
            dt.Rows.Add("C;X3;Y6;K\"Serial No.\"");
            dt.Rows.Add("C;X4;Y6;K\"Indent No.\"");
            dt.Rows.Add("C;X5;Y6;K\"Item No.\"");
            dt.Rows.Add("C;X6;Y6;K\"Parts No.\"");
            dt.Rows.Add("C;X7;Y6;K\"Description\"");
            dt.Rows.Add("C;X8;Y6;K\"Country of Origin\"");
            dt.Rows.Add("C;X9;Y6;K\"H.S. Code\"");
            dt.Rows.Add("C;X10;Y6;K\"QTY\"");
            dt.Rows.Add("C;X11;Y6;K\"Unit Price\"");
            dt.Rows.Add("C;X12;Y6;K\"Amount\"");
            dt.Rows.Add("C;X13;Y6;K\"(0.001KG)\"");
            dt.Rows.Add("C;X14;Y6;K\"(0.001m3)\"");
            dt.Rows.Add("C;X15;Y6;K\"(0.1kg.)\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = 7 + i;
                dt.Rows.Add("C;X1;Y" + itemno + ";K" + '"' + dModel[i].CASE_NO + '"');
                dt.Rows.Add("C;X2;Y" + itemno + ";K" + '"' + dModel[i].SUB_CASE_NO + '"');
                dt.Rows.Add("C;X3;Y" + itemno + ";K" + '"' + dModel[i].SERIAL_NO + '"');
                dt.Rows.Add("C;X4;Y" + itemno + ";K" + '"' + dModel[i].INDENT_NO + '"');
                dt.Rows.Add("C;X5;Y" + itemno + ";K" + '"' + dModel[i].V_ITEM_NO + '"');
                dt.Rows.Add("C;X6;Y" + itemno + ";K" + '"' + dModel[i].PART_NO + '"');
                dt.Rows.Add("C;X7;Y" + itemno + ";K" + '"' + dModel[i].PART_NAME_EN + '"');
                dt.Rows.Add("C;X8;Y" + itemno + ";K" + '"' + dModel[i].ORIGIN_NAME_EN + '"');
                dt.Rows.Add("C;X9;Y" + itemno + ";K" + '"' + dModel[i].TARIFF_CODE + '"');
                dt.Rows.Add("C;X10;Y" + itemno + ";K" + '"' + dModel[i].PACK_QTY + '"');
                dt.Rows.Add("C;X11;Y" + itemno + ";K" + '"' + dModel[i].UNIT_PRICE_C + '"');
                dt.Rows.Add("C;X12;Y" + itemno + ";K" + '"' + dModel[i].AAX + '"');
                dt.Rows.Add("C;X13;Y" + itemno + ";K" + '"' + dModel[i].N_WT + '"');
                dt.Rows.Add("C;X14;Y" + itemno + ";K" + '"' + dModel[i].GROSS_MM + '"');
                dt.Rows.Add("C;X15;Y" + itemno + ";K" + '"' + dModel[i].GROSS_WEIGHT + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        [HttpPost, Route("SNSE043_W_PDF_INVOICE")]
        public async Task<HttpResponseMessage> SNSE043_W_PDF_INVOICE([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_SALES_SNSE043_W";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUST_S", Name = "P_CUST_S", Value = pParams.P_CUST_S });
                reportParams.Add(new ParameterValue { Label = "P_CUST_E", Name = "P_CUST_E", Value = pParams.P_CUST_E });
                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_START", Name = "P_PINV_NO_START", Value = pParams.P_PINV_NO_START });
                reportParams.Add(new ParameterValue { Label = "P_PINV_NO_END", Name = "P_PINV_NO_END", Value = pParams.P_PINV_NO_END });
                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_S", Name = "P_PINV_DATE_S", Value = pParams.P_PINV_DATE_S });
                reportParams.Add(new ParameterValue { Label = "P_PINV_DATE_E", Name = "P_PINV_DATE_E", Value = pParams.P_PINV_DATE_E });
                reportParams.Add(new ParameterValue { Label = "P_SENT_BY", Name = "P_SENT_BY", Value = pParams.P_SENT_BY });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion

        #region SNSE044_W
        [HttpGet, Route("SNSE044_W_getFileName")]
        [Authorize]
        public IHttpActionResult SNSE044_W_getFileName()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_GET_FILE_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_getFileName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044_W_getDocType")]
        [Authorize]
        public IHttpActionResult SNSE044_W_getDocType([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_GET_DOC_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_getDocType");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE044_W_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE044_W_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044_W_getDocNo/{DocNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE044_W_getDocNo(string DocNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            DocNo = ismoUtil.escapeCharacter(DocNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData["DOC_TYPE"];
                cmd.Parameters.Add("P_DOCUMENT_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"] == null ? (object)DBNull.Value : pData["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"] == null ? (object)DBNull.Value : pData["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = DocNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_getDocNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044_W_getData")]
        [Authorize]
        public HttpResponseMessage SNSE044_W_getData([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            string xxx = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE044_W_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE_FROM;
            cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE_TO;
            cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = data.DOC_NO_FROM;
            cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = data.DOC_NO_TO;
            cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = data["INVOICE_DATE_FROM"] == null ? (object)DBNull.Value : data["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = data["INVOICE_DATE_TO"] == null ? (object)DBNull.Value : data["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
            cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = data.DOC_TYPE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSE044_W_ExportExcel(ds.Tables[0], "SNSE044_W.csv", "SNSE044_W");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSE044_W.csv");
            return respons;

        }

        public static string SNSE044_W_ExportExcel(DataTable dt, string fileName, string FloderLocation)
        {
            string invoiceNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            string caseNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            bool Succes = false;
            bool firstInsert = false;
            decimal totalCaseM3 = 0;
            decimal totalCaseGW = 0;
            decimal totalCaseQTY = 0;
            decimal totalCaseAMT = 0;
            decimal totalCaseNW = 0;

            decimal totalConM3 = 0;
            decimal totalConGW = 0;
            decimal totalConQTY = 0;
            decimal totalConAMT = 0;
            decimal totalConNW = 0;


            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                foreach (DataRow row in dt.Rows)
                {

                    if (row[0].ToString() != invoiceNo)
                    {
                        invoiceNo = row[0].ToString();
                        caseNo = row[8].ToString();

                        if (firstInsert)
                        {
                            string RowTotalCase = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                            sb.AppendLine(RowTotalCase);
                            totalConQTY = totalConQTY + totalCaseQTY;
                            totalConAMT = totalConAMT + totalCaseAMT;
                            totalConNW = totalConNW + totalCaseNW;
                            totalConM3 = totalConM3 + totalCaseM3;
                            totalConGW = totalConGW + totalCaseGW;
                            totalCaseQTY = 0;
                            totalCaseAMT = 0;
                            totalCaseNW = 0;
                            totalCaseM3 = 0;
                            totalCaseGW = 0;

                            string RowTotalContainer = "," + "," + "," + "," + "," + "," + "Total By Container" + "," + "," + "," + totalConQTY + "," + "," + totalConAMT + "," + totalConNW + "," + totalConM3 + "," + totalConGW;
                            sb.AppendLine(RowTotalContainer);
                            totalConQTY = 0;
                            totalConAMT = 0;
                            totalConNW = 0;
                            totalConM3 = 0;
                            totalConGW = 0;
                        }

                        string firstRow = "Invoice Attached Sheet";
                        sb.AppendLine(firstRow);

                        string secondRow = "Invoice No" + "," + "Buyer Code" + "," + "Destination" + "," + "Date (yyyy-mm-dd)" + "," + "," + "," + "," + "," + "," + "," + "NISSAN MOTOR (THAILAND) CO. LTD.";
                        sb.AppendLine(secondRow);

                        string thirdRow = row[0].ToString() + "," + row[1].ToString() + "," + row[2].ToString() + "," + row[3].ToString() + "," + "," + "," + "," + "," + "," + "," + "," + "," + ",";
                        sb.AppendLine(thirdRow);

                        string fouthRow = "Container" + "," + "," + "Serial" + "," + "," + "," + "," + "," + "," + "," + "," + row[4].ToString() + "," + "," + "Total";
                        sb.AppendLine(fouthRow);

                        string fifthRow = "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "N/WT" + "," + "M3" + "," + "G/W";
                        sb.AppendLine(fifthRow);

                        string[] HeaderForExport = new string[]
                    {   "Case No.",
                        "Sub No.",
                        "Serial No.",
                        "Indent No.",
                        "Item No.",
                        "Parts No.",
                        "Description",
                        "Country of Origin",
                        "H.S. Code",
                        "QTY",
                        "Unit Price",
                        "Amount",
                        "(0.001KG)",
                        "(0.001m3)",
                        "(0.1kg.)"
                    };

                        IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                        sb.AppendLine(string.Join(",", columnNames));

                        for (int i = 8; i < dt.Columns.Count - 19; i++)
                        {
                            sb.Append(CsvEscape(row[i].ToString()));
                            sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                        }


                        totalCaseQTY = decimal.Parse(row[23].ToString());
                        totalCaseAMT = decimal.Parse(row[24].ToString());
                        totalCaseNW = decimal.Parse(row[25].ToString());

                        totalCaseM3 = decimal.Parse(row[21].ToString());
                        totalCaseGW = decimal.Parse(row[22].ToString());


                        //totalConQTY = decimal.Parse(row[26].ToString());
                        //totalConAMT = decimal.Parse(row[27].ToString());
                        //totalConNW = decimal.Parse(row[28].ToString());

                        //totalCaseM3 = decimal.Parse(row[21].ToString());
                        //totalCaseGW = decimal.Parse(row[22].ToString());

                        //totalConM3 = decimal.Parse(row[31].ToString());
                        //totalConGW = decimal.Parse(row[32].ToString());
                    }
                    else
                    {
                        firstInsert = true;

                        if (row[8].ToString() != caseNo && row[0].ToString() == invoiceNo)
                        {
                            invoiceNo = row[0].ToString();
                            caseNo = row[8].ToString();

                            string RowTotalCase = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                            sb.AppendLine(RowTotalCase);
                            totalConQTY = totalConQTY + totalCaseQTY;
                            totalConAMT = totalConAMT + totalCaseAMT;
                            totalConNW = totalConNW + totalCaseNW;
                            totalConM3 = totalConM3 + totalCaseM3;
                            totalConGW = totalConGW + totalCaseGW;
                            totalCaseQTY = 0;
                            totalCaseAMT = 0;
                            totalCaseNW = 0;
                            totalCaseM3 = 0;
                            totalCaseGW = 0;

                            for (int i = 8; i < dt.Columns.Count - 19; i++)
                            {
                                sb.Append(CsvEscape(row[i].ToString()));
                                sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                            }

                            totalCaseQTY = decimal.Parse(row[23].ToString());
                            totalCaseAMT = decimal.Parse(row[24].ToString());
                            totalCaseNW = decimal.Parse(row[25].ToString());
                            totalCaseM3 = decimal.Parse(row[21].ToString());
                            totalCaseGW = decimal.Parse(row[22].ToString());

                        }
                        else
                        {
                            sb.Append("," + ",");
                            for (int i = 10; i < dt.Columns.Count - 19; i++)
                            {
                                sb.Append(CsvEscape(row[i].ToString()));
                                sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                            }

                            totalCaseQTY = decimal.Parse(row[23].ToString());
                            totalCaseAMT = decimal.Parse(row[24].ToString());
                            totalCaseNW = decimal.Parse(row[25].ToString());
                            totalCaseM3 = decimal.Parse(row[21].ToString());
                            totalCaseGW = decimal.Parse(row[22].ToString());

                        }
                    }
                }
                if (dt.Rows.Count > 0)
                {
                    string RowTotalCaseLast = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                    sb.AppendLine(RowTotalCaseLast);
                    totalConQTY = totalConQTY + totalCaseQTY;
                    totalConAMT = totalConAMT + totalCaseAMT;
                    totalConNW = totalConNW + totalCaseNW;
                    totalConM3 = totalConM3 + totalCaseM3;
                    totalConGW = totalConGW + totalCaseGW;
                    totalCaseQTY = 0;
                    totalCaseAMT = 0;
                    totalCaseNW = 0;
                    totalCaseM3 = 0;
                    totalCaseGW = 0;

                    string RowTotalContainerLast = "," + "," + "," + "," + "," + "," + "Total By Container" + "," + "," + "," + totalConQTY + "," + "," + totalConAMT + "," + totalConNW + "," + totalConM3 + "," + totalConGW;
                    sb.AppendLine(RowTotalContainerLast);
                    totalConQTY = 0;
                    totalConAMT = 0;
                    totalConNW = 0;
                    totalConM3 = 0;
                    totalConGW = 0;
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }
        }

        [HttpPost, Route("SNSE044_W_Paperless")]
        [Authorize]
        public IHttpActionResult SNSE044_W_Paperless([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_INS_PAPERLESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_Paperless");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044_W_Dragon")]
        [Authorize]
        public IHttpActionResult SNSE044_W_Dragon([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044_W_INS_DRAGON";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044_W_Dragon");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE044_W_Dragon_Generate")]
        [Authorize]
        public IHttpActionResult SNSE044_W_Dragon_Generate([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE044_W_INS_DRAGON";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
            cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
            cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
            cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
            cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_HEADER", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dtProcessStatus = ds.Tables[0];

            if (dtProcessStatus.Rows[0]["STATUS"].ToString() == "DONE")
            {
                //Set data table for generate order file
                DataTable dtDragonFile = ds.Tables[1];
                DataTable dtDragonData = ds.Tables[2];

                string gFileDetailName = dtDragonFile.Rows[0]["FILE_NAME"].ToString();

                string gCategory = "SNSE044_W/DRG";

                //Write and get file path.
                string gDetailPath = DataService.WriteFile(dtDragonData, "TEXT", gCategory, gFileDetailName);

                var fileDetail = DataService.GetFileStream(gDetailPath, gFileDetailName);

                return Ok(new { Detail = new { filename = gFileDetailName, file = fileDetail } });
            }
            else
            {
                string errString = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            }
        }


        #endregion

        #region SNWR033
        [HttpPost, Route("SNWR033_CheckReceiveBabyCard")]
        [Authorize]
        public IHttpActionResult SNWR033_CheckReceiveBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_CHECK_RECEIVE_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                string status = ds.Tables[0].Rows[0]["STATUS"].ToString();
                string qrcode = ds.Tables[0].Rows[0]["PICKING_QR_CODE"].ToString();
                return Json(new { data = DTFM.convertToList(ds), status = status, qrcode = qrcode });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR033_CheckReceiveBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR033_GetAvailableBaby/{babyNo}")]
        [Authorize]
        public IHttpActionResult SNWR033_GetAvailableBaby(string babyNo)
        {
            DataSet ds = new DataSet();
            babyNo = ismoUtil.escapeCharacter(babyNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_GET_AVAILABLE_BABY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABY_NO", OracleDbType.Varchar2).Value = babyNo;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_BABYCARD_NO", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                String staTus = cmd.Parameters["R_STATUS"].Value.ToString();
                String baByNo = cmd.Parameters["R_BABYCARD_NO"].Value.ToString();
                return Json(new { Result = staTus, BABYCARD_NO = baByNo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR033_GetAvailableBaby");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR033_GetData")]
        [Authorize]
        public IHttpActionResult SNWR033_GetData([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pParam.INSTRUCTION_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dtMessage = ds.Tables[ds.Tables.Count - 1]; //Last Table in dataset (always message table)
                string message = dtMessage.Rows[0]["MESSAGE"].ToString();
                string processFlag = dtMessage.Rows[0]["PROCESS_FLAG"].ToString();
                string processMessage = dtMessage.Rows[0]["PROCESS_MESSAGE"].ToString();
                string partno = dtMessage.Rows[0]["PART_NO"].ToString();
                return Json(new { data = DTFM.convertToList(ds), message = message, processflag = processFlag, processmessage = processMessage, partno = partno });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR033_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR033_Receive")]
        [Authorize]
        public IHttpActionResult SNWR033_Receive([FromBody] dynamic pDataList)
        {
            try
            {
                var scannedCard = pDataList.SCANNED_CARD;
                List<dynamic> receiveList = new List<dynamic>(pDataList.RECEIVE_LIST);
                DataSet ds = new DataSet();
                OracleCommand cmd;
                int i = 0;
                foreach (var data in receiveList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR033_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = data.PICKING_LIST_NO;
                    cmd.Parameters.Add("P_RECEIVE_QTY", OracleDbType.Varchar2).Value = data.RECEIVE_QTY;
                    cmd.Parameters.Add("P_BABY_QTY", OracleDbType.Varchar2).Value = data.BABY_QTY;
                    cmd.Parameters.Add("P_BABY_LEVEL", OracleDbType.Varchar2).Value = data.BABY_LEVEL;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = i == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_SEQ", OracleDbType.Varchar2).Value = data.SEQ;
                    cmd.Parameters.Add("P_TOGGLE_INPUT", OracleDbType.Varchar2).Value = data.TOGGLE_INPUT;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                if (scannedCard?.USER_SETTING_LEVEL == "Y")
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR033_SAVE_SETTING_BABYLEVEL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = scannedCard.PICKING_LIST_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = scannedCard.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = scannedCard.PART_NO;
                    cmd.Parameters.Add("P_TYPE_A", OracleDbType.Varchar2).Value = scannedCard.TYPE_A_TEMP;
                    cmd.Parameters.Add("P_TYPE_B", OracleDbType.Varchar2).Value = scannedCard.TYPE_B_TEMP;
                    cmd.Parameters.Add("P_TYPE_C", OracleDbType.Varchar2).Value = scannedCard.TYPE_C_TEMP;
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_RECEIVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string staTus = cmd.Parameters["R_STATUS"].Value.ToString();


                return Ok(new { status = "Done" });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR033_Receive");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpPost, Route("SNWR033_CalculateBabyLevelExport")]
        [Authorize]
        public IHttpActionResult SNWR033_CalculateBabyLevelExport([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd;
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR033_CALCULATE_BABYLEVEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pParam.INSTRUCTION_NO;
                cmd.Parameters.Add("P_TYPE_A", OracleDbType.Varchar2).Value = pParam.TYPE_A_TEMP;
                cmd.Parameters.Add("P_TYPE_B", OracleDbType.Varchar2).Value = pParam.TYPE_B_TEMP;
                cmd.Parameters.Add("P_TYPE_C", OracleDbType.Varchar2).Value = pParam.TYPE_C_TEMP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { data = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR033_CalculateBabyLevelExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion SNWR033
        #region SNWR036
        [HttpPost, Route("SNWR036_GetDataAllocated")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataAllocated([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetAllocated");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDetailAllocated")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDetailAllocated([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDetailAllocated");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataNotPick")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataNotPick([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_NOT_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataNotPick");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDetailNotPick")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDetailNotPick([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_NOT_PICK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE;
                //cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE;
                cmd.Parameters.Add("P_PRINT_DATE", OracleDbType.Date).Value = pData.PRINT_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pData.PICKING_LIST_NO;
                // cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDetailNotPick");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataReceived")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataReceived([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_RECEIVED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataReceived");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDetailReceived")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDetailReceived([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_RECIEVED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE;
                cmd.Parameters.Add("P_PRINT_DATE", OracleDbType.Date).Value = pData.PRINT_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "R";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDetailReceived");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataPicked")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataPicked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_PICKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataPicked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDetailPicked")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDetailPicked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_PICKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE;
                //cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE;
                cmd.Parameters.Add("P_PRINT_DATE", OracleDbType.Date).Value = pData.PRINT_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pData.PICKING_LIST_NO;
                //cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = "P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDetailPicked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataPacked")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataPacked([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_PACKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataPacked");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetPackedDetail")]
        [Authorize]
        public IHttpActionResult SNWR036_GetPackedDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_PACKED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE;
                cmd.Parameters.Add("P_PRINT_DATE", OracleDbType.Date).Value = pData.PRINT_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = pData.COMPLETE_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetPackedDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataVan")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataVan([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetVanDetail")]
        [Authorize]
        public IHttpActionResult SNWR036_GetVanDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_VAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("PINVOICE_NO", OracleDbType.Varchar2).Value = pData.PINVOICE_NO;
                //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                //cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                //cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                //cmd.Parameters.Add("P_COMPLETE_FLAG", OracleDbType.Varchar2).Value = pData.COMPLETE_FLAG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetVanDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetDataInvoice")]
        [Authorize]
        public IHttpActionResult SNWR036_GetDataInvoice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DATA_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pData.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pData.END_DATE;
                cmd.Parameters.Add("P_CUSTOMER_ZONE", OracleDbType.Varchar2).Value = pData.CUSTOMER_ZONE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetDataInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNWR036_GetInvoiceDetail([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_DETAIL_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR036_GET_DETAIL_INVOICE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_FindInstruction")]
        [Authorize]
        public IHttpActionResult SNWR036_FindInstruction([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_FIND_BY_INSTRUCTION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pData.INSTRUCTION_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_FindInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetCutOffBKK")]
        [Authorize]
        public IHttpActionResult SNWR036_GetCutOffBKK([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_CUTOFF_BBK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetCutOffBKK");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_GetCutOffUPC")]
        [Authorize]
        public IHttpActionResult SNWR036_GetCutOffUPC([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_GET_CUTOFF_UPC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_GetCutOffUPC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_FindPickinglistNo")]
        [Authorize]
        public IHttpActionResult SNWR036_FindPickinglistNo([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_FIND_BY_PICKING_LIST_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pData.PICKING_LIST_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_FindPickinglistNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR036_FindPartNo")]
        [Authorize]
        public IHttpActionResult SNWR036_FindPartNo([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR036_FIND_BY_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR036_FindPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion SNWR036
        #region SNWR037
        [HttpGet, Route("SNWR037_GetCaseType/{caseType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetCaseType(string caseType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseType = ismoUtil.escapeCharacter(caseType);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_CASE_TYPE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR037_GetCaseTypeInner/{caseType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetCaseTypeInner(string caseType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseType = ismoUtil.escapeCharacter(caseType);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_CASE_TYPE_INNER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR037_GetTypeList/{caseType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetTypeList(string caseType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseType = ismoUtil.escapeCharacter(caseType);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_TYPE_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = "OT";
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR037_GetInnerTypeList/{caseType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetInnerTypeList(string caseType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseType = ismoUtil.escapeCharacter(caseType);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_TYPE_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = "IN";
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR037_Search/{caseType}/{caseGroup}")]
        [Authorize]
        public IHttpActionResult SNWR037_Search(string caseType, string caseGroup)
        {
            caseType = caseType == null ? caseType : ismoUtil.escapeCharacter(caseType);
            caseGroup = caseGroup == null ? caseGroup : ismoUtil.escapeCharacter(caseGroup);
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
            cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup.ToUpper() == "NULL" ? (object)DBNull.Value : caseGroup;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpPost, Route("SNWR037_Insert")]
        [Authorize]
        public IHttpActionResult SNWR037_Insert([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR037_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = pData.CASE_TYPE;
                cmd.Parameters.Add("P_CAN_CONFIG", OracleDbType.Varchar2).Value = pData.CAN_CONFIG;
                cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = pData.CASE_WIDTH;
                cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = pData.CASE_DEPTH;
                cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = pData.CASE_HEIGHT;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = pData.CASE_NAME;
                cmd.Parameters.Add("P_ETG_CODE", OracleDbType.Varchar2).Value = pData.ETG_CODE;
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = pData.CASE_GROUP;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                cmd.Parameters.Add("P_CASE_IN_INVOICE", OracleDbType.Varchar2).Value = pData.CASE_IN_INVOICE;
                cmd.Parameters.Add("P_PACKAGE", OracleDbType.Varchar2).Value = pData.PACKAGE;
                cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = pData.REMARK;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { STATUS = R_STATUS, MESSAGE = R_MESSAGE });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR037_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR037_Update")]
        [Authorize]
        public IHttpActionResult SNWR037_Update([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR037_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pData.ID;
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = pData.CASE_TYPE;
                cmd.Parameters.Add("P_CAN_CONFIG", OracleDbType.Varchar2).Value = pData.CAN_CONFIG;
                cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = pData.CASE_WIDTH;
                cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = pData.CASE_DEPTH;
                cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = pData.CASE_HEIGHT;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = pData.CASE_NAME;
                cmd.Parameters.Add("P_ETG_CODE", OracleDbType.Varchar2).Value = pData.ETG_CODE;
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = pData.CASE_GROUP;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                cmd.Parameters.Add("P_CASE_IN_INVOICE", OracleDbType.Varchar2).Value = pData.CASE_IN_INVOICE;
                cmd.Parameters.Add("P_PACKAGE", OracleDbType.Varchar2).Value = pData.PACKAGE;
                cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = pData.REMARK;
                cmd.Parameters.Add("P_ACTIVE_STATUS", OracleDbType.Varchar2).Value = pData.STATUS;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { STATUS = R_STATUS, MESSAGE = R_MESSAGE });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR037_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR037_Delete")]
        [Authorize]
        public IHttpActionResult SNWR037_Delete([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR037_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = pData.CASE_GROUP;
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pData.ID;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR037_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNWR037_GetCaseName/{caseName}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetCaseName(string caseName, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseName = ismoUtil.escapeCharacter(caseName);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR037_GET_CASE_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = caseName.ToUpper() == "NULL" ? (object)DBNull.Value : caseName;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR037_GetCaseName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNWR037_GetCaseNameInner/{caseName}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetCaseNameInner(string caseName, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseName = ismoUtil.escapeCharacter(caseName);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR037_GET_CASE_NAME_INNER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = caseName.ToUpper() == "NULL" ? (object)DBNull.Value : caseName;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR037_GetCaseNameInner");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR037_Preview_Case")]
        public async Task<HttpResponseMessage> SNWR037_Preview_Case([FromBody] dynamic pParams)
        {
            try
            {
                string reportName = pParams.ReportName.ToString();
                string reportType = pParams.ReportType.ToString();
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CASE_NAME", Name = "P_CASE_NAME", Value = pParams.P_CASE_NAME });
                reportParams.Add(new ParameterValue { Label = "P_QTY", Name = "P_QTY", Value = pParams.P_QTY });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        [HttpPost, Route("SNWR037_GetPackageList/{package}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR037_GetPackageList(string package, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            package = ismoUtil.escapeCharacter(package);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR037_GET_PACKAGE_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PACKAGE_TYPE", OracleDbType.Varchar2).Value = package.ToUpper() == "NULL" ? (object)DBNull.Value : package;
            cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = param["P_CASE_GROUP"];
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion SNWR037
        #region SNWR038
        [HttpGet, Route("SNWR038_GetCaseType/{caseType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR038_GetCaseType(string caseType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            caseType = ismoUtil.escapeCharacter(caseType);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR038_GET_INNERCASE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = caseType.ToUpper() == "NULL" ? (object)DBNull.Value : caseType;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR038_GetCaseType");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR038_Print_Innercase_Number")]
        [Authorize]
        public IHttpActionResult SNWR038_Print_Innercase_Number([FromBody] dynamic parms)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                int printNo = parms["PRINT_NUMBER"];
                int iCount = 0;
                string NoFrom = String.Empty;
                string NoTo = String.Empty;
                for (int i = 0; i < printNo; i++)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_INNERCASE_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = "EXPORT";
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR038";
                    cmd.Parameters.Add("R_CASE_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    if (iCount == 1)
                    {
                        NoFrom = cmd.Parameters["R_CASE_NO"].Value.ToString();
                    }
                    NoTo = cmd.Parameters["R_CASE_NO"].Value.ToString();
                }
                return Json(new { Result = "Done", NoFrom = NoFrom, NoTo = NoTo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR038_Print_Innercase_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR038_Get_Innercase_Number")]
        [Authorize]
        public IHttpActionResult SNWR038_Get_Innercase_Number([FromBody] dynamic parms)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_INNERCASE_RUNNING_NUMBER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = parms["CASE_TYPE"];
                cmd.Parameters.Add("P_INNERCASE_NO", OracleDbType.Varchar2).Value = parms["INNERCASE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR038_Get_Innercase_Number");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR038_Preview_Casenumber")]
        public async Task<HttpResponseMessage> SNWR038_Preview_Casenumber([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR038_INNER_CASE";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_NO_FROM", Name = "P_NO_FROM", Value = pParams.P_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_NO_TO", Name = "P_NO_TO", Value = pParams.P_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        #region SNWR040
        [HttpGet, Route("SNWR040_GetOverViewData/{startDate}/{endDate}")]
        [Authorize]
        public IHttpActionResult SNWR040_GetOverViewData(string startDate, string endDate)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR040_GET_OVERVIEW_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = startDate == "NULL" ? (object)DBNull.Value : startDate;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = endDate == "NULL" ? (object)DBNull.Value : endDate;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR040_GetOverViewData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNWR040_GetTotalData/{startDate}/{endDate}")]
        [Authorize]
        public IHttpActionResult SNWR040_GetTotalData(string startDate, string endDate)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR040_GET_TOTAL_DATA_V1";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = startDate == "NULL" ? (object)DBNull.Value : startDate;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = endDate == "NULL" ? (object)DBNull.Value : endDate;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR040_GetTotalData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        #endregion
        #region SNWR041
        [HttpGet, Route("SNWR041_GetDashConfigData/{usercode}/{startDate}/{endDate}/{configType}")]
        [Authorize]
        public IHttpActionResult SNWR041_GetDashConfigData(string usercode, string startDate, string endDate, string configType)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_GET_WMS_DASH_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_CODE", OracleDbType.Varchar2).Value = usercode == "NULL" ? (object)DBNull.Value : usercode;
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = startDate == "NULL" ? (object)DBNull.Value : startDate;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = endDate == "NULL" ? (object)DBNull.Value : endDate;
                cmd.Parameters.Add("P_CONFIG_TYPE", OracleDbType.Varchar2).Value = configType.ToUpper() == "NULL" ? (object)DBNull.Value : configType;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_GetDashConfigData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNWR041_GetCalendar/{ProcessYear}/{ProcessMonth}")]
        [Authorize]
        public IHttpActionResult SNWR041_GetCalendar(string ProcessYear, string ProcessMonth)
        {
            try
            {
                DataSet ds = new DataSet();
                ProcessYear = ismoUtil.escapeCharacter(ProcessYear);
                ProcessMonth = ismoUtil.escapeCharacter(ProcessMonth);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_DAY_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = ProcessYear;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = ProcessMonth;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_GetCalendar");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNWR041_InsertCapacity")]
        [Authorize]
        public IHttpActionResult SNWR041_InsertCapacity([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_INSERT_CAPACITY_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_CODE", OracleDbType.Varchar2).Value = pParam["USER_CODE"];
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = pParam["START_DATE"];
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = pParam["END_DATE"];
                cmd.Parameters.Add("P_CONFIG_TYPE", OracleDbType.Varchar2).Value = pParam["CONFIG_TYPE"];
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE"];
                cmd.Parameters.Add("P_CONFIG_VALUE_CHECK", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE_CHECK"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string r_status = dt.Rows[0]["STATUS"].ToString();
                string r_message = dt.Rows[0]["MESSAGE"].ToString();
                return Json(new { status = r_status, message = r_message, set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveConfig");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR041_UpdateCapacity")]
        [Authorize]
        public IHttpActionResult SNWR041_UpdateCapacity([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_UPDATE_CAPACITY_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ROWID"];
                cmd.Parameters.Add("P_USER_CODE", OracleDbType.Varchar2).Value = pParam["USER_CODE"];
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = pParam["START_DATE"];
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = pParam["END_DATE"];
                cmd.Parameters.Add("P_CONFIG_TYPE", OracleDbType.Varchar2).Value = pParam["CONFIG_TYPE"];
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE"];
                cmd.Parameters.Add("P_CONFIG_VALUE_CHECK", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE_CHECK"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string r_status = dt.Rows[0]["STATUS"].ToString();
                string r_message = dt.Rows[0]["MESSAGE"].ToString();
                return Json(new { status = r_status, message = r_message, set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveConfig");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR041_InsertCapNormal")]
        [Authorize]
        public IHttpActionResult SNWR041_InsertCapNormal([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_INSERT_CAP_NORMAL_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = pParam["START_DATE"];
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = pParam["END_DATE"];
                cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = pParam["JOB_TYPE"];
                cmd.Parameters.Add("P_CONFIG_TYPE", OracleDbType.Varchar2).Value = pParam["CONFIG_TYPE"];
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE"];
                cmd.Parameters.Add("P_CONFIG_VALUE_CHECK", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE_CHECK"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string r_status = dt.Rows[0]["STATUS"].ToString();
                string r_message = dt.Rows[0]["MESSAGE"].ToString();
                return Json(new { status = r_status, message = r_message, set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveConfig");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR041_UpdateCapNormal")]
        [Authorize]
        public IHttpActionResult SNWR041_UpdateCapNormal([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_UPDATE_CAP_NORMAL_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ROWID"];
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Varchar2).Value = pParam["START_DATE"];
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Varchar2).Value = pParam["END_DATE"];
                cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = pParam["JOB_TYPE"];
                cmd.Parameters.Add("P_CONFIG_TYPE", OracleDbType.Varchar2).Value = pParam["CONFIG_TYPE"];
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE"];
                cmd.Parameters.Add("P_CONFIG_VALUE_CHECK", OracleDbType.Varchar2).Value = pParam["CONFIG_VALUE_CHECK"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string r_status = dt.Rows[0]["STATUS"].ToString();
                string r_message = dt.Rows[0]["MESSAGE"].ToString();
                return Json(new { status = r_status, message = r_message, set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveConfig");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR041_DeleteConfig")]
        [Authorize]
        public IHttpActionResult SNWR041_DeleteConfig([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_DELETE_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ROWID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string r_status = dt.Rows[0]["STATUS"].ToString();
                string r_message = dt.Rows[0]["MESSAGE"].ToString();
                return Json(new { status = r_status, message = r_message, set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveConfig");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        [HttpPost, Route("SNWR041_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR041_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var content_1 = reqUpload.Contents[1].ReadAsByteArrayAsync();
                var content_2 = reqUpload.Contents[2].ReadAsByteArrayAsync();
                var content_3 = reqUpload.Contents[3].ReadAsByteArrayAsync();
                string month_start = Encoding.UTF8.GetString(content_1.Result, 0, content_1.Result.Length);
                string month_end = Encoding.UTF8.GetString(content_2.Result, 0, content_2.Result.Length);
                string configtype = Encoding.UTF8.GetString(content_3.Result, 0, content_3.Result.Length);
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_WMS_DASH_CONFIG_TEMPLATE WHERE ROWNUM = 1";
                string column = @"USER_CODE,CONFIG_VALUE,CONFIG_VALUE_CHECK";
                string[] array = column.Split(',');
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("START_DATE", typeof(String));
                dc.DefaultValue = month_start;
                dt_file.Columns.Add(dc);
                dc = new DataColumn("END_DATE", typeof(String));
                dc.DefaultValue = month_end;
                dt_file.Columns.Add(dc);
                dc = new DataColumn("CONFIG_TYPE", typeof(String));
                dc.DefaultValue = configtype;
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();
                for (int i = 0; i < colCount; i++)
                {
                    dt.Columns[i].ColumnName = array[i].Trim();
                }
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        [HttpPost, Route("SNWR041_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNWR041_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR041_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR041_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }
        #endregion
        #region SNWR042
        [HttpPost, Route("SNWR042_Preview")]
        [Authorize]
        public async Task<IHttpActionResult> SNWR042_Preview(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string FullfileName = String.Concat(gFileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                string gFileNameExtend = String.Concat(FullfileName, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLocation = System.IO.Path.Combine(gRootPath, "tempFile/SNWR042");
                string gFileLocation = System.IO.Path.Combine(gUploadLocation, gFileNameExtend);
                DataService.SaveBytesToFileByLocation(buffer, gUploadLocation, gFileLocation);
                SystemApiController sys = new SystemApiController();
                ErrorList = SNWR042_InsertDataFromExcelFile(gFileLocation);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR042_GET_DATA_TEMP_UPLOAD";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR042_Preview");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
        }
        public List<string> SNWR042_InsertDataFromExcelFile(string fileLocation)
        {
            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            SNWR042_ClearTempData();
            foreach (DataRow row in dt.Rows)
            {
                string documentNo = null;
                string shipDate = null;
                documentNo = row["DOCUMENT_NO"].ToString().Trim();
                shipDate = row["SHIP_DATE"].ToString().Trim();
                if (documentNo != null && shipDate != null)
                {
                    var parsedDate = DateTime.Parse(shipDate);
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR042_INSERT_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = documentNo;
                    cmd.Parameters.Add("P_SHIP_DATE", OracleDbType.Date).Value = parsedDate;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
            }
            return ErrList;
        }
        public string SNWR042_ClearTempData()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR042_CLEAR_TEMP_UPLOAD";
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return "DONE";
        }
        [HttpGet, Route("SNWR042_GetTempData")]
        [Authorize]
        public IHttpActionResult SNWR042_GetTempData()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR042_GET_DATA_TEMP_UPLOAD";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR042_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR042_ConfirmSaveShipDate")]
        [Authorize]
        public IHttpActionResult SNWR042_ConfirmSaveShipDate()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR042_SAVE_SHIP_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNWR042";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR042_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR039
        [HttpPost, Route("SNWR039_CheckInnerPackBabyCard")]
        [Authorize]
        public IHttpActionResult SNWR039_CheckInnerPackBabyCard([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CHECK_INNERPACK_BABYCARD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = pParam.BABYCARD_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable dt = ds.Tables[0];
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_CheckInnerPackBabyCard");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNWR039_GetCaseTypeAll")]
        [Authorize]
        public IHttpActionResult SNWR039_GetCaseTypeAll()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_INNERCASE_TYPE_ALL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_GetCaseTypeAll");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR039_GetAvailableCase/{caseGroup}/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR039_GetAvailableCase(string caseGroup, string caseNo)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_AVAILABLE_INNERCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_GROUP", OracleDbType.Varchar2).Value = caseGroup;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_CASE_DATE", OracleDbType.Varchar2, 24).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                String caseDate = cmd.Parameters["R_CASE_DATE"].Value.ToString();
                return Json(new { Result = "Done", CaseNo = caseNo, CaseDate = caseDate });
            }
            catch (OracleException ex)
            {
                if (ex.Number == 20001)
                {
                    return Json(new { Result = "Invoice", Message = "Case already pre-invoice." });

                }
                else if (ex.Number == 20002)
                {
                    return Json(new { Result = "Exist", Message = "Case is exist, do you want to unpack." });

                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { Result = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER

                }
                else if (ex.Number == 20000)
                {
                    return Json(new { Result = "Notfound", Message = "Case is not found, Please generate case first." });
                }
                else
                {
                    throw (ex);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_GetAvailableCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR039_GetInnerCaseTypeExport/{type}")]
        [Authorize]
        public IHttpActionResult SNWR039_GetInnerCaseTypeExport(string type)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_INNERCASE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = type;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_GetInnerCaseTypeExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR039_GetData/{babyNo}")]
        [Authorize]
        public IHttpActionResult SNWR039_GetData(string babyNo)
        {
            DataSet ds = new DataSet();
            babyNo = ismoUtil.escapeCharacter(babyNo);
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR039_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = babyNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR039_IssueCase")]
        [Authorize]
        public IHttpActionResult SNWR039_IssueCase([FromBody] dynamic pDataList)
        {
            OracleCommand cmd;
            int deleteCount = 0;
            try
            {
                List<dynamic> packingList = new List<dynamic>(pDataList.packingList);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_DELETE_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//Date for delete current case no (protect old case no)
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                deleteCount++;

                int i = 0;
                foreach (var data in packingList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR039_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = data.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(data.CASE_DATE);//Date for new case (But not used ToDateTime Function)(in C# time system is not correct)
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_BABYCARD_NO", OracleDbType.Varchar2).Value = data.BABYCARD_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_INSERT_TYPE", OracleDbType.Varchar2).Value = "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlExecute_CMD(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_SAVE_PACKING_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_IssueCase");
                if (deleteCount > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR039_REVERT_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                    oOracledb.SqlExecute_CMD(cmd);
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        [HttpGet, Route("SNWR039_GetCaseInfo/{caseNo}")]
        [Authorize]
        public IHttpActionResult SNWR039_GetCaseInfo(string caseNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_GET_CASE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                DataTable instructionData = ds.Tables[0];
                return Ok(new { status = "Done", Data = DTFM.convertToList(instructionData) });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                else if (ex.Number == 20003)
                {
                    int startIndex = ex.Message.IndexOf("/") + 1;
                    int endIndex = ex.Message.IndexOf("/", startIndex);
                    string user = ex.Message.Substring(startIndex, endIndex - startIndex);
                    return Json(new { status = "OnUsing", Message = "Case is on another user, can not process this case.", User = user }); //ORA-20003: CASE_ON_ANOTHER_USER
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_GetCaseInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR039_ClearInnerCaseExport")]
        [Authorize]
        public IHttpActionResult SNWR039_ClearInnerCaseExport([FromBody] dynamic pDataList)
        {
            try
            {
                string status = string.Empty;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CLEAR_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.scannedCase.branchCode;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList.scannedCase.orderType;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.scannedCase.caseNo;
                cmd.Parameters.Add("P_CASE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDataList.scannedCase.caseDate);//for not delete old case no
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                status = cmd.Parameters["P_STATUS"].Value.ToString();
                if (status == "USED")
                {
                    return Ok(new { status = "Used" });
                }
                else
                {
                    return Ok(new { status = "Done" });
                }
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_ClearInnerCaseExport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR039_ClearInnerCaseExportTemp")]
        [Authorize]
        public IHttpActionResult SNWR039_ClearInnerCaseExportTemp([FromBody] dynamic pDataList)
        {
            try
            {
                string status = string.Empty;
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR039_CLEAR_CASE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pDataList.CASE_NO;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                string errorMessage = oOracledb.GetErrorMessage(ex);
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Error", Message = errorMessage });
                }
                throw ex;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR039_ClearInnerCaseExportTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR043
        [HttpGet, Route("SNWR043_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR043_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR043_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR043_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR043_Print_PartNo")]
        [Authorize]
        public IHttpActionResult SNWR043_Print_PartNo([FromBody] dynamic parms)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                int printNo = parms["PRINT_NUMBER"];
                int iCount = 0;
                string PartNo = parms["PART_NO"];
                string NoPrint = String.Empty;
                for (int i = 0; i < printNo; i++)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR043_PRINT_PARTNO";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo;
                    cmd.Parameters.Add("R_QRCODE_LABEL", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                NoPrint = cmd.Parameters["R_QRCODE_LABEL"].Value.ToString();
                return Json(new { Result = "Done", PartNo = PartNo, NoPrint = NoPrint });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR043_Print_PartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR043_GetPickinglistLast")]
        [Authorize]
        public IHttpActionResult SNWR043_GetPickinglistLast([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNWR043_GET_PICKINGLIST_LAST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR043_GetPickinglistLast");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR043_Print_PickingList_Part")]
        public async Task<HttpResponseMessage> SNWR043_Print_PickingList_Part([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR043_PRINT_PICKINGLIST_PARTNO";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_PICKING_LIST_NO", Name = "P_PICKING_LIST_NO", Value = pParams.P_PICKING_LIST_NO });
                reportParams.Add(new ParameterValue { Label = "P_PRINT_PICK_DATE", Name = "P_PRINT_PICK_DATE", Value = pParams.P_PRINT_PICK_DATE });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("SNWR043_Print_PickingList_Baby")]
        public async Task<HttpResponseMessage> SNWR043_Print_PickingList_Baby([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR043_PRINT_PICKINGLIST_BABYNO";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_PICKING_LIST_NO", Name = "P_PICKING_LIST_NO", Value = pParams.P_PICKING_LIST_NO });
                reportParams.Add(new ParameterValue { Label = "P_PRINT_PICK_DATE", Name = "P_PRINT_PICK_DATE", Value = pParams.P_PRINT_PICK_DATE });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("SNWR043_Print_QRCodePartNo")]
        public async Task<HttpResponseMessage> SNWR043_Print_QRCodePartNo([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR043_PRINT_PARTNO";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_PART_NO", Name = "P_PART_NO", Value = pParams.P_PART_NO });
                reportParams.Add(new ParameterValue { Label = "P_PRINT_NUMBER", Name = "P_PRINT_NUMBER", Value = pParams.P_PRINT_NUMBER });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion

        #region SNWR044

        [HttpPost, Route("SNWR044_ReturnVan")]
        [Authorize]
        public IHttpActionResult SNWR044_ReturnVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR044_RETURN_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR044";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_ReturnVan");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNWR044_GetVanForEdit")]
        [Authorize]
        public IHttpActionResult SNWR044_GetVanForEdit([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_GETCASE_BY_VANGROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = pData.VAN_GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_GetVanForEdit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNWR044_GetCaseByNewVan")]
        [Authorize]
        public IHttpActionResult SNWR044_GetCaseByNewVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_GETCASE_BY_NEWVANGROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                // cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                // cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckVanCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }


        [HttpPost, Route("SNWR044_SaveVan")]
        [Authorize]
        public IHttpActionResult SNWR044_SaveVan([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                int itemCount = 0;
                string vanGroup = string.Empty;//Guid.NewGuid().ToString();
                string vanNo = string.Empty;
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_GET_DOC_VAN";
                cmd.Parameters.Add("R_DOC", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                vanGroup = cmd.Parameters["R_DOC"].Value.ToString();
                foreach (var data in pData.vanList)
                {
                    vanNo = data.VAN_NO;
                    itemCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR044_UPDATE_VANNING";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = data.VAN_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.customer;
                    cmd.Parameters.Add("P_OLD_GROUP", OracleDbType.Varchar2).Value = data.VAN_GROUP;
                    cmd.Parameters.Add("P_GROUP_ID", OracleDbType.Varchar2).Value = vanGroup;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR035 WEB";
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                    oOracledb.SqlQueryReturnValue(cmd);
                    string A = cmd.Parameters["R_STATUS"].Value.ToString();
                }

                foreach (var data in pData.caseList)
                {
                    itemCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR044_SAVE_TEMP_VAN_CASE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_VAN_CODE", OracleDbType.Varchar2).Value = vanNo;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.customer;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.branch;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.order_type;
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = data.CASE_NAME;
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = data.CASE_TYPE;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = itemCount == 1 ? "Y" : "N";
                    oOracledb.SqlExecute_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_SAVE_VAN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = vanGroup;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNWR044 WEB";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { STATUS_MESSAGE = _STATUS, VAN_GROUP = vanGroup });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_SaveVan WEB");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNWR044_GetCaseDetail")]
        [Authorize]
        public IHttpActionResult SNWR044_GetCaseDetail([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNWR044_GET_CASE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pData.CASE_NO;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pData.PACK_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_GetCaseDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));

                //return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNWR044_N_GetVanNo/{VanNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetVanNo(string VanNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            VanNo = ismoUtil.escapeCharacter(VanNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_VAN_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = VanNo.ToUpper() == "NULL" ? (object)DBNull.Value : VanNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_GET_VAN_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR044_N_GetData")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                //cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pParam.VAN_NO;

                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.VAN_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.VAN_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.VAN_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.VAN_DATE_TO);

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                // cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_GET_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_N_GetDataVan")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetDataVan([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_VAN_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pParam.VAN_NO;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_GET_VAN_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_N_GetDataVanDetail")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetDataVanDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_VAN_DETAIL_DATA";
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pParam.VAN_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_GET_VAN_DETAIL_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR044_N_GetVanNolog/{VanNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetVanNolog(string VanNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            VanNo = ismoUtil.escapeCharacter(VanNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_VAN_NO_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = VanNo.ToUpper() == "NULL" ? (object)DBNull.Value : VanNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_GET_VAN_NO_LOG");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR044_N_GetCustomer/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetCustomer(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);
            customer = customer.Trim();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR044_N_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNWR044_N_Save")]
        [Authorize]
        public IHttpActionResult SNWR044_N_Save([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string vanNo = String.Empty;
            string companyCode = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            string userName = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            try
            {


                if (pData.HEADER.STATUS == "NEW")
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_VAN_RUNNING_NEXT_VAL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = companyCode;
                    //cmd.Parameters.Add("P_VAN_GROUP", OracleDbType.Varchar2).Value = "EXPORT";
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.HEADER.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.HEADER.AIR_SEA;
                    cmd.Parameters.Add("P_UPDATE_PGM", OracleDbType.Varchar2).Value = "SNWR044";
                    cmd.Parameters.Add("R_VAN_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    vanNo = cmd.Parameters["R_VAN_NO"].Value.ToString();
                }
                else
                {
                    vanNo = pData.HEADER.VAN_NO;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_DEL_VAN_DETAIL";
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = vanNo;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = companyCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                foreach (dynamic d in pData.DETAIL)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR044_N_INS_VAN_DETAIL";
                    cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = vanNo;
                    cmd.Parameters.Add("P_CASE_TYPE", OracleDbType.Varchar2).Value = d["CASE_TYPE"];
                    cmd.Parameters.Add("P_CASE_NAME", OracleDbType.Varchar2).Value = d["CASE_NAME"];
                    cmd.Parameters.Add("P_CASE_WIDTH", OracleDbType.Varchar2).Value = d["CASE_WIDTH"];
                    cmd.Parameters.Add("P_CASE_DEPTH", OracleDbType.Varchar2).Value = d["CASE_DEPTH"];
                    cmd.Parameters.Add("P_CASE_HEIGHT", OracleDbType.Varchar2).Value = d["CASE_HEIGHT"];
                    cmd.Parameters.Add("P_VOLUME", OracleDbType.Varchar2).Value = d["VOLUME"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = d["QTY"];
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = userName;
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = companyCode;
                    cmd.Parameters.Add("P_CASE_PACKAGE", OracleDbType.Varchar2).Value = d["CASE_PACKAGE"];
                    cmd.Parameters.Add("P_ROW_NO", OracleDbType.Varchar2).Value = 1;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_INS_TEMP";
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = vanNo;
                cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pData.HEADER.CUSTOMER_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.HEADER.CUSTOMER_CODE;
                cmd.Parameters.Add("P_AIR_SEA", OracleDbType.Varchar2).Value = pData.HEADER.AIR_SEA;
                cmd.Parameters.Add("P_DIMENSION_W", OracleDbType.Varchar2).Value = pData.HEADER.DIMENSION_W;
                cmd.Parameters.Add("P_DIMENSION_H", OracleDbType.Varchar2).Value = pData.HEADER.DIMENSION_H;
                cmd.Parameters.Add("P_DIMENSION_D", OracleDbType.Varchar2).Value = pData.HEADER.DIMENSION_D;
                cmd.Parameters.Add("P_BOX", OracleDbType.Varchar2).Value = 0;
                cmd.Parameters.Add("P_PALLET", OracleDbType.Varchar2).Value = 0;
                cmd.Parameters.Add("P_RACK", OracleDbType.Varchar2).Value = 0;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = userName;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = companyCode;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pData.HEADER.STATUS;
                cmd.Parameters.Add("P_SIMULATE_STATUS", OracleDbType.Varchar2).Value = pData.HEADER.SIMULATE_STATUS;
                cmd.Parameters.Add("P_QTY_CASE", OracleDbType.Varchar2).Value = pData.HEADER.QTY_CASE;
                cmd.Parameters.Add("P_VAN_NAME", OracleDbType.Varchar2).Value = pData.HEADER.VAN_NAME;
                cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = pData.HEADER.CONTAINER_NO;
                cmd.Parameters.Add("P_SEAL_NO", OracleDbType.Varchar2).Value = pData.HEADER.SEAL_NO;
                cmd.Parameters.Add("P_BOOKING_NO", OracleDbType.Varchar2).Value = pData.HEADER.BOOKING_NO;



                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_N_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNWR044_SaveEdit")]
        [Authorize]
        public IHttpActionResult SNWR044_N_SaveEdit([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string companyCode = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_SAVE_EDIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = companyCode;
                cmd.Parameters.Add("P_VAN_NO", OracleDbType.Varchar2).Value = pData.VAN_NO;
                cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = pData.CONTAINER_NO;
                cmd.Parameters.Add("P_SEAL_NO", OracleDbType.Varchar2).Value = pData.SEAL_NO;
                cmd.Parameters.Add("P_BOOKING_NO", OracleDbType.Varchar2).Value = pData.BOOKING_NO;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string _STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                return Json(new { Status = _STATUS});

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_SaveEdit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNWR044_N_GetCaseTypeMaster")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetCaseTypeMaster([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_CASE_TYPE_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_AIR_SEA", OracleDbType.Varchar2).Value = pData.AIR_SEA;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_N_GetCaseTypeMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_N_GetDataVanMaster")]
        [Authorize]
        public IHttpActionResult SNWR044_N_GetDataVanMaster([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_GET_VAN_MASTER";

                cmd.Parameters.Add("VAN_NAME", OracleDbType.Varchar2).Value = pData.VAN_NAME;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_N_GetDataVanMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_N_SaveVanMaster")]
        [Authorize]
        public IHttpActionResult SNWR044_N_SaveVanMaster([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_INS_VAN_MASTER";

                cmd.Parameters.Add("VAN_NAME", OracleDbType.Varchar2).Value = pData.VAN_NAME;
                cmd.Parameters.Add("VAN_WIDTH", OracleDbType.Varchar2).Value = pData.VAN_WIDTH;
                cmd.Parameters.Add("VAN_DEPTH", OracleDbType.Varchar2).Value = pData.VAN_DEPTH;
                cmd.Parameters.Add("VAN_HEIGHT", OracleDbType.Varchar2).Value = pData.VAN_HEIGHT;
                cmd.Parameters.Add("STATUS", OracleDbType.Varchar2).Value = pData.STATUS;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_N_SaveVanMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_N_EditVanMaster")]
        [Authorize]
        public IHttpActionResult SNWR044_N_EditVanMaster([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_N_UPDATE_VAN_MASTER";

                cmd.Parameters.Add("VAN_NAME", OracleDbType.Varchar2).Value = pData.VAN_NAME;
                cmd.Parameters.Add("VAN_WIDTH", OracleDbType.Varchar2).Value = pData.VAN_WIDTH;
                cmd.Parameters.Add("VAN_DEPTH", OracleDbType.Varchar2).Value = pData.VAN_DEPTH;
                cmd.Parameters.Add("VAN_HEIGHT", OracleDbType.Varchar2).Value = pData.VAN_HEIGHT;
                cmd.Parameters.Add("STATUS", OracleDbType.Varchar2).Value = pData.STATUS;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_N_EditVanMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_Preview_Vannumber")]
        public async Task<HttpResponseMessage> SNWR044_Preview_Vannumber([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR044_VAN";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_VAN_NO_FROM", Name = "P_VAN_NO_FROM", Value = pParams.P_VAN_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_VAN_NO_TO", Name = "P_VAN_NO_TO", Value = pParams.P_VAN_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }


        [HttpPost, Route("SNWR044_GetRemainCase")]
        [Authorize]
        public IHttpActionResult SNWR044_GetRemainCase([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR044_GET_REMAIN_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR044_GetRemainCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR044_VAN_DETAIL_REPORT")]
        public async Task<HttpResponseMessage> SNWR044_VAN_DETAIL_REPORT([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "EXCELOPENXML";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "SNWR044", "/");
                string reportName = "PMS_RPT_WMS_SNWR044_VAN_DETAIL";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.P_CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_VAN_NO", Name = "P_VAN_NO", Value = pParams.P_VAN_NO });
                reportParams.Add(new ParameterValue { Label = "P_VAN_GROUP", Name = "P_VAN_GROUP", Value = pParams.P_VAN_GROUP });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/vnd.ms-excel");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        #endregion


        #region SNWR045

        [HttpGet, Route("SNWR045_GetReasonCode/{reasoncode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR045_GetReasonCode(string reasoncode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            reasoncode = ismoUtil.escapeCharacter(reasoncode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNWR045_REASON_CODE";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = reasoncode.ToUpper() == "NULL" ? (object)DBNull.Value : reasoncode;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNWR045_GetIncompleteList")]
        [Authorize]
        public IHttpActionResult SNWR045_GetIncompleteList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_GetIncompleteList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR045_Submit")]
        [Authorize]
        public IHttpActionResult SNWR045_Submit([FromBody] dynamic pData)
        {
            OracleCommand cmd = new OracleCommand();
            List<dynamic> dataList = new List<dynamic>(pData.data);

            try
            {
                string submitId = DateTime.Now.ToString("yyyyMMddHHmmss");
                foreach (dynamic data in dataList)
                {
                    if (data.STATUS == "Send")
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNWR045_SUBMIT_DETAIL";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;
                        cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                        cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                        cmd.Parameters.Add("P_PARTIAL_NO", OracleDbType.Int32).Value = data.PARTIAL_NO;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = pData.reasonCode;
                        cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.remarks;
                        cmd.Parameters.Add("P_SUBMIT_ID", OracleDbType.Varchar2).Value = submitId;
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlExecute_CMD(cmd);
                    }

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_PROCESS_SUBMIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUBMIT_ID", OracleDbType.Varchar2).Value = submitId;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_Submit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR045_ReturnItems")]
        [Authorize]
        public IHttpActionResult SNWR045_ReturnItems([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNWR045_RETURN";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    cmd.Parameters.Add("P_PARTIAL_NO", OracleDbType.Int32).Value = data.PARTIAL_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);


                    //cmd = new OracleCommand();
                    //cmd.CommandText = "SP_NP_SNWR045_RETURN_CHECK";
                    //cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    //cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    //cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    //cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    //cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;


                    //cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    //ds = oOracledb.SqlQueryDS_CMD(cmd);
                    //DataTable dtProcessStatus = ds.Tables[0];
                    //string returnMessage = dtProcessStatus.Rows[0]["CHECK_INVOICE"].ToString();

                    //if (returnMessage == "N")
                    //{
                    //    cmd = new OracleCommand();
                    //    cmd.CommandText = "SP_NP_SNWR045_RETURN";
                    //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    //    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    //    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    //    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    //    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = data.ITEM_NO;
                    //    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    //    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    //    cmd.Parameters.Add("P_QTY", OracleDbType.Int32).Value = data.QTY;
                    //    cmd.Parameters.Add("P_PARTIAL_NO", OracleDbType.Int32).Value = data.PARTIAL_NO;
                    //    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    //    oOracledb.SqlQueryReturnValue(cmd);
                    //}
                    //else
                    //{
                    //    return Ok("Invoiced");
                    //}


                }

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_ReturnItems");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        #endregion

        #region SNWR045_M
        [HttpGet, Route("SNWR045_GetSubmitList")]
        [Authorize]
        public IHttpActionResult SNWR045_GetSubmitList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_GET_SUBMIT_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_GetIncompleteList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR045_ProcessApprove/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR045_ProcessApprove([FromBody] dynamic pData, string refDocNo)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_PROCESS_APPROVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_ProcessApprove");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNWR045_A
        [HttpGet, Route("SNWR045_GetApproveList")]
        [Authorize]
        public IHttpActionResult SNWR045_GetApproveList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_GET_APPROVE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_GetApproveList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNWR045_GetSubmitDetail/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR045_GetSubmitDetail(string refDocNo)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_GET_SUBMIT_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_GetSubmitDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNWR045_ProcessSuspend/{refDocNo}")]
        [Authorize]
        public IHttpActionResult SNWR045_ProcessSuspend([FromBody] dynamic pData, string refDocNo)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR045_PROCESS_SUSPEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SUSPEND_NO", OracleDbType.Varchar2).Value = refDocNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlExecute_CMD(cmd);

                return Ok("Done");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNWR045_ProcessSuspend");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion


        #region SNWR046
        [HttpPost, Route("SNWR046_GetBabyCardNo/{BabyCardNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNWR046_GetBabyCardNo(string BabyCardNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            BabyCardNo = ismoUtil.escapeCharacter(BabyCardNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNWR046_N_GET_BABY_CARD_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BABYCARD_GROUP", OracleDbType.Varchar2).Value = param["BABYCARD_GROUP"];
                cmd.Parameters.Add("P_BABY_CARD_NO", OracleDbType.Varchar2).Value = BabyCardNo.ToUpper() == "NULL" ? (object)DBNull.Value : BabyCardNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNWR046_N_GET_BABY_CARD_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNWR046_Preview_BabyCard")]
        public async Task<HttpResponseMessage> SNWR046_Preview_BabyCard([FromBody] dynamic pParams)
        {
            try
            {
                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();
                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDeliveryLabel", "/");
                string reportName = "PMS_RPT_WMS_SNWR046_BABYCARD";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = pParams.P_COMPANY_CODE });
                reportParams.Add(new ParameterValue { Label = "P_CUSTOMER_CODE", Name = "P_CUSTOMER_CODE", Value = pParams.P_CUSTOMER_CODE });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_GROUP", Name = "P_BABYCARD_GROUP", Value = pParams.P_BABYCARD_GROUP });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_NO_FROM", Name = "P_BABYCARD_NO_FROM", Value = pParams.P_BABYCARD_NO_FROM });
                reportParams.Add(new ParameterValue { Label = "P_BABYCARD_NO_TO", Name = "P_BABYCARD_NO_TO", Value = pParams.P_BABYCARD_NO_TO });
                byte[] bytes = await SYSCore.aControl.ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);
                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");
                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        [HttpPost, Route("GetDateByFormat")]
        [Authorize]
        public IHttpActionResult GetDateByFormat([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_DATE_BY_FORMAT";
                cmd.Parameters.Add("P_TYPE_DATE", OracleDbType.Varchar2).Value = pParam.TYPE_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDateByFormat");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
    }

}
